"use strict";

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var moment = require("moment");

var validate = module.exports = require("validate.js");

// Set default options
validate.options = {
  fullMessages: false,
  format: "jsonPath"
};

validate.async.options = {
  cleanAttributes: false,
  fullMessages: false,
  format: "jsonPath"
};

var ISO_8601_DATE = "YYYY-MM-DD";
var ISO_8601_DATETIME = moment.ISO_8601;

validate.validators.remote = require("./validators/remote");
validate.validators.time = require("./validators/time");
validate.validators.geometry = require("./validators/geometry");
validate.validators.monthDay = require("./validators/month-day");
validate.validators.monthDayRange = require("./validators/month-day-range");
validate.validators.crossCheck = require("./validators/cross-check");
validate.validators.compareDate = require("./validators/compare-date");
validate.validators.transectBiotope = require("./validators/transect-biotope");
validate.validators.compareNumerically = require("./validators/compare-numerically");
validate.validators.duplicateCheck = require("./validators/duplicate-check");
validate.validators.stringArray = require("./validators/string-array");
validate.validators.timeArray = require("./validators/time-array");
validate.validators.unitCount = require("./validators/unit-count");
validate.validators.timeDifference = require("./validators/time-difference");
validate.validators.valueInArray = require("./validators/value-in-array");
validate.validators.conditionalNumericality = require("./validators/conditional-numericality");
validate.validators.waterBirdShortHand = require("./validators/water-bird-short-hand");
validate.validators.waterBirdPairCount = require("./validators/water-bird-pair-count");

// This can be moved when no longer needed
validate.validators.dateSpan = function (value, options) {
  if (typeof options === "undefined") {
    options = {};
  }
  var message = options.message || "date begin (%{dateBegin}) must be before date end (%{dateEnd})";
  var format = options.dateOnly ? ISO_8601_DATE : ISO_8601_DATETIME;
  if (value) {
    var dateBegin = value.dateBegin,
        dateEnd = value.dateEnd;

    if (dateBegin && dateEnd && moment(dateBegin, format).isAfter(moment(dateEnd, format)) || dateEnd && !dateBegin) {
      var errorMessage = message || "";
      ["dateBegin", "dateEnd"].forEach(function (field) {
        while (value[field] && errorMessage.indexOf("%{" + field + "}") !== -1) {
          errorMessage = errorMessage.replace("%{" + field + "}", value[field]);
        }
      });
      return errorMessage;
    }
  }
};

validate.extend(validate.validators.datetime, {
  parse: function parse(value, options) {
    if (value === "now") {
      var lastMonth = moment().add(12, "hours");
      return +moment.utc(lastMonth.format("YYYY-MM-DD HH:mm:ss"));
    } else if (value === "lastMonth") {
      var _lastMonth = moment().subtract(1, "months");
      var _format = options.dateOnly ? "YYYY-MM-DD" : "YYYY-MM-DD HH:mm:ss";
      return +moment.utc(_lastMonth.format(_format), _format, true);
    } else if (value === "lastSixMonths") {
      var lastSixMonths = moment().subtract(6, "months");
      var _format2 = options.dateOnly ? "YYYY-MM-DD" : "YYYY-MM-DD HH:mm:ss";
      return +moment.utc(lastSixMonths.format(_format2), _format2, true);
    } else if (value === "lastYear") {
      var _lastMonth2 = moment().subtract(1, "years");
      var _format3 = options.dateOnly ? "YYYY-MM-DD" : "YYYY-MM-DD HH:mm:ss";
      return +moment.utc(_lastMonth2.format(_format3), _format3, true);
    } else if (value === "nowExact") {
      // need to fake that the current time is utc so that date validators would work
      return +moment().utc(moment().format("YYYY-MM-DD HH:mm:ss"));
    }
    var format = options.dateOnly ? ISO_8601_DATE : ISO_8601_DATETIME;
    return +moment.utc(value, format, true);
  },
  format: function format(value, options) {
    var format = options.dateOnly ? "YYYY-MM-DD" : "YYYY-MM-DD HH:mm:ss";
    return moment(value).format(format);
  }
});

var runValidations = validate.runValidations;
validate.runValidations = function (attributes, constraints, options, formData) {
  if ((typeof attributes === "undefined" ? "undefined" : (0, _typeof3.default)(attributes)) === "object" && !Array.isArray(attributes) && attributes !== null && attributes.skipped) {
    return [];
  }
  return runValidations(attributes, constraints, options, formData);
};