"use strict";

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var geojsonBounds = require("geojson-bounds");

var _require = require("laji-map/lib/utils"),
    convertLatLng = _require.convertLatLng;

function _findRoot(path, object, fullObject) {
  if (path[0] === "#") {
    path.shift();
    return fullObject;
  } else if (path[0] !== "..") {
    return object;
  }

  var childKeys = {
    gatheringEvent: {},
    gatherings: {
      units: {
        unitFact: {},
        identifications: {},
        measurement: {}
      },
      gatheringFact: {},
      measurement: {}
    }
  };

  // First try to get it the fast way
  var parents = _findParents(object, fullObject, [], childKeys);
  if (!parents) {
    // if parent where not found use the brute force way
    parents = _findParents(object, fullObject, []);
  }

  if (!parents) {
    // No parents where found so returning empty handed
    return {};
  }

  var root = {};
  while (path[0] === "..") {
    root = parents.pop();
    path.shift();
  }
  return root;
}

function _findParents(object, fullObject, parents, travelBy) {
  if (object === fullObject) {
    return parents;
  }
  if (typeof travelBy === "undefined") {
    travelBy = fullObject;
  }
  if ((typeof travelBy === "undefined" ? "undefined" : (0, _typeof3.default)(travelBy)) !== "object" || travelBy === null) {
    return false;
  }

  var result = false;
  var properties = (0, _keys2.default)(travelBy);
  var newParents = parents.slice();

  newParents.push(fullObject);
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)(properties), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var i = _step.value;

      if (fullObject[i]) {
        if (Array.isArray(fullObject[i])) {
          for (var idx in fullObject[i]) {
            if (!fullObject[i].hasOwnProperty(idx)) {
              continue;
            }
            if (object === fullObject[i]) {
              return newParents;
            }
            result = _findParents(object, fullObject[i][idx], newParents, Array.isArray(travelBy[i]) ? travelBy[i][idx] : travelBy[i]);
            if (result) {
              return result;
            }
          }
        } else if ((0, _typeof3.default)(fullObject[i]) === "object" && fullObject[i] !== null) {
          result = _findParents(object, fullObject[i], newParents, travelBy[i]);
          if (result) {
            return result;
          }
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return result;
}

module.exports = {
  extend: function extend(obj) {
    [].slice.call(arguments, 1).forEach(function (source) {
      for (var attr in source) {
        obj[attr] = source[attr];
      }
    });
    return obj;
  },
  errorMessage: function getMessage(errorKey, options, key, value) {
    var msg = "%{key} is invalid";
    if (options && options.message) {
      msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
    }
    return msg.replace("%{key}", key).replace("%{value}", value);
  },
  parseJSONPointer: function parseJSONPointer(path, object, fullObject) {
    var splitted = path.split("/").filter(function (s) {
      return s.length;
    });
    object = _findRoot(splitted, object, fullObject);
    return splitted.reduce(function (o, s, i) {
      [["~1", "/"], ["~0", "~"]].forEach(function (decoderPair) {
        var _decoderPair = (0, _slicedToArray3.default)(decoderPair, 2),
            escaped = _decoderPair[0],
            decoded = _decoderPair[1];

        while (s.indexOf(escaped) !== -1) {
          s = s.replace(escaped, decoded);
        }
      });
      return s in o || i === splitted.length - 1 ? o[s] : {};
    }, object || {});
  },
  isJSONPointer: function isJSONPointer(pointer) {
    return typeof pointer === "string" && (pointer[0] === "/" || pointer[0] === "#");
  },
  getCheckValues: function getCheckValues(check, attributes, formData, checkNull) {
    var _this = this;

    if (Array.isArray(check)) {
      return check.reduce(function (arr, curr) {
        arr.push.apply(arr, _this.getCheckValues(curr, attributes, formData, checkNull));
        return arr;
      }, []);
    }

    var splitted = check.split(/\*(.+)/);
    if (splitted.length === 1) {
      var value = this.parseJSONPointer(check, attributes, formData);
      if (value === undefined) {
        value = null;
      }
      return checkNull || value != null ? Array.isArray(value) ? value : [value] : [];
    } else {
      var array = this.parseJSONPointer(splitted[0], attributes, formData);
      if (!Array.isArray(array)) {
        return [];
      }
      return array.reduce(function (arr, curr) {
        arr.push.apply(arr, _this.getCheckValues(splitted[1], curr, formData, checkNull));
        return arr;
      }, []);
    }
  },
  shouldValidateWithCoordinates: function shouldValidateWithCoordinates(options, attributes, formData) {
    if (options.onlyWhenYkjNorthCoordinateSmallerThan || options.onlyWhenYkjNorthCoordinateBiggerThan) {
      var geometryValue = this.parseJSONPointer(options.geometryField, attributes, formData);
      if (!geometryValue) {
        return false;
      }

      var maxNorthCoordinate = geojsonBounds.yMax(geometryValue);

      if (options.onlyWhenYkjNorthCoordinateSmallerThan) {
        var threshold = convertLatLng([options.onlyWhenYkjNorthCoordinateSmallerThan * 10000, 3333333], "EPSG:2393", "WGS84")[0];

        if (maxNorthCoordinate >= threshold) {
          return false;
        }
      }
      if (options.onlyWhenYkjNorthCoordinateBiggerThan) {
        var _threshold = convertLatLng([options.onlyWhenYkjNorthCoordinateBiggerThan * 10000, 3333333], "EPSG:2393", "WGS84")[0];

        if (maxNorthCoordinate <= _threshold) {
          return false;
        }
      }
    }
    return true;
  }
};