"use strict";

var moment = require("moment");
var util = require("../util");

var TIME_FORMATS = [moment.ISO_8601, "YYYY-MM-DD"];

var defaultOptions = {
  isAfter: undefined,
  isBefore: undefined,
  hasTimeIfOtherHas: undefined,
  message: {
    tooEarly: "%{key} must be later than %{value}.",
    tooLate: "%{key} must be earlier than %{value}.",
    hasTimeIfOtherHas: "must have time if ${target} has."
  }
};

module.exports = function compareDate(value, options, key, attributes) {
  if (!value) {
    return;
  }
  options = util.extend({}, defaultOptions, options);
  var isAfter = attributes[options.isAfter] || "";
  var isBefore = attributes[options.isBefore] || "";
  var hasTimeIfOtherHas = attributes[options.hasTimeIfOtherHas] || "";
  if (isAfter && !moment(value, TIME_FORMATS).isSameOrAfter(moment(isAfter, TIME_FORMATS))) {
    return util.errorMessage("tooEarly", options, value, isAfter);
  }
  if (isBefore && !moment(value, TIME_FORMATS).isSameOrBefore(moment(isBefore, TIME_FORMATS))) {
    return util.errorMessage("tooLate", options, value, isBefore);
  }
  if (hasTimeIfOtherHas && value && !(value.indexOf("T") !== -1) && hasTimeIfOtherHas.indexOf("T") !== -1) {
    var message = util.errorMessage("hasTimeIfOtherHas", options, value, hasTimeIfOtherHas);
    message = message.replace("${target}", options.hasTimeIfOtherHas);
    return message;
  }
};