"use strict";

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require("../util");

var defaultOptions = {
  greaterThan: undefined,
  greaterThanOrEqualTo: undefined,
  lessThan: undefined,
  lessThanOrEqualTo: undefined,
  equalTo: undefined,
  onlyWhenAttrIn: undefined,
  message: {
    tooSmall: "%{key} is too small compared to %{field}",
    tooBig: "%{key} is too small compared to %{field}"
  }
};

function getMessage(errorKey, options, key, field) {
  var msg = "%{key} is invalid compared to %{field}";
  if (options && options.message) {
    msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
  }
  return msg.replace("%{key}", key).replace("%{field}", field);
}

function getNumericValue(attr, key) {
  if (Array.isArray(key)) {
    var sum = 0;
    for (var i = 0; i < key.length; i++) {
      var value = typeof key[i] !== "number" ? util.parseJSONPointer(key[i], attr) : key[i];
      if (value) {
        value = getNumericValueForOne(attr, key[i]);
        if (value === null) {
          return null;
        }
        sum += value;
      }
    }
    return sum;
  } else {
    return getNumericValueForOne(attr, key);
  }
}

function getNumericValueForOne(attr, key) {
  if (typeof key === "number") {
    return key;
  }
  var value = (typeof attr === "undefined" ? "undefined" : (0, _typeof3.default)(attr)) === "object" ? util.parseJSONPointer(key, attr) : attr;
  if (!value) {
    return null;
  }
  if (typeof value === "number") {
    return value;
  }
  var num = ("" + value).match(/[\d-\.]/g);
  if (!Array.isArray(num)) {
    return null;
  }
  return +num.join("");
}

module.exports = function compareNumerically(value, options, key, attributes) {
  // check that the context is an object
  if ((typeof attributes === "undefined" ? "undefined" : (0, _typeof3.default)(attributes)) !== "object") {
    return;
  }

  // Only do work when value is string or number
  if (typeof value !== "string" && typeof value !== "number") {
    return;
  }

  value = getNumericValue(value);
  options = util.extend({}, defaultOptions, options);

  if (value === null) {
    return;
  }

  if (options.onlyWhenAttrIn) {
    for (var _key in options.onlyWhenAttrIn) {
      var parsedAttr = util.parseJSONPointer(_key, attributes);
      if (!parsedAttr || options.onlyWhenAttrIn[_key].indexOf(parsedAttr) === -1) {
        return;
      }
    }
  }

  if (options.greaterThan) {
    var number = getNumericValue(attributes, options.greaterThan);
    if (number !== null && value <= number) {
      return getMessage("tooSmall", options, key, options.greaterThan);
    }
  }
  if (options.greaterThanOrEqualTo) {
    var _number = getNumericValue(attributes, options.greaterThanOrEqualTo);
    if (_number !== null && value < _number) {
      return getMessage("tooSmall", options, key, options.greaterThanOrEqualTo);
    }
  }

  if (options.lessThan) {
    var _number2 = getNumericValue(attributes, options.lessThan);
    if (_number2 !== null && value >= _number2) {
      return getMessage("tooBig", options, key, options.lessThan);
    }
  }
  if (options.lessThanOrEqualTo) {
    var _number3 = getNumericValue(attributes, options.lessThanOrEqualTo);
    if (_number3 !== null && value > _number3) {
      return getMessage("tooSmall", options, key, options.lessThanOrEqualTo);
    }
  }

  if (options.equalTo) {
    var _number4 = getNumericValue(attributes, options.equalTo);
    if (_number4 !== null && value !== _number4) {
      if (value < _number4) {
        return getMessage("tooSmall", options, key, options.equalTo);
      } else {
        return getMessage("tooBig", options, key, options.equalTo);
      }
    }
  }
};