"use strict";

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require("../util");
var validate = require("../main");

var defaultOptions = {
  check: "",
  onlyWhenCheckIn: false,
  onlyWhenCheckNotIn: false,
  onlyWhenNoCheck: false,
  mustBeEmpty: false,
  mustHaveValue: true,
  valueMustBeIn: false,
  requireCheck: false,
  geometryField: "geometry",
  onlyWhenYkjNorthCoordinateSmallerThan: false,
  onlyWhenYkjNorthCoordinateBiggerThan: false,
  message: {
    missing: "%{key} is required when %{target} is given",
    checkMissing: "%{target} is required",
    notEmpty: "%{key} has to be empty when %{target} is given",
    notValid: "%{key} is not accepted when %{target} is given%"
  },
  checkNull: false
};

function getMessage(errorKey, options, key, target) {
  var msg = "%{key} is required when %{target} is given";
  if (options && options.message) {
    msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
  }
  return msg.replace("%{key}", key).replace("%{target}", target);
}

module.exports = function crossCheck(value, options, key, attributes, globalOptions, formData) {
  options = util.extend({}, defaultOptions, options);

  // check that the value is an object
  if ((typeof attributes === "undefined" ? "undefined" : (0, _typeof3.default)(attributes)) !== "object") {
    return;
  }

  if (!util.shouldValidateWithCoordinates(options, attributes, formData)) {
    return;
  }

  // pick the check value
  var checkValues = util.getCheckValues(options.check, attributes, formData, options.checkNull);

  // validate check value
  if (checkValues.length < 1 && !options.onlyWhenNoCheck && !options.onlyWhenCheckNotIn && !options.onlyWhenCheckEquals || checkValues.length >= 1 && options.onlyWhenNoCheck) {
    if (options.requireCheck) {
      return getMessage("checkMissing", options, key, options.check);
    }
    return;
  }

  // only if check has acceptable value
  if (options.onlyWhenCheckIn && options.onlyWhenCheckIn.every(function (v) {
    return checkValues.indexOf(v) === -1;
  })) {
    return;
  }
  if (options.onlyWhenCheckNotIn && options.onlyWhenCheckNotIn.some(function (v) {
    return checkValues.indexOf(v) !== -1;
  })) {
    return;
  }
  if (options.onlyWhenCheckEquals) {
    var checkAgainstValues = util.getCheckValues(options.onlyWhenCheckEquals, attributes, formData, options.checkNull);
    if (checkValues.some(function (v) {
      return checkAgainstValues.indexOf(v) === -1;
    })) {
      return;
    }
  }

  if (options.validators) {
    var innerValidation = validate.processValidationResults(validate.runValidations(attributes, (0, _defineProperty3.default)({}, key, options.validators), globalOptions || {}, formData), globalOptions || {});
    if (innerValidation && innerValidation[key] && innerValidation[key].length) {
      return Array.isArray(innerValidation[key]) && innerValidation[key].length === 1 ? innerValidation[key][0] : innerValidation[key];
    }
    return;
  }

  // Check that the given
  var errorKey = "missing";
  if ((typeof value === "string" || Array.isArray(value)) && value.length > 0 || typeof value === "number" && !isNaN(value) || typeof value === "boolean") {
    if (options.mustBeEmpty) {
      errorKey = "notEmpty";
    } else if (options.valueMustBeIn && options.valueMustBeIn.indexOf(value) === -1) {
      errorKey = "notValid";
    } else {
      return;
    }
  } else if (options.mustBeEmpty || !options.mustHaveValue) {
    return;
  }

  return getMessage(errorKey, options, key, options.check);
};