"use strict";

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require("../util");
var geojsonArea = require("@mapbox/geojson-area");
var geojsonBounds = require("geojson-bounds");

var _require = require("laji-map/lib/utils"),
    latLngTuplesDistance = _require.latLngTuplesDistance,
    convertLatLng = _require.convertLatLng,
    convertGeoJSON = _require.convertGeoJSON;

var defaultOptions = {
  requireShape: false,
  includeGatheringUnits: false,
  allowedTypes: ["Point", "MultiPoint", "LineString", "MultiLineString", "Polygon", "MultiPolygon", "GeometryCollection"],
  allowedKeys: ["type", "coordinates", "coordinateVerbatim", "geometries"],
  overlapWith: false,
  minDistanceWith: false,
  minDistance: 0,
  message: {
    notGeometry: "Value in %{key} is not correct geometry",
    missingType: "Value in %{key} is missing required type property",
    invalidRadius: "Radius should be number in %{key}",
    missingGeometries: "Geometries are required in %{key}",
    invalidCoordinates: "Invalid coordinates in %{key}",
    invalidGeometries: "Geometries should not be given in %{key} when type is not GeometryCollection",
    noOverlap: "Area %{key} must be at least a little within the master area specified",
    tooClose: "Area is too close to the specified area",
    minLength: "Line length must be over %{key}",
    areaMinHectares: "Area mush be over %{key}",
    ykjOverlap: "Geometries must be inside a single YKJ grid"
  }
};

function checkCoordinates(coordinates, type) {
  switch (type) {
    case "Point":
      return coordinates[0] && coordinates[1] && typeof coordinates[0] === "number" && typeof coordinates[1] === "number" && (!coordinates[2] || typeof coordinates[1] === "number") && coordinates[0] >= -180 && coordinates[0] <= 180 && coordinates[1] >= -90 && coordinates[1] <= 90 && coordinates.length <= 3;
    case "LineString":
      return coordinates[0] && coordinates[1] && checkCoordinates(coordinates[0], "Point") && checkCoordinates(coordinates[1], "Point");
    case "Polygon":
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(coordinates), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var coordinateSet = _step.value;
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = (0, _getIterator3.default)(coordinateSet), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var coordinate = _step2.value;

              if (!checkCoordinates(coordinate, "Point")) {
                return false;
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return true;
    case "MultiPoint":
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = (0, _getIterator3.default)(coordinates), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var _coordinate = _step3.value;

          if (!checkCoordinates(_coordinate, "Point")) {
            return false;
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      return true;
    case "MultiLineString":
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = (0, _getIterator3.default)(coordinates), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var _coordinate2 = _step4.value;

          if (!checkCoordinates(_coordinate2, "LineString")) {
            return false;
          }
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      return true;
    default:
      return false;
  }
}

function getMessage(errorKey, options, key) {
  var msg = "%{key} is invalid geometry";
  if (options && options.message) {
    msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
  }
  while (msg.indexOf("%{key}") !== -1) {
    msg = msg.replace("%{key}", key);
  }
  return msg;
}

function geomToPolygon(geom, value) {
  return geom.type === "Point" && "radius" in value ? circleToPolygon(geom.coordinates, value.radius, 8) : geom;
}

function validateBoundingBox(value, options, key) {
  var boundingBox = geojsonBounds.envelope(value).geometry;
  var squareMeters = geojsonArea.geometry(boundingBox);
  var hectares = squareMeters / 10000;
  if (hectares > options.boundingBoxMaxHectares) {
    var msg = getMessage("invalidBoundingBoxHectares", options, key);

    if (msg) {
      while (msg.indexOf("%{max}") !== -1) {
        msg = msg.replace("%{max}", options.boundingBoxMaxHectares);
      }
      return msg;
    }
  }
}

function validateOverlap(value, options, key) {
  var boundingBox = geojsonBounds.envelope(value);
  var targetArea = options.overlapWith;
  if (!boundingBox || !boundingBox.geometry || !boundingBox.geometry.coordinates || !boundingBox.geometry.coordinates[0] || !boundingBox.geometry.coordinates[0][0] || boundingBox.geometry.coordinates[0][0].some(function (c) {
    return isNaN(c);
  })) {
    return;
  }
  if (geojsonBounds.xMin(boundingBox) <= geojsonBounds.xMax(targetArea) && geojsonBounds.xMax(boundingBox) >= geojsonBounds.xMin(targetArea) && geojsonBounds.yMin(boundingBox) <= geojsonBounds.yMax(targetArea) && geojsonBounds.yMax(boundingBox) >= geojsonBounds.yMin(targetArea)) {
    return;
  }
  return getMessage("noOverlap", options, key);
}

function validateMinDistance(value, options, key) {
  var centroid1 = getCentroid(value);
  var centroid2 = getCentroid(normalizeGeoJson(options.minDistanceWith));

  var distance = latLngTuplesDistance([centroid1[1], centroid1[0]], [centroid2[1], centroid2[0]]);
  if (distance < options.minDistance) {
    return getMessage("tooClose", options, key);
  }
}

// TODO Validates only LineString type. Doesn't take into account MultiLineString or GeometryCollection
// containing lines, or unitGathering geometries.
function validateMinLength(value, options, key) {
  if (value.type === "LineString") {
    var dist = latLngTuplesDistance.apply(undefined, (0, _toConsumableArray3.default)(value.coordinates.map(function (_ref) {
      var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
          lng = _ref2[0],
          lat = _ref2[1];

      return [lat, lng];
    })));
    if (dist < options.minLineLength) {
      var msg = getMessage("minLineLength", options, key);
      while (msg.indexOf("%{minLength}") !== -1) {
        msg = msg.replace("%{minLength}", options.minLineLength);
      }
      while (msg.indexOf("%{length}") !== -1) {
        msg = msg.replace("%{length}", dist.toFixed(2));
      }
      return msg;
    }
  }
}

function validateArea(value, options, key) {
  var hectares = areaInHectares(value);
  if (hectares < options.areaMinHectares) {
    var msg = getMessage("areaMinHectares", options, key);

    if (msg) {
      while (msg.indexOf("%{min}") !== -1) {
        msg = msg.replace("%{min}", options.areaMinHectares);
      }
      return msg;
    }
  }
}

function _geometry(value, options, key, attributes) {

  // allow empty if no shape is required
  if (!options.requireShape && typeof value === "undefined") {
    return;
  }

  // check that the value is an object
  if (value === null || (typeof value === "undefined" ? "undefined" : (0, _typeof3.default)(value)) !== "object" || Array.isArray(value)) {
    return getMessage("notGeometry", options, key);
  }

  var objectKeys = (0, _keys2.default)(value);
  if (objectKeys.length === 0 && !options.requireShape) {
    return;
  }

  // check that type exits and that it"s a string
  if (typeof value.type !== "string" || options.allowedTypes.indexOf(value.type) === -1) {
    return getMessage("missingType", options, key);
  }

  // check that type exits and that it"s a string
  // check that object has only allowed keys
  var allowed = options.allowedKeys.slice(0);
  if (value.type === "Point") {
    allowed.push("radius");
    if (value.radius && typeof value.radius !== "number") {
      return getMessage("invalidRadius", options, key);
    }
  }
  if (objectKeys.filter(function (key) {
    return allowed.indexOf(key) === -1;
  }).length > 0) {
    return getMessage("invalidProperty", options, key);
  }

  // validate geometry collections little differently
  if (value.type === "GeometryCollection") {
    if (!value.geometries || options.requireShape && value.geometries.length === 0) {
      return getMessage("missingGeometries", options, key);
    }
    var _iteratorNormalCompletion5 = true;
    var _didIteratorError5 = false;
    var _iteratorError5 = undefined;

    try {
      for (var _iterator5 = (0, _getIterator3.default)(value.geometries), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
        var geom = _step5.value;

        var _result = _geometry(geom, options, key, attributes);
        if (_result) {
          return _result;
        }
      }
    } catch (err) {
      _didIteratorError5 = true;
      _iteratorError5 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion5 && _iterator5.return) {
          _iterator5.return();
        }
      } finally {
        if (_didIteratorError5) {
          throw _iteratorError5;
        }
      }
    }

    if (options.hasOwnProperty("boundingBoxMaxHectares")) {
      var geometriesWithPolygonCircles = value.geometries.map(function (geom) {
        return geomToPolygon(geom, value);
      });

      if (options.includeGatheringUnits && attributes.units) {
        attributes.units.map(function (unit) {
          if (unit.unitGathering && unit.unitGathering.geometry) {
            geometriesWithPolygonCircles.push(geomToPolygon(unit.unitGathering.geometry, value));
          }
        });
      }

      var result = validateBoundingBox({ type: "GeometryCollection", geometries: geometriesWithPolygonCircles }, options, key);
      if (result) {
        return result;
      }
    }
    return;
  }

  // validate coordinates on simple types
  if (!value.coordinates || !checkCoordinates(value.coordinates, value.type)) {
    return getMessage("invalidCoordinates", options, key);
  }

  // check that no geometries property when not of type GeometryCollection
  if (value.geometries) {
    return getMessage("invalidGeometries", options, key);
  }
}

var getGeometriesFlat = function getGeometriesFlat(g) {
  switch (g.type) {
    case "GeometryCollection":
      return g.geometries.reduce(function (gs, g) {
        return [].concat((0, _toConsumableArray3.default)(gs), (0, _toConsumableArray3.default)(getGeometriesFlat(g)));
      }, []);
    default:
      return [g];
  }
};

var getCoordinatesFlat = function getCoordinatesFlat(g) {
  // Lat lng pair
  if (Array.isArray(g) && typeof g[0] === "number") {
    return [g];
  }
  // Array of coordinates
  if (Array.isArray(g)) {
    return g.reduce(function (cs, c) {
      return [].concat((0, _toConsumableArray3.default)(cs), (0, _toConsumableArray3.default)(getCoordinatesFlat(c)));
    }, []);
  }
  switch (g.type) {
    case "FeatureCollection":
      return g.features.reduce(function (cs, g) {
        return [].concat((0, _toConsumableArray3.default)(cs), (0, _toConsumableArray3.default)(getCoordinatesFlat(g.feature)));
      }, []);
    case "Feature":
      return getCoordinatesFlat(g.geometry);
    case "GeometryCollection":
      return g.geometries.reduce(function (cs, g) {
        return [].concat((0, _toConsumableArray3.default)(cs), (0, _toConsumableArray3.default)(getCoordinatesFlat(g)));
      }, []);
    case "MultiPolygon":
    case "Polygon":
    case "MultiLineString":
    case "LineString":
    case "MultiPoint":
    case "Point":
      return getCoordinatesFlat(g.coordinates);
  }
};

var getGridVerbatim = function getGridVerbatim(g) {
  if (g.coordinateVerbatim && g.coordinateVerbatim.match(/^\d{3}:\d{3}$/)) {
    return [g.coordinateVerbatim, g.coordinateVerbatim].join("-");
  }
  var getGridFromYKJ = function getGridFromYKJ(numb) {
    return ("" + Math.floor(numb / 10000) * 10000).substr(0, 3);
  };

  var coordinates = getCoordinatesFlat(convertGeoJSON(g, "WGS84", "EPSG:2393"));

  var west = void 0,
      south = void 0,
      east = void 0,
      north = void 0;

  coordinates.forEach(function (c) {
    var _c = (0, _slicedToArray3.default)(c, 2),
        lng = _c[0],
        lat = _c[1];

    if (west === undefined || lng < west) {
      west = lng;
    }
    if (east === undefined || lng > east) {
      east = lng;
    }
    if (south === undefined || lat < south) {
      south = lat;
    }
    if (north === undefined || lat > north) {
      north = lat;
    }
  });

  return [[south, west], [north, east]].map(function (i) {
    return i.map(getGridFromYKJ).join(":");
  }).join("-");
};

function validateYKJOverlap(value) {
  var geometries = getGeometriesFlat(value);
  var prevGridVerbatim = void 0;
  for (var i in geometries) {
    var g = geometries[i];
    var gridVerbatim = getGridVerbatim(g);
    var gridExtents = gridVerbatim.split("-");
    if (prevGridVerbatim && prevGridVerbatim !== gridVerbatim || gridExtents[0] !== gridExtents[1]) {
      return false;
    }
    prevGridVerbatim = gridVerbatim;
  }
  return true;
}

function normalizeGeoJson(value) {
  if (Array.isArray(value)) {
    return value.map(function (item) {
      return normalizeGeoJson(item);
    });
  }
  if ((typeof value === "undefined" ? "undefined" : (0, _typeof3.default)(value)) !== "object" || value === null) {
    return value;
  }
  if (value.type) {
    if (value.type === "GeometryCollection") {
      value.geometries = normalizeGeoJson(value.geometries);
      return value;
    } else if (value.type === "Point" && typeof value.radius === "number") {
      return circleToPolygon(value.coordinates, value.radius, 8);
    }
  }
  return value;
}

module.exports = function geometry(value, options, key, attributes) {
  options = util.extend({}, defaultOptions, options);
  var normalizedGeoJson = (typeof value === "undefined" ? "undefined" : (0, _typeof3.default)(value)) === "object" ? normalizeGeoJson(JSON.parse((0, _stringify2.default)(value))) : value;
  var result = _geometry(normalizedGeoJson, options, key, attributes);
  if (result) {
    return result;
  }
  if (options.hasOwnProperty("boundingBoxMaxHectares")) {
    var _result2 = validateBoundingBox(normalizedGeoJson, options, key);
    if (_result2) {
      return _result2;
    }
  }

  if (options.overlapWith) {
    var _result3 = validateOverlap(normalizedGeoJson, options, key);
    if (_result3) {
      return _result3;
    }
  }

  if (options.minDistanceWith) {
    var _result4 = validateMinDistance(normalizedGeoJson, options, key);
    if (_result4) {
      return _result4;
    }
  }

  if (options.hasOwnProperty("minLineLength")) {
    var _result5 = validateMinLength(normalizedGeoJson, options, key);
    if (_result5) {
      return _result5;
    }
  }

  if (options.hasOwnProperty("areaMinHectares")) {
    var _result6 = validateArea(normalizedGeoJson, options, key);
    if (_result6) {
      return _result6;
    }
  }

  if (options.ykjOverlap) {
    var valid = validateYKJOverlap(value);
    if (!valid) {
      return getMessage("ykjOverlap", options, key);
    }
  }
};

/** Taken from https://github.com/arg20/circle-to-radius
 *  (Copied here because the library didn't act nice with exporting)
**/
function circleToPolygon(center, radius, numberOfSegments) {
  function toRadians(angleInDegrees) {
    return angleInDegrees * Math.PI / 180;
  }

  function toDegrees(angleInRadians) {
    return angleInRadians * 180 / Math.PI;
  }

  function offset(c1, distance, bearing) {
    var lat1 = toRadians(c1[1]);
    var lon1 = toRadians(c1[0]);
    var dByR = distance / 6378137; // distance divided by 6378137 (radius of the earth) wgs84
    var lat = Math.asin(Math.sin(lat1) * Math.cos(dByR) + Math.cos(lat1) * Math.sin(dByR) * Math.cos(bearing));
    var lon = lon1 + Math.atan2(Math.sin(bearing) * Math.sin(dByR) * Math.cos(lat1), Math.cos(dByR) - Math.sin(lat1) * Math.sin(lat));
    return [toDegrees(lon), toDegrees(lat)];
  }

  var n = numberOfSegments ? numberOfSegments : 32;
  var flatCoordinates = [];
  var coordinates = [];
  for (var i = 0; i < n; ++i) {
    flatCoordinates.push.apply(flatCoordinates, offset(center, radius, 2 * Math.PI * i / n));
  }
  flatCoordinates.push(flatCoordinates[0], flatCoordinates[1]);

  for (var _i = 0, j = 0; j < flatCoordinates.length; j += 2) {
    coordinates[_i++] = flatCoordinates.slice(j, j + 2);
  }

  return {
    type: "Polygon",
    coordinates: [coordinates]
  };
}

function getCentroid(value) {
  var centroid = geojsonBounds.centroid(value);

  // centroid returns [NaN, NaN] for points
  if (isNaN(centroid[0])) {
    return [geojsonBounds.xMin(value), geojsonBounds.yMin(value)];
  }

  return centroid;
}

function areaInHectares(value) {
  if (!value) {
    return;
  }
  var geometries = value.geometries ? value.geometries : [value];
  var sumArea = geometries.filter(function (_ref3) {
    var type = _ref3.type,
        radius = _ref3.radius;
    return type === "Polygon" || type === "Point" && radius;
  }).reduce(function (area, geometry) {
    return area + geojsonArea.geometry(geometry);
  }, 0);
  return sumArea !== 0 ? sumArea / 10000 : undefined;
}