"use strict";

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require("../util");

var defaultOptions = {
  validator: undefined,
  path: undefined,
  message: {
    invalidValue: "invalid value given.",
    genericError: "something bad happened..."
  }
};

function getMessage(errorKey, options, key, target) {
  var msg = "%{key} is required when %{target} is given";
  if (options && options.message) {
    msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
  }
  return msg.replace("%{key}", key).replace("%{target}", target);
}

module.exports = function remote(value, options, key, attributes, globalOptions, formData) {
  var _this = this;

  if (!value) {
    return;
  }
  var hasUserMessage = options && options.message;
  options = util.extend({}, defaultOptions, options);
  var path = options.path || (key.startsWith(".") ? key : "." + key);
  return new _promise2.default(function (resolve) {
    _this.fetch("/documents/validate", {
      validator: options.validator,
      field: path,
      informalTaxonGroup: options.informalTaxonGroup,
      validationErrorFormat: "jsonPath"
    }, {
      method: "POST",
      headers: {
        "accept": "application/json",
        "content-type": "application/json"
      },
      body: (0, _stringify2.default)(formData)
    }).catch(function (err) {
      if (typeof err._body === "string") {
        return {
          status: 422,
          json: function json() {
            return JSON.parse(err._body);
          }
        };
      }
      return { status: 0 };
    }).then(function (response) {
      if (!response.status) {
        return response;
      }
      if (response.status >= 200 && response.status < 300) {
        return resolve();
      }
      if (response.status >= 400 && response.status < 500) {
        return response.json();
      }
      return resolve(getMessage("genericError", options, key));
    }).then(function (body) {
      if (body && body.error && body.error.details && (0, _typeof3.default)(body.error.details) === "object") {
        var _key = (0, _keys2.default)(body.error.details)[0];
        if (hasUserMessage || !_key) {
          return resolve(getMessage("invalidValue", options, _key));
        }
        if (body.error.details[path]) {
          return resolve(body.error.details[path].join(", "));
        }
        if (body.error.details[_key]) {
          return resolve(body.error.details[_key].join(", "));
        }
        return resolve(getMessage("invalidValue", options, _key));
      }
      return resolve();
    });
  });
};