"use strict";

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require("../util");
var moment = require("moment");

var defaultOptions = {
  field: "timeStart",
  order: false,
  count: undefined,
  upperRange: undefined,
  lowerRange: undefined,
  message: {
    order: "Times must be chronologically in order",
    count: "More than %{count} times must be given",
    range: "Times must be between %{lowerRange}-%{upperRange}"
  }
};

function getMessage(errorKey, options, key) {
  var msg = "%{key} is required when %{target} is given";
  if (options && options.message) {
    msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
  }
  return msg.replace("%{key}", key).replace("%{count}", options.count).replace("%{upperRange}", options.upperRange).replace("%{lowerRange}", options.lowerRange);
}

module.exports = function timeArray(value, options, key, attributes) {
  options = util.extend({}, defaultOptions, options);
  var _options = options,
      field = _options.field,
      fieldContainer = _options.fieldContainer,
      order = _options.order,
      count = _options.count;
  var _options2 = options,
      upperRange = _options2.upperRange,
      lowerRange = _options2.lowerRange;


  var origValue = value;
  value = !fieldContainer ? value : util.isJSONPointer(fieldContainer) ? util.parseJSONPointer(fieldContainer, value, attributes) : value[fieldContainer];

  // cannot do anything with value that is not an array
  if (!Array.isArray(value)) {
    return;
  }

  if (order) {
    var prevTime = undefined;
    if (!value.every(function () {
      var item = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var time = item[field];
      var timeMoment = moment(time, "HH:mm");
      var prevTimeMoment = moment(prevTime, "HH:mm");
      var isOk = !time || !prevTime || timeMoment.isAfter(prevTimeMoment) || timeMoment.isSame(prevTimeMoment);
      if (time) {
        prevTime = time;
      }
      return isOk;
    })) {
      return getMessage("order", options, key);
    }
  }

  if (count !== undefined) {
    if (value.filter(function () {
      var item = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      return item[field];
    }).length < count) {
      return getMessage("count", options, key);
    }
  }

  if (upperRange || lowerRange) {
    if (util.isJSONPointer(upperRange)) {
      upperRange = util.parseJSONPointer(upperRange, origValue, attributes);
    }
    if (util.isJSONPointer(lowerRange)) {
      lowerRange = util.parseJSONPointer(lowerRange, origValue, attributes);
    }
    var upperMoment = upperRange && moment(upperRange, "HH:mm");
    var lowerMoment = lowerRange && moment(lowerRange, "HH:mm");
    if (value.some(function () {
      var item = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var time = moment(item[field], "HH:mm");
      return upperMoment && time.isAfter(upperMoment) || lowerMoment && time.isBefore(lowerMoment);
    })) {
      return getMessage("range", (0, _extends3.default)({}, options, { upperRange: upperRange, lowerRange: lowerRange }), key);
    }
  }
};