"use strict";

var moment = require("moment");
var util = require("../util");

var defaultOptions = {
  minDifference: undefined,
  maxDifference: undefined,
  compareTo: undefined,
  message: {
    tooSmall: "time difference is less than %{limit} minutes",
    tooBig: "time difference is more than %{limit} minutes"
  }
};

function getMessage(errorKey, options, key, limit) {
  var msg = "%{key} is invalid";
  if (options && options.message) {
    msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
  }
  return msg.replace("%{key}", key).replace("%{limit}", limit);
}

module.exports = function timeDifference(value, options, key, attributes, globalOptions, formData) {
  options = util.extend({}, defaultOptions, options);

  // cannot do anything with value that is not string
  if (typeof value !== "string" || !value) {
    return;
  }
  var value2 = util.parseJSONPointer(options.compareTo, attributes, formData);
  if (typeof value2 !== "string" || !value2) {
    return;
  }

  var parts = value.split(":");
  var parts2 = value2.split(":");

  var now = new Date();
  var time = moment(now);
  time.set({ hour: parseInt(parts[0], 10), minute: parseInt(parts[1], 10) });
  var time2 = moment(now);
  time2.set({ hour: parseInt(parts2[0], 10), minute: parseInt(parts2[1], 10) });

  var duration = moment.duration(time.diff(time2));
  var difference = duration.asMinutes();

  if (options.minDifference && difference < options.minDifference) {
    return getMessage("tooSmall", options, key, options.minDifference);
  }
  if (options.maxDifference && difference > options.maxDifference) {
    return getMessage("tooBig", options, key, options.maxDifference);
  }
};