"use strict";

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require("../util");

var defaultOptions = {
  onlyWhenValue: "", // Validated only when value is equal to this
  onlyWhenField: "", // Validated only if this field exists
  onlyWhenFieldEquals: "", // Validated only if the fields above exists and has this value
  countWhen: "hasCount", // Not used atm
  unitsField: null, // Path from the validated fields to units (can be relative)
  unitCountFields: ["count", "pairCount", "adultIndividualCount", "femaleIndividualCount", "individualCount", "juvenileIndividualCount", "maleIndividualCount", "abundanceString"], // count unit if any of these fields have value
  max: null, // Unit count needs to be smaller or equal than this
  min: null, // Unit count needs to be greater or equal than this
  message: {
    tooMany: "There should not be more than %{key} units. But now there was %{value} units",
    tooFew: "There should not be less than %{key} units. But now there was only %{value} units"
  }
};

module.exports = function unitCount(value, options, key, attributes, globalOptions, formData) {
  options = util.extend({}, defaultOptions, options);

  if (options.onlyWhenField) {
    var searchValue = util.parseJSONPointer(options.onlyWhenField, value, formData);
    if (searchValue === undefined || options.onlyWhenFieldEquals !== "" && searchValue !== options.onlyWhenFieldEquals) {
      return;
    }
  }

  if (options.onlyWhenValue !== "" && options.onlyWhenValue !== value) {
    return;
  }

  var units = options.unitsField ? util.parseJSONPointer(options.unitsField, attributes, formData) : value && value.units ? value.units : attributes && attributes.units ? attributes.units : undefined;
  var unitCnt = 0;
  if (typeof units !== "undefined") {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = (0, _getIterator3.default)(units), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var unit = _step.value;

        // TODO: add different counting methods when needed
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = (0, _getIterator3.default)(options.unitCountFields), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var cntField = _step2.value;

            if (typeof unit[cntField] !== "undefined" && unit[cntField] !== "" && unit[cntField] !== null) {
              unitCnt++;
              break;
            }
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  if (typeof options.max === "number" && options.max < unitCnt) {
    return util.errorMessage("tooMany", options, options.max, unitCnt);
  }

  if (typeof options.min === "number" && options.min > unitCnt) {
    return util.errorMessage("tooFew", options, options.min, unitCnt);
  }
};