"use strict";

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require("../util");

var defaultOptions = {
  value: undefined,
  arrayField: "",
  arrayValueMap: {},
  onlyWhenField: "",
  onlyWhenFieldIn: false,
  message: {
    missing: "%{key} does not belong to %{target}"
  }
};

function getMessage(errorKey, options, key, target) {
  var msg = "%{key} is invalid";
  if (options && options.message) {
    msg = typeof options.message === "string" ? options.message : options.message[errorKey] || msg;
  }
  return msg.replace("%{key}", key).replace("%{target}", target);
}

module.exports = function valueInArray(value, options, key, attributes, globalOptions, formData) {
  options = util.extend({}, defaultOptions, options);

  // check that the value is an object
  if ((typeof attributes === "undefined" ? "undefined" : (0, _typeof3.default)(attributes)) !== "object") {
    return;
  }

  value = options.value != null ? options.value : value;

  if (options.onlyWhenField) {
    var searchValues = util.getCheckValues(options.onlyWhenField, attributes, formData);
    if (searchValues.length < 1) {
      return;
    }
    if (options.onlyWhenFieldIn && options.onlyWhenFieldIn.every(function (v) {
      return searchValues.indexOf(v) === -1;
    })) {
      return;
    }
  }

  // pick the check value
  var checkValues = util.getCheckValues(options.arrayField, attributes, formData);
  checkValues = checkValues.reduce(function (result, value) {
    value = options.arrayValueMap[value] || value;
    if (Array.isArray(value)) {
      result = result.concat(value);
    } else {
      result.push(value);
    }
    return result;
  }, []);

  if ((typeof value === "string" || Array.isArray(value)) && value.length > 0 || typeof value === "number" && !isNaN(value) || typeof value === "boolean") {
    if (checkValues.indexOf(value) === -1) {
      return getMessage("missing", options, key, options.arrayField);
    }
  }
};