"use strict";

var util = require("../util");
var waterBirdShortHand = require("./water-bird-short-hand");

var defaultOptions = {
  shortHandField: "shortHandText",
  message: {
    tooBig: "Pair count is too big"
  }
};

function getTotalCount(shortHandText) {
  shortHandText = shortHandText.replace(/\s+/g, ""); // remove spaces
  shortHandText = shortHandText.replace(/K/g, "k").replace(/N/g, "n").replace(/P/g, "p").replace(/\./g, ",");
  shortHandText = shortHandText.replace(/p+/g, "p").replace(/k+/g, "k").replace(/n+/g, "n").replace(/Ä+/g, "Ä").replace(/ä+/g, "ä"); // replace repeated characters with one

  var totalCount = 0;

  shortHandText.split(",").filter(function (val) {
    return !!val;
  }).map(function (val) {
    var observations = val.match(/(?!$)\d*([knpÄä]|$)/g);
    observations.forEach(function (obs) {
      var parts = obs.split(/(?=[knpÄä])/g);
      var count = !isNaN(parseInt(parts[0], 10)) ? parseInt(parts[0], 10) : 1;
      var type = isNaN(parseInt(parts[parts.length - 1], 10)) ? parts[parts.length - 1] : undefined;

      if (type === "p") {
        totalCount += 2 * count;
      } else {
        totalCount += count;
      }
    });
  });

  return totalCount;
}

module.exports = function waterBirdPairCount(value, options, key, attributes, globalOptions, formData) {
  options = util.extend({}, defaultOptions, options);

  if (typeof value !== "number") {
    return;
  }

  var shortHandText = util.parseJSONPointer(options.shortHandField, attributes, formData);

  if (!shortHandText || waterBirdShortHand(shortHandText, {}) !== undefined) {
    return;
  }

  if (value > getTotalCount(shortHandText)) {
    return util.errorMessage("tooBig", options, value);
  }
};