/**
 * These utils allow functions to delay their initial execution until their dependencies have executed (they are
 * provided, as to speak).
 */
export declare function depsProvided(target: any, name: any, args: any): boolean;
/**
 * A decorator. If a function is reflected, it will execute always when some of it's dependencies is reprovided.
 */
export declare function reflect(): (target: any, property: any) => void;
/**
 * A decorator. If the dependencies are not provided on initial execution, the initial execution MUST be delayed until
 * all dependencies are provided. When all dependencies are provided, the initial execution is executed.
 *
 * NOTE: You MUST manually check in the function body if the dependencies are provided! Just put this as the first line
 * of the function body:
 *
 *   if (!depsProvided(this, "functionName", arguments)) return;
 *
 */
export declare function dependsOn(...deps: any[]): (target: any, property: any) => void;
/**
 * Provide a dependency. All delayed functions depending on this function are executed after this.
 */
export declare function provide(target: any, prov: any): void;
export declare function isProvided(target: any, prov: any): any;
