import * as L from "leaflet";
import * as G from "geojson";
import "leaflet-draw";
import "proj4leaflet";
import "Leaflet.vector-markers";
import "leaflet.markercluster";
import "./lib/Leaflet.rrose/leaflet.rrose-src";
import "leaflet.smoothwheelzoom";
import "leaflet-contextmenu";
import "leaflet-textpath";
import "leaflet-measure-path";
import * as NonTiledLayer from "leaflet.nontiledlayer";
import { Provider } from "leaflet-geosearch/lib/providers/provider";
import { Data, DataItemType, DataItemLayer, DataOptions, OverlayName, IdxTuple, DrawHistoryEntry, Lang, Options, Draw, LajiMapFitBoundsOptions, TileLayerName, DrawOptions, LajiMapEvent, GetFeatureStyleOptions, ZoomToDataOptions, TileLayersOptions, InternalTileLayersOptions, UserLocationOptions, OnChangeGeometryFormat, LayerNames, WorldLayerNames, FinnishLayerNames, OverlayNames } from "./map.defs";
import { CustomControl, ControlOptions, InternalControlsOptions } from "./controls.defs";
import { LineTransectEvent, LineTransectGeometry, SegmentIdxTuple, LineTransectFeature, PointIdxTuple, LineTransectOptions, LineTransectHistoryEntry, SegmentLayer, TooltipMessages } from "./line-transect.defs";
export declare function isPolyline(layer: DataItemLayer): boolean;
interface ContextmenuItemOptions {
    text: string;
    iconCls: string;
    callback: () => void;
}
interface ContextmenuOptions {
    contextmenu?: boolean;
    contextmenuInheritItems?: boolean;
    contextmenuItems?: ContextmenuItemOptions[];
    contextmenuWidth?: number;
}
export declare class ContextmenuItem {
}
interface Contextmenu {
    addItem(options: ContextmenuItemOptions | "-"): HTMLElement;
    removeAllItems(): void;
    setDisabled(elem: HTMLElement | number, disabled: boolean): this;
    isVisible(): boolean;
}
declare module "leaflet" {
    interface MapOptions extends ContextmenuOptions {
    }
    interface Path {
        bindContextMenu(options: ContextmenuOptions): Path;
        unbindContextMenu(): any;
    }
    interface Marker {
        bindContextMenu(options: ContextmenuOptions): Marker;
        unbindContextMenu(): any;
        setStyle: (style: L.PathOptions) => void;
    }
    interface Map {
        contextmenu: Contextmenu;
    }
    namespace Contextmenu {
        interface Options extends ContextmenuOptions {
        }
        interface ItemOptions extends ContextmenuItemOptions {
        }
    }
    interface WMSOptions {
        defaultOpacity?: number;
    }
}
declare type MaybeGroupedTileLayer = L.TileLayer | NonTiledLayer.WMS | L.LayerGroup<L.TileLayer | NonTiledLayer.WMS>;
export declare function isMultiTileLayer(layer: L.Layer): layer is L.LayerGroup<L.TileLayer | NonTiledLayer.WMS>;
export default class LajiMap {
    googleApiKey: string;
    googleSearchUrl: string;
    container: HTMLElement;
    mapElem: HTMLElement;
    blockerElem: HTMLElement;
    rootElem: HTMLElement;
    translations: any;
    data: Data[];
    map: L.Map;
    _onDrawReverse: (layer: DataItemLayer) => void;
    _onDrawRemove: (layer: DataItemLayer) => void;
    idxsToIds: {
        [dataIdx: number]: {
            [featureIdx: number]: number;
        };
    };
    idsToIdxs: {
        [dataIdx: number]: {
            [id: number]: number;
        };
    };
    idsToIdxTuples: {
        [id: number]: IdxTuple;
    };
    editIdxTuple: IdxTuple;
    drawing: boolean;
    _drawHistoryPointer: number;
    _drawHistory: DrawHistoryEntry[];
    polyline: L.Polyline;
    userLocation: {
        latlng: L.LatLng;
        accuracy: number;
    };
    userLocationMarker: L.CircleMarker;
    userLocationRadiusMarker: L.Circle;
    userLocationLayer: L.LayerGroup;
    popup: any;
    featurePopupOffset: number;
    markerPopupOffset: number;
    lang: Lang;
    tileLayerOpacity: number;
    onSetLangHooks: (() => void)[];
    options: Options;
    _dialogRoot: HTMLElement;
    _openDialogs: any[];
    _closeDialog: (e?: Event) => void;
    draw: Draw;
    drawIdx: number;
    _draftDrawLayer: DataItemLayer;
    _idxsToHovered: {
        [dataIdx: number]: {
            [featureIdx: number]: boolean;
        };
    };
    _idxsToContextMenuOpen: {
        [dataIdx: number]: {
            [id: number]: boolean;
        };
    };
    locate: [(event: L.LocationEvent) => void, (error: L.ErrorEvent) => void] | boolean;
    _located: boolean;
    locatingOn: boolean;
    popupOnHover: boolean;
    popupCounter: number;
    onPopupClose: () => void;
    _drawReverseLayers: L.Polyline[];
    _drawRemoveLayers: DataItemLayer[];
    _zoomToData: LajiMapFitBoundsOptions | boolean;
    _disableDblClickZoom: boolean;
    dictionary: {
        [lang: string]: any;
    };
    _domCleaners: (() => void)[];
    _documentEvents: {
        [eventName: string]: EventListener[];
    };
    zoom: number;
    center: L.LatLngExpression;
    tileLayer: MaybeGroupedTileLayer;
    overlaysByNames: {
        [name: string]: MaybeGroupedTileLayer;
    };
    availableOverlaysByNames: {
        [name: string]: MaybeGroupedTileLayer;
    };
    overlays: MaybeGroupedTileLayer[];
    finnishTileLayers: {
        [name in (FinnishLayerNames | Extract<OverlayNames, "ykjGrid" | "ykjGridLabels">)]?: MaybeGroupedTileLayer;
    };
    worldTileLayers: {
        [name in WorldLayerNames]?: MaybeGroupedTileLayer;
    };
    tileLayers: {
        [name in LayerNames]?: MaybeGroupedTileLayer;
    };
    tileLayerName: TileLayerName;
    _tileLayers: InternalTileLayersOptions;
    availableTileLayers: {
        [name: string]: MaybeGroupedTileLayer;
    };
    _listenedEvents: L.LeafletEventHandlerFnMap;
    _keyListeners: {
        [eventName: string]: {
            [key: string]: ((e: Event) => boolean | void)[];
        };
    };
    _swapToWorldFlag: boolean;
    _mouseLatLng: L.LatLng;
    _contextMenuLayer: L.Layer;
    _preventScrollDomCleaner: () => void;
    _onDrawStopPreventScrolling: () => void;
    _onDrawStartPreventScrolling: () => void;
    _preventScroll: boolean;
    _scrollPreventElem: HTMLElement;
    _scrollPreventTextElemContainer: HTMLElement;
    _scrollPreventTextElem: HTMLElement;
    _scrollPreventScrollListeners: ([string, EventListener])[];
    _startPreventScrollingTimeout: () => void;
    _onControlClickPreventScrolling: () => boolean;
    _onMouseDownPreventScrolling: EventListener;
    _onTouchPreventScrolling: EventListener;
    _preventScrolling: () => void;
    _showPreventAnimationTimeout: any;
    _showPreventShowTimeout: any;
    _showPreventHideTimeout: any;
    _showingPreventScroll: boolean;
    _LTEditPointIdxTuple: IdxTuple;
    _clickBeforeZoomAndPan: boolean;
    _origLatLngs: {
        [id: string]: L.LatLng[];
    };
    _tooltip: L.Draw.Tooltip;
    _tooltipTranslationHook: () => void;
    _onMouseMove: (e: any) => void;
    providers: ["Google" | "MML", Provider<any, any>][];
    leafletOptions: L.MapOptions;
    _viewCriticalSection: number;
    _tileLayersSet: boolean;
    activeProjName: TileLayersOptions["active"];
    _tileLayerOrder: string[];
    viewLocked: boolean;
    _locateParam: UserLocationOptions | boolean;
    locateOptions: UserLocationOptions;
    availableTileLayersWhitelist: TileLayerName[];
    availableTileLayersBlacklist: TileLayerName[];
    availableOverlaysBlacklist: OverlayName[];
    availableOverlaysWhitelist: OverlayName[];
    _initialized: boolean;
    mmlProj: L.Proj.CRS;
    _trySwapToFinnishOnInitialization: boolean;
    _fullscreen: boolean;
    _fullscreenElem: HTMLElement;
    _fullscreenCloseElem: HTMLElement;
    _fullscreenTranslateHook: () => void;
    _beforeFullscreen: {
        rootElem: HTMLElement;
        bodyAsDialogRoot: boolean;
        clickBeforeZoomAndPan: boolean;
        bodyOverflowY: string;
    };
    lajiGeoServerAddress: string;
    controls: L.Control[];
    _customControls: CustomControl[];
    layerControl: L.Control.Layers;
    controlItems: ControlOptions[];
    _controlItemsByName: {
        [controlName: string]: ControlOptions;
    };
    activeControl: L.Control;
    controlSettings: InternalControlsOptions;
    drawControl: L.Control.Draw;
    _locateOn: boolean;
    _controlButtons: {
        [controlName: string]: HTMLElement;
    };
    _opacitySetBySlide: boolean;
    _slider: any;
    _contextMenuItems: {
        [buttonName: string]: HTMLElement;
    };
    _internalTileLayersUpdate: boolean;
    _allPoints: L.CircleMarker[];
    _hoveredIdxTuple: SegmentIdxTuple;
    LTFeature: LineTransectFeature;
    _splitIdxTuple: PointIdxTuple;
    _splitPoint: L.LatLng;
    _lineLayers: L.Polyline<G.LineString>[][];
    _pointLayers: L.CircleMarker[][];
    _pointIdsToIdxTuples: {
        [id: number]: PointIdxTuple;
    };
    _corridorLayers: L.Polygon<G.Polygon>[][];
    _allSegments: L.Polyline<G.LineString>[];
    _allCorridors: L.Polygon<G.Polygon>[];
    _lineSplitFn: (idxTuple: SegmentIdxTuple, splitPoint: L.LatLng) => void;
    _selectLTMode: "segment" | "line";
    _LTActiveIdx: number;
    _onLTChange: LineTransectOptions["onChange"];
    _LTDragging: boolean;
    _getLTFeatureStyle: LineTransectOptions["getFeatureStyle"];
    _getLTTooltip: LineTransectOptions["getTooltip"];
    _LTHistory: LineTransectHistoryEntry[];
    _LTHistoryPointer: number;
    _LTPrintMode: boolean;
    _LTEditable: boolean;
    _pointDragSnapMode: boolean;
    _dragCorridor: L.Polygon<G.Polygon>;
    _origLineTransect: L.FeatureGroup;
    _pointLayerGroup: L.FeatureGroup;
    _lineLayerGroup: L.FeatureGroup;
    _corridorLayerGroup: L.FeatureGroup;
    _overlappingNonadjacentPointIdxTuples: {
        [idxTupleString: string]: PointIdxTuple;
    };
    _overlappingAdjacentPointIdxTuples: {
        [idxTupleString: string]: PointIdxTuple;
    };
    _lineIdxsTupleStringsToLineGroupIdxs: {
        [idxTupleString: string]: number;
    };
    _groupIdxsToLineIdxs: {
        [groupIdx: number]: number[];
    };
    _LTStartText: L.Polyline;
    _LTGroups: L.FeatureGroup[];
    _tooltipIdx: number;
    _overlappingPointDialogSegmentIdxTuple: SegmentIdxTuple;
    leafletIdsToCorridorLineIdxs: {
        [id: string]: number;
    };
    leafletIdsToCorridorSegmentIdxs: {
        [id: string]: number;
    };
    leafletIdsToFlatCorridorSegmentIdxs: {
        [id: string]: number;
    };
    leafletIdsToFlatPointIdxs: {
        [id: string]: number;
    };
    lineIdsToCorridorIds: {
        [id: string]: number;
    };
    corridorFlatIdxsToLeafletIds: {
        [id: string]: number;
    };
    _hoveredType: "point" | "corridor";
    _LTClickTimeout: number;
    _closebyPointIdxTuple: PointIdxTuple;
    _pointLTShiftMode: boolean;
    _onSelectLT: (idxTuple: SegmentIdxTuple) => void;
    _firstLTSegmentToRemoveIdx: SegmentIdxTuple;
    _LTPointExpander: L.CircleMarker;
    _LTdragPoint: L.CircleMarker;
    _LTContextMenuLayer: SegmentLayer;
    _featureBeforePointDrag: LineTransectFeature;
    _LTPointLatLngBeforeDrag: L.LatLng;
    _hoveringDragPoint: boolean;
    _dragPointStart: L.LatLng;
    _dragMouseStart: L.LatLng;
    _cutLine: L.Polygon;
    _lineCutIdx: SegmentIdxTuple;
    _ltTooltip: L.Draw.Tooltip;
    messages: TooltipMessages;
    constructor(props: Options);
    getOptionKeys(): any;
    setOptions(options?: Options): void;
    setOption(option: keyof Options, value: any): void;
    getOption(option: keyof Options): any;
    getOptions(): Options;
    setRootElem(rootElem: HTMLElement): void;
    setBodyAsDialogRoot(value?: boolean): void;
    shouldNotPreventScrolling(): boolean;
    setClickBeforeZoomAndPan(value?: boolean): void;
    getMMLProj(): L.CRS;
    _initializeMap(): void;
    _initializeView(): void;
    _isOutsideFinland(latLng: L.LatLngExpression): boolean;
    _swapToWorldOutsideFinland(latLng: L.LatLngExpression): void;
    _initializeMapEvents(): void;
    keyHandlerForType(type: string, e: any): boolean;
    _addDocumentEventListener(type: string, fn: EventListener, useCapture?: boolean): void;
    _removeDocumentEventListener(type: string, fn: EventListener): void;
    _addKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
    _removeKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
    _triggerKeyEvent(key: string, e: Event, type: string): boolean;
    setEventListeners(eventListeners: L.LeafletEventHandlerFnMap): void;
    _getDefaultCRSLayers(): MaybeGroupedTileLayer[];
    _getMMLCRSLayers(): MaybeGroupedTileLayer[];
    setTileLayerByName(name: TileLayerName): void;
    getListForAvailableLayers(names: LayerNames[], condition: any, _tileLayers?: {
        [name: string]: MaybeGroupedTileLayer;
    }): {};
    setAvailableTileLayers(names: TileLayerName[], condition: boolean, _tileLayers?: {
        [name: string]: MaybeGroupedTileLayer;
    }): void;
    getDefaultTileLayerBlacklist(): TileLayerName[];
    getDefaultTileLayerWhitelist(): TileLayerName[];
    setAvailableTileLayerBlacklist(names: TileLayerName[]): void;
    setAvailableTileLayerWhitelist(names: TileLayerName[]): void;
    getAvailableOverlaysByNames(): {
        [name: string]: MaybeGroupedTileLayer[];
    };
    getAvailableWorldTileLayers(): {
        [name: string]: MaybeGroupedTileLayer[];
    };
    getAvailableFinnishTileLayers(): {
        [name: string]: MaybeGroupedTileLayer[];
    };
    getAvailableLayersFor(_layers: any, availables: any): {
        [name: string]: MaybeGroupedTileLayer[];
    };
    setTileLayer(layer: MaybeGroupedTileLayer): void;
    getActiveLayers(options: InternalTileLayersOptions): InternalTileLayersOptions["layers"];
    _getInternalTileLayerOptions(options: TileLayersOptions): InternalTileLayersOptions;
    setTileLayers(options: TileLayersOptions): void;
    getTileLayers(): TileLayersOptions;
    setTileLayerOpacity(val: any, triggerEvent?: boolean): void;
    setOverlays(overlays?: MaybeGroupedTileLayer[]): void;
    _setOverlaysByName(overlayNames: OverlayName[]): void;
    getOverlaysByName(): OverlayName[];
    setOverlaysByName(overlayNames?: OverlayName[]): void;
    getDefaultOverlaysBlacklist(): OverlayName[];
    getDefaultOverlaysWhitelist(): OverlayName[];
    setAvailableOverlaysBlacklist(overlayNames: OverlayName[]): void;
    setAvailableOverlaysWhitelist(overlayNames: OverlayName[]): void;
    setAvailableOverlays(overlayNames: OverlayName[], condition: any, _tileLayers?: {
        [name: string]: MaybeGroupedTileLayer;
    }): void;
    getNormalizedZoom(zoom?: number, tileLayer?: MaybeGroupedTileLayer): number;
    getDenormalizedZoom(zoom?: number, tileLayer?: MaybeGroupedTileLayer): number;
    initZoom(zoom: any): void;
    setNormalizedZoom(zoom: any, options?: {
        animate: boolean;
    }): void;
    setCenter(center: L.LatLngExpression): void;
    destroy(): void;
    cleanDOM(): void;
    _addDomCleaner(fn: () => void): void;
    _removeDomCleaner(fn: () => void): void;
    _constructDictionary(): void;
    setLang(lang: Lang, shouldProvide?: boolean): void;
    formatFeatureOut(feature: G.Feature, layer?: DataItemLayer): G.Feature;
    formatFeatureIn(feature: G.Feature, idx: number): G.Feature;
    cloneFeatures(features: G.Feature[]): G.Feature[];
    initializeDataItem(options: DataOptions, dataIdx?: number): void;
    _getAllData(): {
        group: L.FeatureGroup;
    }[];
    zoomToData(options?: ZoomToDataOptions | boolean): void;
    getBoundsForZoomToDataOptions(options?: ZoomToDataOptions | boolean): L.LatLngBounds;
    getBoundsForDraw(): L.LatLngBounds;
    getBoundsForIdxs(...idxs: any[]): L.LatLngBounds;
    getBoundsForData(datas?: {
        group: L.FeatureGroup;
    }[]): L.LatLngBounds;
    getBoundsForLayers(layers: L.Layer[]): L.LatLngBounds;
    _mapLayersToBoundableLayers(layers: L.Layer[]): L.Layer[];
    _initializeLayer(layer: DataItemLayer, idxTuple: IdxTuple): void;
    fitBounds(_bounds: L.LatLngBoundsExpression, options?: LajiMapFitBoundsOptions): void;
    setData(data: Data[] | Data): void;
    getData: () => Data[];
    addData(items: DataOptions[]): void;
    updateData(dataIdx: any, item: DataOptions): void;
    updateDrawData(item: DrawOptions): void;
    removeData(dataIdx: number): void;
    getDrawOptions(options: DrawOptions | boolean): DrawOptions;
    setDraw(options: DrawOptions): void;
    getDraw(): Draw;
    drawIsAllowed(): boolean;
    drawIsEditable(): boolean;
    resetDrawUndoStack(): void;
    _getEventsWithLayers(events: LajiMapEvent[]): (import("./map.defs").LajiMapDeleteEvent | import("./map.defs").LajiMapActivateEvent | {
        layer: DataItemLayer;
        type: "create";
        feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
    } | {
        layer: DataItemLayer;
        type: "insert";
        idx: number;
        feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
    } | {
        layers: {};
        type: "edit";
        features: {
            [idx: number]: G.Feature<G.Geometry, {
                [name: string]: any;
            }>;
        };
    })[];
    drawUndo(): void;
    drawRedo(): void;
    wrapGeoJSONCoordinate([lng, lat]: G.Position): G.Position;
    _setOnChangeForItem(item: any, format?: OnChangeGeometryFormat, crs?: string): void;
    clearItemData(item: Data, commit?: boolean): void;
    clearDrawData(): void;
    _startDrawRemove(): void;
    _stopDrawRemove(): boolean;
    _finishDrawRemove(): void;
    _cancelDrawRemove(): boolean;
    _onDrawReverseHandler({ layer }: L.GeoJSONEvent): void;
    _startDrawReverse(): void;
    _stopDrawReverse(): void;
    _finishDrawReverse(): void;
    _cancelDrawReverse(): void;
    _createIcon(options?: L.PathOptions, icon?: (pathOptions: L.PathOptions, feature: G.Feature) => L.Icon, feature?: G.Feature): L.Icon;
    _getClusterIcon(data: Data): (cluster: L.MarkerCluster) => L.DivIcon;
    setActive(layer: DataItemLayer): void;
    _resetIds(dataIdx: number): void;
    _setIdForLayer(layer: DataItemLayer, idxTuple: IdxTuple): void;
    recluster(): void;
    _reclusterDrawData(): void;
    _reclusterData(): void;
    getClusterOptionsFor(item: Data): {
        maxClusterRadius?: number | ((zoom: number) => number);
        iconCreateFunction: ((cluster: L.MarkerCluster) => L.Icon<L.IconOptions> | L.DivIcon) | ((cluster: L.MarkerCluster) => L.DivIcon);
        clusterPane?: string;
        spiderfyOnEveryZoom?: boolean;
        spiderfyOnMaxZoom?: boolean;
        showCoverageOnHover?: boolean;
        zoomToBoundsOnClick?: boolean;
        singleMarkerMode?: boolean;
        disableClusteringAtZoom?: number;
        removeOutsideVisibleBounds?: boolean;
        animate?: boolean;
        animateAddingMarkers?: boolean;
        spiderfyShapePositions?: (count: number, centerPoint: L.Point) => L.Point[];
        spiderfyDistanceMultiplier?: number;
        spiderLegPolylineOptions?: L.PolylineOptions;
        chunkedLoading?: boolean;
        chunkDelay?: number;
        chunkInterval?: number;
        chunkProgress?: (processedMarkers: number, totalMarkers: number, elapsedTime: number) => void;
        polygonOptions?: L.PolylineOptions;
        pane?: string;
        attribution?: string;
    };
    _reclusterDataItem(item: Data): void;
    reorderData(): void;
    redrawDrawData(): void;
    redrawDataItem(idx: number): void;
    redrawData(): void;
    redraw(): void;
    _initializePopup(layer: DataItemLayer): void;
    _initializeTooltip(layer: DataItemLayer): void;
    _updateContextMenuForLayer(layer: DataItemLayer): void;
    setLocate(locate?: UserLocationOptions | boolean): void;
    _setLocateOn(triggerEvent?: boolean): void;
    _setLocateOff(): void;
    _onLocationFound({ latlng, accuracy, bounds }: L.LocationEvent): void;
    createUserLocationLayer(latlng: L.LatLng, accuracy: number): {
        layerGroup: L.LayerGroup;
        markerLayer: L.CircleMarker;
        radiusLayer: L.Circle;
    };
    _onLocationNotFound(e: L.ErrorEvent): void;
    getDrawLayerByIdx(idx: number): DataItemLayer;
    _getLayerByIdxTuple: (idxTuple: IdxTuple) => DataItemLayer;
    getLayerByIdxTuple(idxTuple: IdxTuple): DataItemLayer;
    _getLayerById(id: number): DataItemLayer;
    _getIdxTupleByLayer(layer: any): IdxTuple;
    _getDrawLayerById(id: any): DataItemLayer;
    _triggerEvent(e: any, handler: any): void;
    _decoratePolyline(layer: L.Polyline): void;
    _reversePolyline(layer: any): void;
    _getStartCircleStyle(lineLayer: any): L.PathOptions;
    _updateDrawUndoStack(events: LajiMapEvent[] | LajiMapEvent, prevFeatureCollection: any, prevActiveIdx?: any): void;
    _onAdd(dataIdx: number, layer: DataItemLayer, coordinateVerbatim?: string): void;
    _onEdit(dataIdx: any, data: any): void;
    _onDelete(dataIdx: any, deleteIds: any): void;
    _removeLayerFromItem(item: Data, layer: DataItemLayer): void;
    _onActiveChange(idxTuple: IdxTuple): void;
    focusToLayerByIdxs(idxTuple: IdxTuple): void;
    focusToDrawLayer(idx: number): void;
    _setEditable(layer: DataItemLayer): void;
    _clearEditable(): void;
    _commitEdit(): void;
    _interceptClick(): boolean;
    setLayerStyle(layer: DataItemLayer | L.FeatureGroup, style: L.PathOptions): void;
    _featureToLayer(getFeatureStyle: (options: GetFeatureStyleOptions) => L.PathOptions, dataIdx?: number): (feature: any) => any;
    _getDefaultDrawDraftStyle(): L.PathOptions;
    _fillStyleWithGlobals<T extends L.PathOptions>(idxTuple: IdxTuple): T;
    _getStyleForType(idx: IdxTuple | number, overrideStyles?: L.PathOptions): L.PathOptions;
    _getStyleForLayer(layer: DataItemLayer, overrideStyles?: any): L.PathOptions;
    updateLayerStyle(layer: DataItemLayer): void;
    _getDefaultDrawStyle(): L.PathOptions;
    _getDefaultDrawClusterStyle(item: Data): L.PathOptions;
    _getDefaultDataStyle(item: Data): () => L.PathOptions;
    _getDefaultDataClusterStyle: (item: any) => () => {
        color: string;
        opacity: any;
    };
    _getDefaultDraftStyle(dataIdx: number): L.PathOptions;
    _updateDataLayerGroupStyle(idx: number): void;
    addTranslationHook(elemOrFn: HTMLElement | (() => void), translationKey?: string | (() => string), attr?: string): () => void;
    removeTranslationHook(hook: () => void): void;
    _getDrawOptionsForType<T extends L.PathOptions>(featureType: DataItemType): T;
    abortDrawing(e?: Event | L.LeafletEvent): void;
    addDrawAbortListeners(): void;
    triggerDrawing(featureType: DataItemType): void;
    addFeatureToDraw(feature: G.Feature): void;
    addFeatureToData(feature: G.Feature, dataIdx: number): void;
    getFeatureTypes(): DataItemType[];
    _showError(e: Error): void;
    showClosableElement(elem: HTMLElement, onClose: (e: Event) => void, blocker?: boolean, getContainer?: () => HTMLElement): void;
    _createTooltip(translationKey: string, error?: boolean): L.Draw.Tooltip;
    _disposeTooltip(): void;
    _showDialog(container: HTMLElement, onClose?: (e: Event) => void): void;
    setLineTransectGeometry(geometry: LineTransectGeometry, events?: LineTransectEvent[]): void;
    setGoogleApiKey(googleApiKey: string): void;
    setGoogleSearchUrl(url: string): void;
    setGeocodingProvider(): void;
    geocode(query: string, additional?: any, zoom?: number, provider?: "mml" | "google"): void;
    setViewLocked(value: any): void;
    toggleFullscreen(): void;
    setFullscreenOn(): void;
    setFullscreenOff(): void;
}
export {};
