"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMultiTileLayer = exports.ContextmenuItem = exports.isPolyline = void 0;
const L = require("leaflet");
require("leaflet-draw");
require("proj4leaflet");
require("Leaflet.vector-markers");
require("leaflet.markercluster");
require("./lib/Leaflet.rrose/leaflet.rrose-src");
require("leaflet.smoothwheelzoom");
require("leaflet-contextmenu");
require("leaflet-textpath");
require("leaflet-measure-path");
const NonTiledLayer = require("leaflet.nontiledlayer");
const leaflet_geosearch_1 = require("leaflet-geosearch");
const mml_provider_1 = require("./mml-provider");
const utils_1 = require("./utils");
const dependency_utils_1 = require("./dependency-utils");
const globals_1 = require("./globals");
const map_defs_1 = require("./map.defs");
const translations_1 = require("./translations");
function capitalizeFirstLetter(s) {
    return s.charAt(0).toUpperCase() + s.slice(1);
}
function isPolyline(layer) {
    return layer instanceof L.Polyline && ["Rectangle", "Polygon"].every(type => !(layer instanceof L[type]));
}
exports.isPolyline = isPolyline;
const DEFAULT_LAYER_NAME = map_defs_1.TileLayerName.taustakartta;
// Override the tooltip to turn when it overflows.
const tooltipOrigPrototype = L.Draw.Tooltip.prototype;
L.Draw.Tooltip = L.Draw.Tooltip.extend({
    updatePosition(latlng) {
        tooltipOrigPrototype.updatePosition.call(this, latlng);
        const { width, x } = this._container.getBoundingClientRect();
        const { width: mapWidth, x: mapX } = this._map._container.getBoundingClientRect();
        if (width + x > mapWidth + mapX) {
            const { x: _x, y: _y } = this._map.latLngToLayerPoint(latlng);
            L.DomUtil.setPosition(this._container, new L.Point(_x - width - 30, _y));
            if (this._container.className.indexOf(" reversed") === -1) {
                this._container.className += " reversed";
            }
        }
        else if (this._container.className.indexOf(" reversed") !== -1) {
            this._container.className = this._container.className.replace(" reversed", "");
        }
        return this;
    }
});
class GoogleProvider extends leaflet_geosearch_1.GoogleProvider {
    constructor(options) {
        super(options);
        if (options.searchUrl) {
            this.searchUrl = options.searchUrl;
        }
    }
}
// Patch L.Marker to have `setStyle()` so it works the same as all the other rendered features (polygon, polyline etc).
// If icons are customized, they should have a `setStyle()` implementation.
const _initIcon = L.Marker.prototype._initIcon;
L.Marker.include({
    _initIcon() {
        _initIcon.call(this);
        // Order of init and setStyle() not guaranteed, so we use the stored _initStyle if setStyle() has ran before.
        if (this._initStyle) {
            this.setStyle(this._initStyle);
        }
    },
    setStyle(style) {
        this.__storedStyle = style;
        if (!this._icon) {
            this._initStyle = style;
            return;
        }
        if (this.options.icon && !this.options.icon.setStyle) {
            // eslint-disable-next-line max-len
            console.warn("[laji-map warning] Seems like you are using a customized marker icon. You should implement 'setStyle({color, opacity})' for it if you wish it to work with the coloring & other styling that laji-map provides.");
        }
        else {
            if (this.setStyle !== this.options.icon.setStyle) {
                this.options.icon.setStyle(this._icon, style);
            }
        }
        if (this._shadow && style.hasOwnProperty("opacity")) {
            this._shadow.style.opacity = style.opacity;
        }
    },
    // Take control of opacity setting so that it's handled by the icon setStyle() (called by this.setStyle()) instead of
    // messing up with the DOM straight by Leaflet.
    setOpacity(opacity) {
        this.setStyle(Object.assign(Object.assign({}, this.__storedStyle), { opacity }));
    },
});
L.Icon.Default.include({
    setStyle() {
        console.warn("[laji-map warning] The Leaflet default icon doesn't have an implementation for 'setStyle({color, opacity})'");
    }
});
// Implement `setStyle()` for Vector Marker which is the default icon for laji-map.
const origInitialize = L.VectorMarkers.Icon.prototype.initialize;
L.VectorMarkers.Icon.include({
    initialize(options) {
        if (options.color) {
            options.markerColor = options.color;
        }
        origInitialize.call(this, options);
    },
    setStyle(iconDomElem, style) {
        var _a;
        if (!iconDomElem.firstChild) {
            return;
        }
        iconDomElem.firstChild.firstChild.style.fill = style.color;
        iconDomElem.style.opacity = (_a = style.opacity) !== null && _a !== void 0 ? _a : 1;
    }
});
//
// Remove tabindex from markers https://github.com/Leaflet/Leaflet/issues/3472#issuecomment-103058457
L.Marker.addInitHook(function () {
    this.options.keyboard = false;
});
L.MarkerCluster.addInitHook(function () {
    this.options.keyboard = false;
});
class ContextmenuItem {
}
exports.ContextmenuItem = ContextmenuItem;
function isMultiTileLayer(layer) {
    return layer instanceof L.LayerGroup;
}
exports.isMultiTileLayer = isMultiTileLayer;
const computeOpacities = (visible, opacity, maxFillOpacity = 0.4) => {
    return { opacity: visible ? opacity : 0, fillOpacity: visible ? (maxFillOpacity * opacity) : 0 };
};
class LajiMap {
    constructor(props) {
        this.data = [];
        this.idxsToIds = [];
        this.idsToIdxs = [];
        this.idsToIdxTuples = {};
        this.onSetLangHooks = [];
        this._openDialogs = [];
        this._idxsToHovered = {};
        this._idxsToContextMenuOpen = {};
        this._domCleaners = [];
        this._documentEvents = {};
        this._keyListeners = {};
        this._viewCriticalSection = 0;
        this._tileLayerOrder = [
            "taustakartta",
            "maastokartta",
            "laser",
            "ortokuva",
            "ykjGrid",
            "ykjGridLabels",
            "atlasGrid",
            "afeGrid",
            "openStreetMap",
            "googleSatellite",
            "cgrsGrid",
            "geobiologicalProvinces",
            "geobiologicalProvincesBorders",
            "municipalities",
            "counties",
            "ely",
            "kiinteistojaotus",
            "kiinteistotunnukset",
            "forestVegetationZones",
            "mireVegetationZones",
            "threatenedSpeciesEvaluationZones",
            "biodiversityForestZones",
            "habitat",
            "ageOfTrees",
            "currentProtectedAreas",
            "plannedProtectedAreas",
            "flyingSquirrelPredictionModel",
            "birdAtlasSocietyGridZones",
            "barentsRegion",
        ];
        this._initialized = false;
        this._trySwapToFinnishOnInitialization = true;
        this._fullscreen = false;
        this.lajiGeoServerAddress = "https://geoserver.laji.fi/geoserver";
        this._pointDragSnapMode = false;
        this.getData = () => {
            const data = [...this.data];
            return data;
        };
        this._getLayerByIdxTuple = this.getLayerByIdxTuple;
        this._getDefaultDataClusterStyle = (item) => () => {
            let color = item.editable ? globals_1.EDITABLE_DATA_LAYER_COLOR : globals_1.DATA_LAYER_COLOR;
            if (item.getFeatureStyle) {
                const style = item.getFeatureStyle({});
                if (style.color)
                    color = style.color;
            }
            return { color, opacity: item.opacity };
        };
        this._constructDictionary();
        const options = {
            lang: map_defs_1.Lang.en,
            data: [],
            locate: false,
            center: [65, 26],
            zoom: 2,
            zoomToData: false,
            popupOnHover: false,
            draw: false,
            bodyAsDialogRoot: true,
            clickBeforeZoomAndPan: false,
            viewLocked: false,
            availableOverlayNameBlacklist: [
                map_defs_1.OverlayName.kiinteistojaotus,
                map_defs_1.OverlayName.kiinteistotunnukset,
                map_defs_1.OverlayName.flyingSquirrelPredictionModel,
                map_defs_1.OverlayName.barentsRegion
            ],
            googleSearchUrl: "https://proxy.laji.fi/google-geocode/json"
        };
        // Since options are applied in undeterministic order, make sure that props tile layers has higher preference.
        if (!props.tileLayerName && !props.tileLayers) {
            options.tileLayerName = DEFAULT_LAYER_NAME;
        }
        this.options = Object.assign(Object.assign({}, options), props);
        this.leafletOptions = {};
        this.setOptions(this.options);
        this._initializeMap();
        this._initializeView();
        this.setGeocodingProvider();
        this._stopDrawRemove = this._stopDrawRemove.bind(this);
        this._stopDrawReverse = this._stopDrawReverse.bind(this);
        this._onDrawReverseHandler = this._onDrawReverseHandler.bind(this);
        this.abortDrawing = this.abortDrawing.bind(this);
        this.setFullscreenOff = this.setFullscreenOff.bind(this);
        this._initialized = true;
    }
    getOptionKeys() {
        return {
            rootElem: "setRootElem",
            lang: "setLang",
            data: ["setData", () => this.getData()],
            draw: ["setDraw", () => this.getDraw()],
            tileLayerName: "setTileLayerByName",
            availableTileLayerNamesBlacklist: "setAvailableTileLayerBlacklist",
            availableTileLayerNamesWhitelist: "setAvailableTileLayerWhitelist",
            overlayNames: ["_setOverlaysByName", () => this.getOverlaysByName()],
            availableOverlayNameBlacklist: "setAvailableOverlaysBlacklist",
            availableOverlayNameWhitelist: "setAvailableOverlaysWhitelist",
            tileLayerOpacity: "setTileLayerOpacity",
            tileLayers: ["setTileLayers", () => this.getTileLayers()],
            center: ["setCenter", () => this.map.getCenter()],
            zoom: ["initZoom", () => this.getNormalizedZoom()],
            zoomToData: ["zoomToData", "_zoomToData"],
            locate: ["setLocate", "locate"],
            onPopupClose: true,
            markerPopupOffset: true,
            featurePopupOffset: true,
            popupOnHover: true,
            on: ["setEventListeners", "_listenedEvent"],
            polyline: true,
            polygon: true,
            rectangle: true,
            circle: true,
            marker: true,
            bodyAsDialogRoot: ["setBodyAsDialogRoot", () => this._dialogRoot === document.body],
            clickBeforeZoomAndPan: ["setClickBeforeZoomAndPan", "_clickBeforeZoomAndPan"],
            googleApiKey: "setGoogleApiKey",
            googleSearchUrl: "setGoogleSearchUrl",
            viewLocked: "setViewLocked",
            lajiGeoServerAddress: true
        };
    }
    setOptions(options = {}) {
        Object.keys(options).forEach((option) => {
            this.setOption(option, options[option]);
        });
    }
    setOption(option, value) {
        const optionKeys = this.getOptionKeys();
        if (!optionKeys.hasOwnProperty(option)) {
            if (!this._initialized) {
                this.leafletOptions[option] = value;
            }
            else {
                console.warn(`setting leaflet options works only during initialization. '${option}' isn't a LajiMap option.`);
            }
            return;
        }
        const optionKey = Array.isArray(optionKeys[option]) ? optionKeys[option][0] : optionKeys[option];
        if (optionKey === true) {
            this[option] = value;
        }
        else {
            this[optionKey](value);
        }
    }
    getOption(option) {
        const optionKeys = this.getOptionKeys();
        if (Array.isArray(optionKeys[option])) {
            const getter = optionKeys[option][1];
            return typeof getter === "function" ? getter() : this[getter];
        }
        else if (option in this) {
            return this[option];
        }
    }
    getOptions() {
        const optionKeys = this.getOptionKeys();
        return Object.keys(optionKeys).reduce((options, key) => {
            options[key] = this.getOption(key);
            return options;
        }, {});
    }
    setRootElem(rootElem) {
        var _a, _b, _d, _f;
        const oldContainer = this.container;
        const { children } = oldContainer || {};
        this.mapElem = this.mapElem || document.createElement("div");
        this.blockerElem = this.blockerElem || document.createElement("div");
        const oldActive = document.activeElement;
        const isFocused = ((_a = this.map) === null || _a === void 0 ? void 0 : _a.getContainer()) && this.map.getContainer().contains(document.activeElement);
        if (!isFocused) {
            (_b = this.map) === null || _b === void 0 ? void 0 : _b.fire("blur");
        }
        this.container = document.createElement("div");
        this.container.className = "laji-map";
        this.blockerElem.className = "laji-map-blocker";
        if (children) {
            for (let child of Array.from(children)) {
                this.container.appendChild(child);
            }
        }
        else {
            this.container.appendChild(this.mapElem);
        }
        const shouldSetZoomAndPan = this._clickBeforeZoomAndPan;
        shouldSetZoomAndPan && this.setClickBeforeZoomAndPan(false);
        this.rootElem = rootElem;
        this.rootElem.appendChild(this.container);
        if (oldContainer && oldContainer !== this.container && oldContainer.parentElement) {
            oldContainer.parentElement.removeChild(oldContainer);
        }
        shouldSetZoomAndPan && this.setClickBeforeZoomAndPan(true);
        this._dialogRoot && this.setBodyAsDialogRoot(this._dialogRoot === document.body);
        this.map && this.map.invalidateSize();
        if (isFocused && oldActive && document.activeElement !== oldActive) {
            (_f = (_d = oldActive).focus) === null || _f === void 0 ? void 0 : _f.call(_d);
        }
        dependency_utils_1.provide(this, "rootElem");
    }
    setBodyAsDialogRoot(value = true) {
        var _a;
        if (!dependency_utils_1.depsProvided(this, "setBodyAsDialogRoot", arguments))
            return;
        const prevBodyRoot = this._dialogRoot;
        if (value) {
            this._dialogRoot = document.body;
        }
        else {
            this._dialogRoot = this.container;
        }
        if (prevBodyRoot) {
            if (this.blockerElem.parentNode === prevBodyRoot)
                prevBodyRoot.removeChild(this.blockerElem);
            if (this.blockerElem.className.indexOf("fixed") !== -1) {
                this.blockerElem.className = this.blockerElem.className.replace(" fixed", "");
            }
            (_a = this._openDialogs) === null || _a === void 0 ? void 0 : _a.forEach(elem => {
                this._dialogRoot.appendChild(elem);
            });
        }
        this._dialogRoot.appendChild(this.blockerElem);
        if (value)
            this.blockerElem.className = `${this.blockerElem.className} fixed`;
    }
    shouldNotPreventScrolling() {
        return !!(this.drawing || this._LTEditPointIdxTuple);
    }
    setClickBeforeZoomAndPan(value = true) {
        if (!dependency_utils_1.depsProvided(this, "setClickBeforeZoomAndPan", arguments))
            return;
        if (this.viewLocked && value) {
            return;
        }
        const valueWas = this._clickBeforeZoomAndPan;
        this._clickBeforeZoomAndPan = value;
        const isOutsideLajiMap = elem => document.body.contains(elem)
            && !this.container.contains(elem)
            && elem !== this.blockerElem
            && (!this._openDialogs.length || this._openDialogs.every(dialog => !dialog.contains(elem)));
        let translationHook = undefined;
        this._scrollPreventElem = this._scrollPreventElem || document.createElement("div");
        this._scrollPreventTextElemContainer = this._scrollPreventTextElemContainer || document.createElement("div");
        this._scrollPreventTextElem = this._scrollPreventTextElem || document.createElement("span");
        this._scrollPreventTextElemContainer.appendChild(this._scrollPreventTextElem);
        this._scrollPreventElem.className = "laji-map-scroll-prevent left";
        this._scrollPreventTextElemContainer.className = "laji-map-scroll-prevent text-container left";
        translationHook = this.addTranslationHook(this._scrollPreventTextElem, "ClickBeforeZoom");
        this._showingPreventScroll = false;
        if (!this._preventScrolling) {
            this._preventScrolling = () => {
                if (this.shouldNotPreventScrolling())
                    return;
                this.map.smoothWheelZoom.disable();
                this.map.dragging.disable();
                this._preventScroll = true;
            };
        }
        if (!this._startPreventScrollingTimeout) {
            this._startPreventScrollingTimeout = () => {
                clearTimeout(this._showPreventShowTimeout);
                this._showPreventShowTimeout = setTimeout(() => {
                    this._preventScrolling();
                }, 10000);
            };
        }
        const showPreventElem = () => {
            const showingAlready = this._showingPreventScroll;
            this._showingPreventScroll = true;
            if (!showingAlready) {
                [].forEach.call(document.querySelectorAll(".laji-map-scroll-prevent"), elem => elem.className = elem.className.replace("left", "enter"));
                setTimeout(() => {
                    [].forEach.call(document.querySelectorAll(".laji-map-scroll-prevent"), elem => elem.className = elem.className.replace(" enter", ""));
                });
            }
            clearTimeout(this._showPreventHideTimeout);
            clearTimeout(this._showPreventShowTimeout);
            clearTimeout(this._showPreventAnimationTimeout);
            this._showPreventHideTimeout = setTimeout(() => {
                if (!document.body.contains(this._scrollPreventElem))
                    return;
                hidePreventElem();
            }, 2000);
            [].forEach.call(document.querySelectorAll(".laji-map-scroll-prevent"), elem => elem.style.display = "block");
        };
        const hidePreventElem = () => {
            if (!this._showingPreventScroll)
                return;
            clearTimeout(this._showPreventHideTimeout);
            clearTimeout(this._showPreventShowTimeout);
            clearTimeout(this._showPreventAnimationTimeout);
            [].forEach.call(document.querySelectorAll(".laji-map-scroll-prevent"), elem => elem.className = `${elem.className} leaving`);
            this._showPreventAnimationTimeout = setTimeout(() => {
                this._showingPreventScroll = false;
                [].forEach.call(document.querySelectorAll(".laji-map-scroll-prevent"), elem => elem.className = elem.className.replace("leaving", "left"));
                this._startPreventScrollingTimeout();
            }, 200); // Should match transition time in css
        };
        const _onTouchOrMouseEventAgnostic = (isOutside) => {
            this._startPreventScrollingTimeout();
            if (!this._preventScroll && isOutside) {
                this._preventScrolling();
            }
            else if (this._preventScroll && !isOutside && !this.viewLocked) {
                this.map.smoothWheelZoom.enable();
                this.map.dragging.enable();
                hidePreventElem();
                this._preventScroll = false;
                return true;
            }
        };
        const onTouchOrMouse = (touch) => (e) => {
            if (touch)
                e.stopPropagation();
            const enabled = _onTouchOrMouseEventAgnostic(isOutsideLajiMap(e.target));
            if (enabled && !touch) {
                this.map.dragging._draggable._onDown(e);
            }
        };
        if (!this._onTouchPreventScrolling) {
            this._onTouchPreventScrolling = onTouchOrMouse(!!"touch");
            this._onMouseDownPreventScrolling = onTouchOrMouse(!"mouse");
            this._onControlClickPreventScrolling = () => _onTouchOrMouseEventAgnostic(false);
            this._onDrawStartPreventScrolling = () => !this.viewLocked && this.map.dragging.disable();
            this._onDrawStopPreventScrolling = () => !this.viewLocked && this.map.dragging.enable();
        }
        if (this._preventScrollDomCleaner) {
            this._removeDomCleaner(this._preventScrollDomCleaner);
        }
        this._preventScrollDomCleaner = () => {
            if (this._preventScrollDomCleaner) {
                this._removeDomCleaner(this._preventScrollDomCleaner);
            }
            this._preventScrollDomCleaner = undefined;
            clearTimeout(this._showPreventHideTimeout);
            clearTimeout(this._showPreventShowTimeout);
            clearTimeout(this._showPreventAnimationTimeout);
            document.removeEventListener("touch", this._onTouchPreventScrolling);
            document.removeEventListener("mousedown", this._onMouseDownPreventScrolling);
            this.map.removeEventListener("zoomstart", this._startPreventScrollingTimeout);
            this.map.removeEventListener("controlClick", this._onControlClickPreventScrolling);
            this.map.removeEventListener("draw:drawstart", this._onDrawStartPreventScrolling);
            this.map.removeEventListener("draw:drawstop", this._onDrawStopPreventScrolling);
            const scrollPreventElemParent = this._scrollPreventElem.parentNode;
            if (scrollPreventElemParent)
                scrollPreventElemParent.removeChild(this._scrollPreventElem);
            this._scrollPreventElem = undefined;
            const scrollPreventTextElemContainerParent = this._scrollPreventTextElemContainer.parentNode;
            if (scrollPreventTextElemContainerParent) {
                scrollPreventTextElemContainerParent.removeChild(this._scrollPreventTextElemContainer);
            }
            this._scrollPreventTextElemContainer = undefined;
            (this._scrollPreventScrollListeners || []).forEach(([name, fn]) => window.removeEventListener(name, fn));
            this._scrollPreventScrollListeners = undefined;
            !this.viewLocked && this.map.smoothWheelZoom.enable();
            !this.viewLocked && this.map.dragging.enable();
        };
        if (value && !valueWas) {
            this._preventScrolling();
            document.addEventListener("touch", this._onTouchPreventScrolling);
            document.addEventListener("mousedown", this._onMouseDownPreventScrolling);
            this.map.addEventListener("controlClick", this._onControlClickPreventScrolling);
            this._scrollPreventScrollListeners = [];
            "wheel touchstart".split(" ").forEach(eventName => {
                const eventListener = (e) => {
                    const coordinatesSource = eventName === "touchstart" ? e.touches[0] : e;
                    const pointedElem = document.elementFromPoint(coordinatesSource.clientX, coordinatesSource.clientY);
                    const isOutside = isOutsideLajiMap(pointedElem);
                    if (this._preventScroll && !isOutside) {
                        this.removeTranslationHook(translationHook);
                        translationHook = this.addTranslationHook(this._scrollPreventTextElem, `ClickBefore${eventName === "wheel" ? "Zoom" : "Pan"}`);
                        showPreventElem();
                    }
                    else {
                        this._startPreventScrollingTimeout();
                    }
                };
                window.addEventListener(eventName, eventListener);
                this._scrollPreventScrollListeners.push([eventName, eventListener]);
                this.map.addEventListener("zoomstart", this._startPreventScrollingTimeout);
            });
            this.container.appendChild(this._scrollPreventElem);
            this.container.appendChild(this._scrollPreventTextElemContainer);
            this.map.addEventListener("draw:drawstart", this._onDrawStartPreventScrolling);
            this.map.addEventListener("draw:drawstop", this._onDrawStopPreventScrolling);
            this._addDomCleaner(this._preventScrollDomCleaner);
        }
        else if (!value && valueWas) {
            if (this._preventScrollDomCleaner)
                this._preventScrollDomCleaner();
        }
    }
    getMMLProj() {
        return this.mmlProj;
    }
    _initializeMap() {
        try {
            if (!dependency_utils_1.depsProvided(this, "_initializeMap", arguments))
                return;
            L.Icon.Default.imagePath = "https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/images/";
            this.map = L.map(this.mapElem, Object.assign({ contextmenu: true, contextmenuItems: [], zoomControl: false, attributionControl: false, doubleClickZoom: false, zoomSnap: 0, maxZoom: 19, zoom: this.zoom, center: this.center, scrollWheelZoom: false, smoothWheelZoom: true, smoothSensitivity: 3 }, this.leafletOptions));
            this.mmlProj = new L.Proj.CRS("EPSG:3067", globals_1.EPSG3067String, {
                origin: [-548576, 8388608],
                bounds: L.bounds([-548576, 8388608], [1548576, 6291456]),
                resolutions: [
                    8192, 4096, 2048, 1024, 512, 256,
                    128, 64, 32, 16, 8, 4, 2, 1, 0.5,
                    0.25, 0.125, 0.0625, 0.03125, 0.015625
                ],
            });
            // Scale controller won't work without this hack.
            // Fixes also circle projection.
            this.mmlProj.distance = L.CRS.Earth.distance;
            this.mmlProj.R = 6378137;
            const getAttribution = (link, text) => `<a href="${link}" target="_blank" rel="noopener noreferrer" tabindex="-1">&copy; ${text}</a>`;
            const mmlAttribution = getAttribution("https://www.maanmittauslaitos.fi/avoindata_lisenssi_versio1_20120501", "Maanmittauslaitos");
            const sykeAttribution = getAttribution("https://www.syke.fi/fi-FI/Avoin_tieto/Kayttolupa_ja_vastuut", "SYKE");
            const lukeAttribution = getAttribution("https://kartta.luke.fi/", "LUKE");
            const getMMLLayer = (layerService) => (name, options = { format: "png" }) => L.tileLayer(`https://proxy.laji.fi/mml_wmts/${layerService}/wmts/1.0.0/${name}/default/ETRS-TM35FIN/{z}/{y}/{x}.${options.format}`, Object.assign(Object.assign({}, options), { minZoom: 0, maxZoom: 15, attribution: mmlAttribution }));
            const getMaastoLayer = getMMLLayer("maasto");
            const createLayer = (address) => (options, nonTiled = false) => {
                const constructor = nonTiled ? NonTiledLayer.WMS : L.TileLayer.WMS;
                const _options = Object.assign({ maxZoom: 15, format: "image/png", transparent: true }, options);
                if (_options) {
                    _options.useCanvas = false;
                }
                return new constructor(`${address}/ows`, _options);
            };
            const createLajiLayer = createLayer(this.lajiGeoServerAddress);
            const getTilastokeskusLayer = (layer) => L.tileLayer.wms("https://geo.stat.fi/geoserver/tilastointialueet/wms", {
                layers: layer,
                transparent: true,
                format: "image/png",
                attribution: getAttribution("https://www.stat.fi/org/lainsaadanto/copyright.html", "Tilastokeskus")
            });
            this.finnishTileLayers = {
                taustakartta: getMaastoLayer("taustakartta"),
                maastokartta: getMaastoLayer("maastokartta"),
                ortokuva: getMaastoLayer("ortokuva", { format: "jpg", maxZoom: 14 }),
                laser: L.tileLayer("https://wmts.mapant.fi/wmts.php?z={z}&x={x}&y={y}", {
                    maxZoom: 19,
                    minZoom: 0,
                    tileSize: 256,
                    attribution: `${mmlAttribution}, ${getAttribution("https://www.mapant.fi", "Mapant")}`
                }),
                ykjGrid: createLajiLayer({
                    layers: "LajiMapData:YKJlines100,LajiMapData:YKJlines1000,LajiMapData:YKJlines10000,LajiMapData:YKJlines100000",
                }),
                ykjGridLabels: createLajiLayer({
                    layers: "LajiMapData:YKJlabels1000,LajiMapData:YKJlabels10000,LajiMapData:YKJlabels100000",
                }, true),
                atlasGrid: L.layerGroup([
                    createLajiLayer({
                        layers: "LajiMapData:AtlasYKJ1kmLines,LajiMapData:AtlasYKJ10kmLines",
                    }),
                    createLajiLayer({
                        layers: "LajiMapData:AtlasYKJ1kmLabels,LajiMapData:AtlasYKJ10kmLabels",
                    }, true)
                ]),
                afeGrid: createLajiLayer({
                    layers: "LajiMapData:afe_grid"
                }),
            };
            this.worldTileLayers = {
                openStreetMap: L.tileLayer("https://proxy.laji.fi/osm/{z}/{x}/{y}.png", {
                    attribution: getAttribution("https://osm.org/copyright", "OpenStreetMap contributors")
                }),
                googleSatellite: L.tileLayer("https://{s}.google.com/vt/lyrs=s&x={x}&y={y}&z={z}", {
                    subdomains: ["mt0", "mt1", "mt2", "mt3"],
                    attribution: getAttribution("https://developers.google.com/maps/terms", "Google")
                }),
                cgrsGrid: createLajiLayer({
                    layers: "LajiMapData:cgrs_grid"
                })
            };
            this.tileLayers = Object.assign(Object.assign({}, this.finnishTileLayers), this.worldTileLayers);
            this.availableTileLayers = this.tileLayers;
            this.overlaysByNames = {
                geobiologicalProvinces: createLajiLayer({
                    layers: "LajiMapData:biogeographical_provinces",
                    version: "1.3.0"
                }),
                geobiologicalProvincesBorders: createLajiLayer({
                    layers: "LajiMapData:biogeographical_provinces_borders",
                    version: "1.3.0"
                }),
                municipalities: getTilastokeskusLayer("kunta1000k"),
                counties: getTilastokeskusLayer("maakunta1000k"),
                ely: getTilastokeskusLayer("ely1000k"),
                // eslint-disable-next-line max-len
                forestVegetationZones: createLayer("https://paikkatieto.ymparisto.fi/arcgis/services/INSPIRE/SYKE_EliomaantieteellisetAlueet/MapServer/WmsServer")({
                    layers: "br.Metsakasvillisuusvyohykkeet",
                    defaultOpacity: 0.5,
                    attribution: sykeAttribution
                }),
                // eslint-disable-next-line max-len
                mireVegetationZones: createLayer("https://paikkatieto.ymparisto.fi/arcgis/services/INSPIRE/SYKE_EliomaantieteellisetAlueet/MapServer/WmsServer")({
                    layers: "br.Suokasvillisuusvyohykkeet",
                    defaultOpacity: 0.5,
                    attribution: sykeAttribution
                }),
                threatenedSpeciesEvaluationZones: createLajiLayer({
                    layers: "LajiMapData:threatened_species_evaluation_zones",
                    version: "1.1.0",
                    defaultOpacity: 0.5,
                    attribution: sykeAttribution
                }),
                // eslint-disable-next-line max-len
                biodiversityForestZones: createLayer("https://paikkatieto.ymparisto.fi/arcgis/services/SYKE/SYKE_MonimuotoisuudelleTarkeatMetsaalueetZonation/MapServer/WmsServer")({
                    layers: "8",
                    defaultOpacity: 0.5,
                    attribution: sykeAttribution
                }),
                habitat: createLayer("https://kartta.luke.fi/geoserver/ows")({
                    layers: "MVMI:kasvupaikka_1519",
                    defaultOpacity: 0.5,
                    attribution: lukeAttribution
                }),
                ageOfTrees: createLayer("https://kartta.luke.fi/geoserver/ows")({
                    layers: "MVMI:ika_1519",
                    defaultOpacity: 0.5,
                    attribution: lukeAttribution
                }),
                kiinteistojaotus: getMMLLayer("kiinteisto")("kiinteistojaotus"),
                kiinteistotunnukset: getMMLLayer("kiinteisto")("kiinteistotunnukset"),
                currentProtectedAreas: L.layerGroup([
                    createLayer("https://paikkatiedot.ymparisto.fi/geoserver/inspire_ps/wms")({
                        layers: [
                            "PS.ProtectedSitesValtionOmistamaLuonnonsuojelualue",
                            "PS.ProtectedSitesYksityistenMaillaOlevaLuonnonsuojelualue",
                            "PS.ProtectedSitesEramaaAlue",
                            "PS.ProtectedSitesProposedSiteOfCommunityImportance",
                            "PS.ProtectedSitesSpecialAreaOfConservation",
                            "PS.ProtectedSitesSpecialProtectionArea"
                        ].join(","),
                        attribution: sykeAttribution,
                        styles: [
                            "PS.ValtionMaidenSuojelualueet.Luokiteltu",
                            "PS.YksityisetSuojelualueet.Luokiteltu",
                            "PS.EramaaAlueet",
                            "PS.Natura2000SCI",
                            "PS.Natura2000SAC",
                            "PS.Natura2000SPA"
                        ].join(",")
                    }),
                    createLajiLayer({
                        layers: "ProtectedAreas:protectedArea_labels",
                    }, true)
                ], { defaultOpacity: 0.5 }),
                plannedProtectedAreas: L.layerGroup([
                    createLajiLayer({
                        layers: "ProtectedAreas:otherProtectedAreas",
                    }, true),
                    createLayer("https://paikkatiedot.ymparisto.fi/geoserver/syke_luonnonsuojeluohjelma_alueet/wms")({
                        layers: "syke_luonnonsuojeluohjelma_alueet:Luonnonsuojeluohjelmaalueet",
                        attribution: sykeAttribution,
                        styles: "lsohjelma_alueet_luokiteltu"
                    })
                ], { defaultOpacity: 0.5 }),
                flyingSquirrelPredictionModel: createLajiLayer({
                    layers: "LajiMapData:flyingSquirrel_predictionModel",
                    defaultOpacity: 0.5
                }),
                birdAtlasSocietyGridZones: L.layerGroup([
                    createLajiLayer({
                        layers: "LajiMapData:BirdAtlasSocietyGrid",
                    }),
                    createLajiLayer({
                        layers: "LajiMapData:BirdAtlasSocietyGridLabels",
                    }, true),
                    createLajiLayer({
                        layers: "LajiMapData:BirdLifeSocieties",
                    }),
                    createLajiLayer({
                        layers: "LajiMapData:BirdLifeSocietyLabels",
                    }, true)
                ], { defaultOpacity: 0.5 }),
                barentsRegion: createLajiLayer({
                    layers: "LajiMapData:barentsRegion2"
                }),
            };
            const combined = Object.assign(Object.assign({}, this.tileLayers), this.overlaysByNames);
            Object.keys(combined).forEach(name => {
                combined[name].setZIndex(this._tileLayerOrder.indexOf(name));
            });
            this.availableOverlaysByNames = this.overlaysByNames;
            if (this.locateOptions && this.locateOptions.on) {
                this._setLocateOn();
            }
            this._initializeMapEvents();
            dependency_utils_1.provide(this, "map");
        }
        catch (e) {
            if (e._lajiMapError) {
                this._showError(e);
            }
            else {
                throw e;
            }
        }
    }
    _initializeView() {
        if (!dependency_utils_1.depsProvided(this, "_initializeView", arguments))
            return;
        this._viewCriticalSection++;
        let tileLayerOptions;
        if (this.options.tileLayers) {
            tileLayerOptions = this.options.tileLayers;
        }
        else if (this.options.tileLayerName) {
            tileLayerOptions = { layers: { [this.options.tileLayerName]: true } };
        }
        else {
            tileLayerOptions = { layers: { [DEFAULT_LAYER_NAME]: true } };
        }
        tileLayerOptions = this._getInternalTileLayerOptions(tileLayerOptions);
        let center, bounds;
        if (this.options.zoomToData) {
            bounds = this.getBoundsForZoomToDataOptions(this.options.zoomToData);
            if (bounds.isValid()) {
                center = bounds.getCenter();
            }
            else {
                bounds = undefined;
            }
        }
        if (!center) {
            center = this.options.center;
        }
        if (tileLayerOptions.active === "finnish" && this._isOutsideFinland(center)) {
            tileLayerOptions.active = "world";
        }
        this._trySwapToFinnishOnInitialization = this._isOutsideFinland(center);
        this.setTileLayers(tileLayerOptions);
        if (bounds) {
            this.zoomToData(this.options.zoomToData);
        }
        else {
            this.map.setView(this.center, this.getDenormalizedZoom(this.zoom), { animate: false });
        }
        dependency_utils_1.provide(this, "view");
        this.reorderData();
        this._viewCriticalSection--;
    }
    _isOutsideFinland(latLng) {
        return !L.latLngBounds(globals_1.FINLAND_BOUNDS).contains(latLng);
    }
    _swapToWorldOutsideFinland(latLng) {
        if (!dependency_utils_1.depsProvided(this, "_swapToWorldOutsideFinland", arguments))
            return;
        if (Object.keys(this.worldTileLayers).some(name => !!this.availableTileLayers[name])
            && this._isOutsideFinland(latLng)) {
            this._swapToWorldFlag = true;
            let options;
            const someWorldMapVisible = Object.keys(this.worldTileLayers).some(name => !!this._tileLayers.layers[name].visible);
            if (someWorldMapVisible) {
                options = Object.assign(Object.assign({}, this._tileLayers), { active: "world" });
            }
            else {
                options = Object.assign(Object.assign({}, this._tileLayers), { layers: Object.assign(Object.assign({}, this._tileLayers.layers), { openStreetMap: true }), active: "world" });
            }
            this.setTileLayers(options);
        }
    }
    _initializeMapEvents() {
        if (!dependency_utils_1.depsProvided(this, "_initializeMapEvents", arguments))
            return;
        this.map.addEventListener({
            // We have to handle dblclick zoom manually, since the default event can't be cancelled.
            "dblclick": (e) => {
                setTimeout(() => {
                    if (this._disableDblClickZoom)
                        return;
                    const oldZoom = this.map.getZoom();
                    const delta = this.map.options.zoomDelta;
                    const zoom = e.originalEvent.shiftKey ? oldZoom - delta : oldZoom + delta;
                    this.map.setZoomAround(e.containerPoint, zoom);
                });
            },
            "click": () => this._interceptClick(),
            "mousemove": ({ latlng }) => {
                this._mouseLatLng = latlng;
            },
            "draw:created": ({ layer }) => this._onAdd(this.drawIdx, layer),
            "draw:drawstart": () => {
                this.drawing = true;
                this.map.fire("controlClick", { name: "draw" });
            },
            "draw:drawstop": () => { this.drawing = false; },
            "draw:drawvertex": (e) => {
                const layers = e.layers.getLayers();
                const keys = Object.keys(layers);
                const latlng = layers[keys[keys.length - 1]].getLatLng();
                const { x, y } = this.map.latLngToContainerPoint(latlng);
                const { width, height } = this.rootElem.getBoundingClientRect();
                const treshold = Math.min(width, height) / 4;
                if ([y, y - height, x, x - width].some(dist => Math.abs(dist) < treshold)) {
                    this.map.panTo(latlng);
                }
            },
            "locationfound": (e) => this._onLocationFound(e),
            "locationerror": (e) => this._onLocationNotFound(e),
            "contextmenu.show": (e) => {
                if (e.relatedTarget) {
                    this._contextMenuLayer = e.relatedTarget;
                    const tuple = this._getIdxTupleByLayer(this._contextMenuLayer);
                    if (tuple) {
                        const [dataIdx, featureIdx] = tuple;
                        if (this.data[dataIdx] && this.data[dataIdx].editable) {
                            this._idxsToContextMenuOpen[dataIdx][featureIdx] = true;
                            this.updateLayerStyle(e.relatedTarget);
                        }
                    }
                }
                this._interceptClick();
            },
            "contextmenu.hide": () => {
                const contextMenuLayer = this._contextMenuLayer;
                this._contextMenuLayer = undefined;
                if (!contextMenuLayer)
                    return;
                const tuple = this._getIdxTupleByLayer(contextMenuLayer);
                if (tuple) {
                    const [dataIdx, featureIdx] = tuple;
                    if (this.data[dataIdx] && this.data[dataIdx].editable) {
                        this._idxsToContextMenuOpen[dataIdx][featureIdx] = false;
                        this.updateLayerStyle(contextMenuLayer);
                    }
                }
                this.map.fire("mousemove", { latlng: this._mouseLatLng });
            },
            "moveend": () => {
                if (!this.map) {
                    return;
                }
                if (this._viewCriticalSection) {
                    return;
                }
                if (this._swapToWorldFlag) {
                    this._swapToWorldFlag = false;
                    return;
                }
                this._swapToWorldOutsideFinland(this.map.getCenter());
            }
        });
        this._addDocumentEventListener("click", (e) => {
            if (e.target !== this.rootElem && !this.rootElem.contains(e.target)) {
                this._interceptClick();
            }
        }, true); // useCapture flag to make sure that this runs before other targets beneath in the DOM tree.
        this._addDocumentEventListener("keydown", e => this.keyHandlerForType("keydown", e));
        this._addDocumentEventListener("keyup", e => this.keyHandlerForType("keyup", e));
    }
    keyHandlerForType(type, e) {
        e = e || window.event;
        let key = "key" in e ? e.key : e.keyCode;
        const isEscape = ["Escape", "Esc", 27].includes(key);
        if (isEscape) {
            key = globals_1.ESC;
        }
        if (this._triggerKeyEvent(key.toLowerCase(), e, type)) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            return true;
        }
    }
    _addDocumentEventListener(type, fn, useCapture) {
        if (!this._documentEvents[type]) {
            this._documentEvents[type] = [];
        }
        this._documentEvents[type].push(fn);
        document.addEventListener(type, fn, useCapture);
    }
    _removeDocumentEventListener(type, fn) {
        if (!this._documentEvents[type]) {
            return;
        }
        this._documentEvents[type] = this._documentEvents[type].filter(_fn => _fn !== fn);
        document.removeEventListener(type, fn);
    }
    _addKeyListener(key, fn, type = "keydown") {
        if (!this._keyListeners[type])
            this._keyListeners[type] = {};
        if (!this._keyListeners[type][key])
            this._keyListeners[type][key] = [];
        this._keyListeners[type][key] = [fn, ...this._keyListeners[type][key]];
    }
    _removeKeyListener(key, fn, type = "keydown") {
        var _a, _b;
        const index = (_b = (_a = this._keyListeners[type]) === null || _a === void 0 ? void 0 : _a[key]) === null || _b === void 0 ? void 0 : _b.indexOf(fn);
        if (index >= 0) {
            this._keyListeners[type][key].splice(index, 1);
        }
    }
    _triggerKeyEvent(key, e, type) {
        var _a;
        for (let fn of ((_a = this._keyListeners[type]) === null || _a === void 0 ? void 0 : _a[key]) || []) {
            const result = fn(e);
            if (result !== false) {
                return true;
            }
        }
    }
    setEventListeners(eventListeners) {
        if (!dependency_utils_1.depsProvided(this, "setEventListeners", arguments))
            return;
        if (!eventListeners)
            return;
        Object.keys(this._listenedEvents || {}).forEach(name => {
            this.map.removeEventListener(name, this._listenedEvents[name]);
        });
        this._listenedEvents = eventListeners;
        this.map.addEventListener(eventListeners);
    }
    _getDefaultCRSLayers() {
        return [
            this.tileLayers.openStreetMap,
            this.tileLayers.googleSatellite,
            this.tileLayers.cgrsGrid
        ];
    }
    _getMMLCRSLayers() {
        return [
            this.tileLayers.maastokartta,
            this.tileLayers.taustakartta,
            this.tileLayers.ortokuva,
            this.tileLayers.laser,
            this.tileLayers.afeGrid,
        ];
    }
    setTileLayerByName(name) {
        if (!dependency_utils_1.depsProvided(this, "setTileLayerByName", arguments))
            return;
        if (!this._initialized && this._tileLayersSet) {
            return;
        }
        this.setTileLayer(this.tileLayers[name]);
    }
    getListForAvailableLayers(names, condition, _tileLayers) {
        const list = names.reduce((_list, name) => {
            _list[name] = true;
            return _list;
        }, {});
        const layers = _tileLayers || this.tileLayers;
        return Object.keys(layers).reduce((tileLayers, name) => {
            if (name in list === condition)
                tileLayers[name] = layers[name];
            return tileLayers;
        }, {});
    }
    setAvailableTileLayers(names, condition, _tileLayers) {
        if (!dependency_utils_1.depsProvided(this, "setAvailableTileLayers", arguments))
            return;
        this.availableTileLayers = this.getListForAvailableLayers(names, condition, _tileLayers);
        !dependency_utils_1.isProvided(this, "availableTileLayers") && dependency_utils_1.provide(this, "availableTileLayers");
        dependency_utils_1.isProvided(this, "tileLayer") && this.setTileLayers(this._tileLayers);
    }
    getDefaultTileLayerBlacklist() { return []; }
    getDefaultTileLayerWhitelist() { return Object.keys(this.tileLayers); }
    setAvailableTileLayerBlacklist(names) {
        if (!dependency_utils_1.depsProvided(this, "setAvailableTileLayerBlacklist", arguments))
            return;
        if (!names) {
            names = this.getDefaultTileLayerBlacklist();
        }
        this.availableTileLayersBlacklist = names;
        const layers = this.getListForAvailableLayers(this.availableTileLayersWhitelist || this.getDefaultTileLayerWhitelist(), true);
        this.setAvailableTileLayers(names, false, layers);
    }
    setAvailableTileLayerWhitelist(names) {
        if (!dependency_utils_1.depsProvided(this, "setAvailableTileLayerWhitelist", arguments))
            return;
        if (!names) {
            names = this.getDefaultTileLayerWhitelist();
        }
        this.availableTileLayersWhitelist = names;
        const layers = this.getListForAvailableLayers(this.availableTileLayersBlacklist || this.getDefaultTileLayerBlacklist(), false);
        this.setAvailableTileLayers(names, true, layers);
    }
    getAvailableOverlaysByNames() {
        return this.getAvailableLayersFor(this.overlaysByNames, this.availableOverlaysByNames);
    }
    getAvailableWorldTileLayers() {
        return this.getAvailableLayersFor(this.worldTileLayers, this.availableTileLayers);
    }
    getAvailableFinnishTileLayers() {
        return this.getAvailableLayersFor(this.finnishTileLayers, this.availableTileLayers);
    }
    getAvailableLayersFor(_layers, availables) {
        return Object.keys(_layers).reduce((layers, name) => {
            if (availables[name]) {
                layers[name] = _layers[name];
            }
            return layers;
        }, {});
    }
    setTileLayer(layer) {
        const name = Object.keys(this.tileLayers).find(_name => {
            if (this.tileLayers[_name] === layer) {
                return !!_name;
            }
        });
        const overlays = this.getOverlaysByName().reduce((_overlays, _name) => {
            _overlays[_name] = this._tileLayers.layers[_name];
            return _overlays;
        }, {});
        this.setTileLayers({ layers: Object.assign({ [name]: { opacity: 1, visible: true } }, overlays) });
    }
    getActiveLayers(options) {
        let { layers, active } = options;
        if (!active) {
            let useFinnishProj;
            let foundMultiple = false;
            Object.keys(layers).some(name => {
                if (foundMultiple) {
                    return true;
                }
                const layerOptions = layers[name];
                if (!layerOptions.visible) {
                    return;
                }
                if (useFinnishProj === undefined) {
                    useFinnishProj = this.finnishTileLayers[name];
                }
                else if (useFinnishProj && this.worldTileLayers[name] || !useFinnishProj && !this.finnishTileLayers[name]) {
                    useFinnishProj = true;
                    foundMultiple = true;
                }
            });
            active = useFinnishProj ? "finnish" : "world";
        }
        return Object.keys(layers).reduce((_layers, name) => {
            if (layers[name].visible
                && (this.overlaysByNames[name]
                    || active === "finnish" && this.finnishTileLayers[name]
                    || active === "world" && this.worldTileLayers[name])) {
                _layers[name] = layers[name];
            }
            return _layers;
        }, {});
    }
    _getInternalTileLayerOptions(options) {
        const combinedLayers = Object.assign(Object.assign({}, this.availableTileLayers), this.availableOverlaysByNames);
        const newOptions = Object.assign(Object.assign({}, options), { layers: Object.keys(combinedLayers).reduce((_layers, name) => {
                const layerOptions = options.layers[name];
                _layers[name] = typeof layerOptions === "boolean" || layerOptions === undefined
                    ? { opacity: layerOptions ? (combinedLayers[name].options.defaultOpacity || 1) : 0, visible: !!layerOptions }
                    : {
                        opacity: layerOptions.opacity,
                        visible: layerOptions.hasOwnProperty("visible") ? layerOptions.visible : !!layerOptions.opacity
                    };
                return _layers;
            }, {}) });
        const activeLayers = this.getActiveLayers(newOptions);
        newOptions.active = options.active
            || (Object.keys(activeLayers).length === 0 || this.finnishTileLayers[Object.keys(activeLayers)[0]]
                ? "finnish"
                : "world");
        return newOptions;
    }
    setTileLayers(options) {
        this._tileLayersSet = true;
        if (!dependency_utils_1.depsProvided(this, "setTileLayers", arguments))
            return;
        const newOptions = this._getInternalTileLayerOptions(options);
        const defaultCRSLayers = this._getDefaultCRSLayers();
        const mmlCRSLayers = this._getMMLCRSLayers();
        const combinedLayers = Object.assign(Object.assign({}, this.availableTileLayers), this.availableOverlaysByNames);
        const prevActiveLayers = this._tileLayers && this.getActiveLayers(this._tileLayers);
        const activeLayers = this.getActiveLayers(newOptions);
        const oldActive = this.activeProjName;
        newOptions.active = options.active
            || (Object.keys(activeLayers).length === 0 || this.finnishTileLayers[Object.keys(activeLayers)[0]]
                ? "finnish"
                : "world");
        const findNonOverlay = name => !!this.finnishTileLayers[name] || !!this.worldTileLayers[name];
        const layer = this.tileLayers[Object.keys(activeLayers).find(findNonOverlay)];
        // For BW compatibility.
        this.tileLayer = layer;
        const existingLayer = this._tileLayers && this.tileLayers[Object.keys(this.getActiveLayers(this._tileLayers)).find(findNonOverlay)];
        const center = this.map.getCenter();
        this.map.options.crs = defaultCRSLayers.indexOf(layer) !== -1 ? L.CRS.EPSG3857 : this.mmlProj;
        let zoom = this.map.getZoom();
        if (this.activeProjName && newOptions.active !== this.activeProjName
            && this.activeProjName !== "finnish"
            && (!layer
                || (mmlCRSLayers.indexOf(layer) !== -1 && mmlCRSLayers.indexOf(existingLayer) === -1))) {
            if (dependency_utils_1.isProvided(this, "tileLayer")) {
                zoom = zoom - 3;
            }
        }
        else if (newOptions.active !== this.activeProjName
            && this.activeProjName !== "world"
            && (!layer
                || (defaultCRSLayers.indexOf(layer) !== -1 && defaultCRSLayers.indexOf(existingLayer) === -1))) {
            zoom = zoom + 3;
        }
        this.activeProjName = newOptions.active;
        const prevOptions = this._tileLayers;
        this._tileLayers = newOptions;
        const changedLayers = Object.keys(this._tileLayers.layers).reduce((changed, name) => {
            const activeLayer = activeLayers[name] || { visible: false, opacity: 0 };
            const prevActiveLayer = (prevActiveLayers || {})[name] || { visible: false, opacity: 0 };
            if (!prevActiveLayers
                || activeLayer.visible !== prevActiveLayer.visible
                || activeLayer.opacity !== prevActiveLayer.opacity
                || (oldActive && oldActive !== newOptions.active && this.overlaysByNames[name])) {
                changed[name] = true;
            }
            return changed;
        }, {});
        Object.keys(changedLayers).forEach(name => {
            const _layer = combinedLayers[name];
            const { opacity, visible } = this._tileLayers.layers[name];
            prevOptions && (!visible || !activeLayers[name]) && this.map.hasLayer(_layer) && _layer.remove();
            // Overlays must be reapplied if projection changed.
            oldActive && oldActive !== newOptions.active && this.overlaysByNames[name] && this.map.removeLayer(_layer);
            if (visible && activeLayers[name]) {
                !this.map.hasLayer(_layer) && this.map.addLayer(_layer);
                isMultiTileLayer(_layer)
                    ? _layer.eachLayer((l) => l.setOpacity(opacity))
                    : _layer.setOpacity(opacity);
            }
        });
        // Zoom levels behave differently on different projections.
        // We bypass the default max zoom behaviour and handle it manually
        // below.
        const maxZoom = Object.keys(activeLayers).reduce((_maxZoom, name) => {
            if (!activeLayers[name]) {
                return;
            }
            const _layer = this.tileLayers[name];
            return _layer && _layer.options.maxZoom < _maxZoom
                ? _layer.options.maxZoom
                : _maxZoom;
        }, 19);
        this.map.setMaxZoom(19);
        if (this.activeProjName !== oldActive) {
            // Prevent moveend event triggering layer swap, since view reset below must be ran sequentially.
            this._viewCriticalSection++;
            // Redraw all layers according to new projection.
            this.map._resetView(this.map.getCenter(), this.map.getZoom(), true); // lähettää ekan
            this.map.setView(center, zoom, { animate: false });
            this.recluster();
            this._viewCriticalSection--;
            this.map.fire("projectionChange", newOptions.active);
        }
        this.map.setMaxZoom(maxZoom);
        let currentLayerName = undefined;
        for (let tileLayerName in this.tileLayers) {
            if (this.tileLayer === this.tileLayers[tileLayerName]) {
                currentLayerName = tileLayerName;
                this.tileLayerName = currentLayerName;
            }
        }
        if (dependency_utils_1.isProvided(this, "tileLayer")) {
            this.map.fire("tileLayerChange", { tileLayerName: currentLayerName });
            this.map.fire("tileLayersChange", { tileLayers: this._tileLayers });
            this.map.fire("overlaysChange", { overlayNames: this.getOverlaysByName() });
        }
        // We need to provide before swapping, or else swapping causes callback loop since it might call this function.
        const _isProvided = dependency_utils_1.isProvided(this, "tileLayer");
        if (!_isProvided) {
            dependency_utils_1.provide(this, "tileLayer");
            this._trySwapToFinnishOnInitialization && this._swapToWorldOutsideFinland(this.map.getCenter());
        }
    }
    getTileLayers() {
        return this._tileLayers;
    }
    setTileLayerOpacity(val, triggerEvent = true) {
        if (!dependency_utils_1.depsProvided(this, "setTileLayerOpacity", arguments))
            return;
        if (val === undefined)
            return;
        if (!this._initialized && this.options.tileLayers) {
            return;
        }
        let initialCall = this.tileLayerOpacity === undefined;
        this.tileLayerOpacity = val;
        if (this.tileLayerName) {
            this.setTileLayers(Object.assign(Object.assign({}, this._tileLayers), { layers: Object.assign(Object.assign({}, this._tileLayers.layers), { [this.tileLayerName]: Object.assign(Object.assign({}, this._tileLayers.layers[this.tileLayerName]), { opacity: val }) }) }));
        }
        if (!initialCall && triggerEvent)
            this.map.fire("tileLayerOpacityChange", { tileLayerOpacity: val });
    }
    setOverlays(overlays = []) {
        if (!this._initialized && this.options.tileLayers) {
            return;
        }
        const bwCompatibleOverlays = Object.assign(Object.assign({}, this.overlaysByNames), { ykjGrid: this.tileLayers.ykjGrid, ykjGridLabels: this.tileLayers.ykjGridLabels });
        const uniq = (layer) => layer._url + JSON.stringify(layer.options) + JSON.stringify(layer.wmsParams);
        const urlsToNames = Object.keys(bwCompatibleOverlays).reduce((_map, name) => {
            _map[uniq(bwCompatibleOverlays[name])] = name;
            return _map;
        }, {});
        const names = overlays.reduce((_names, layer) => {
            _names[urlsToNames[uniq(layer)]] = true;
            return _names;
        }, {});
        const changes = Object.keys(bwCompatibleOverlays).reduce((_names, name) => {
            _names[name] = { visible: !!names[name], opacity: names[name] ? bwCompatibleOverlays[name].defaultOpacity || 1 : 0 };
            return _names;
        }, {});
        this.setTileLayers(Object.assign(Object.assign({}, this._tileLayers), { layers: Object.assign(Object.assign({}, this._tileLayers.layers), changes) }));
        dependency_utils_1.provide(this, "overlays");
    }
    // Wrapper that prevents overlay event triggering on initial call.
    _setOverlaysByName(overlayNames) {
        this.setOverlaysByName(overlayNames);
    }
    getOverlaysByName() {
        return this._tileLayers ? Object.keys(this._tileLayers.layers).reduce((names, name) => {
            if (!this.overlaysByNames[name]) {
                return names;
            }
            const { visible, opacity } = this._tileLayers.layers[name];
            if (visible && opacity) {
                names.push(name);
            }
            return names;
        }, []) : [];
    }
    setOverlaysByName(overlayNames = []) {
        if (!dependency_utils_1.depsProvided(this, "setOverlaysByName", arguments))
            return;
        this.setOverlays(overlayNames.map(name => this.overlaysByNames[name] || this.tileLayers[name]));
    }
    getDefaultOverlaysBlacklist() { return []; }
    getDefaultOverlaysWhitelist() { return Object.keys(this.overlaysByNames); }
    setAvailableOverlaysBlacklist(overlayNames) {
        if (!dependency_utils_1.depsProvided(this, "setAvailableOverlaysBlacklist", arguments))
            return;
        if (!overlayNames) {
            overlayNames = this.getDefaultOverlaysBlacklist();
        }
        this.availableOverlaysBlacklist = overlayNames;
        const layers = this.getListForAvailableLayers(this.availableOverlaysWhitelist || this.getDefaultOverlaysWhitelist(), true, this.overlaysByNames);
        this.setAvailableOverlays(overlayNames, false, layers);
    }
    setAvailableOverlaysWhitelist(overlayNames) {
        if (!dependency_utils_1.depsProvided(this, "setAvailableOverlaysWhitelist", arguments))
            return;
        if (!overlayNames) {
            overlayNames = Object.keys(this.overlaysByNames);
        }
        this.setAvailableOverlays(overlayNames, true);
    }
    setAvailableOverlays(overlayNames = [], condition, _tileLayers) {
        if (!dependency_utils_1.depsProvided(this, "setAvailableOverlays", arguments))
            return;
        if (!dependency_utils_1.depsProvided(this, "setAvailableTileLayers", arguments))
            return;
        this.availableOverlaysByNames = this.getListForAvailableLayers(overlayNames, condition, _tileLayers);
        !dependency_utils_1.isProvided(this, "availableOverlays") && dependency_utils_1.provide(this, "availableOverlays");
        dependency_utils_1.isProvided(this, "tileLayer") && this.setTileLayers(this._tileLayers);
    }
    getNormalizedZoom(zoom, tileLayer) {
        if (!zoom) {
            zoom = this.map.getZoom();
        }
        return (this._getMMLCRSLayers().indexOf(tileLayer || this.tileLayer) !== -1) ? zoom : zoom - 3;
    }
    getDenormalizedZoom(zoom, tileLayer) {
        if (typeof zoom !== "number" || isNaN(zoom)) {
            zoom = this.map.getZoom();
        }
        return this._getDefaultCRSLayers().indexOf(tileLayer || this.tileLayer) !== -1 ? zoom + 3 : zoom;
    }
    initZoom(zoom) {
        this.setNormalizedZoom(zoom);
    }
    setNormalizedZoom(zoom, options = { animate: false }) {
        this.zoom = zoom;
        if (!dependency_utils_1.depsProvided(this, "setNormalizedZoom", arguments)) {
            dependency_utils_1.provide(this, "zoom");
            return;
        }
        if (this.map) {
            this.map.setZoom(this.getDenormalizedZoom(this.zoom), options);
        }
        if (!dependency_utils_1.isProvided(this, "zoom"))
            dependency_utils_1.provide(this, "zoom");
    }
    setCenter(center) {
        this.center = center;
        if (!dependency_utils_1.depsProvided(this, "setCenter", arguments)) {
            dependency_utils_1.provide(this, "center");
            return;
        }
        if (this.map) {
            this.map.setView(center, this.getDenormalizedZoom(this.zoom), { animate: false });
        }
        if (!dependency_utils_1.isProvided(this, "center"))
            dependency_utils_1.provide(this, "center");
    }
    destroy() {
        this.cleanDOM();
        this.map && this.map.remove();
        this.map = null;
        Object.keys(this._documentEvents).forEach(type => this._documentEvents[type].forEach(fn => document.removeEventListener(type, fn)));
    }
    cleanDOM() {
        const safeRemove = (from, elem) => from && elem && from.contains(elem) && from.removeChild(elem);
        safeRemove(this.rootElem, this.container);
        safeRemove(this._dialogRoot, this.blockerElem);
        if (this._closeDialog)
            this._closeDialog();
        this._domCleaners.forEach(cleaner => cleaner());
        this._domCleaners = [];
    }
    _addDomCleaner(fn) {
        this._domCleaners.push(fn);
    }
    _removeDomCleaner(fn) {
        this._domCleaners = this._domCleaners.filter(_fn => _fn !== fn);
    }
    _constructDictionary() {
        function decapitalizeFirstLetter(s) {
            return s.charAt(0).toLowerCase() + s.slice(1);
        }
        let dictionaries = {};
        for (let word in translations_1.default) {
            for (let lang in translations_1.default[word]) {
                const translation = translations_1.default[word][lang];
                if (!dictionaries.hasOwnProperty(lang)) {
                    dictionaries[lang] = {};
                }
                dictionaries[lang][decapitalizeFirstLetter(word)] = decapitalizeFirstLetter(translation);
                dictionaries[lang][capitalizeFirstLetter(word)] = capitalizeFirstLetter(translation);
            }
        }
        for (let lang in dictionaries) {
            const dictionary = dictionaries[lang];
            for (let key in dictionary) {
                while (dictionary[key].indexOf("$") !== -1) {
                    const keyToReplace = dictionary[key].match(/\$\w+/)[0];
                    const replaceKey = keyToReplace.substring(1);
                    dictionary[key] = dictionary[key].replace(keyToReplace, dictionary[replaceKey]);
                }
            }
        }
        this.dictionary = dictionaries;
    }
    setLang(lang, shouldProvide = true) {
        if (!dependency_utils_1.depsProvided(this, "setLang", arguments))
            return;
        if (!this.translations || this.lang !== lang) {
            this.lang = lang;
            this.translations = this.dictionary[this.lang];
            this.onSetLangHooks.forEach(hook => hook());
            shouldProvide && dependency_utils_1.provide(this, "translations");
        }
    }
    formatFeatureOut(feature, layer) {
        if (layer && layer instanceof L.Circle) {
            // GeoJSON circles doesn't have radius, so we extend GeoJSON.
            feature.geometry.radius = layer.getRadius();
        }
        else if (feature.geometry.type === "Polygon") {
            // If the coordinates are ordered clockwise, reverse them.
            const coordinates = feature.geometry.coordinates[0].slice(0);
            const isClockwise = utils_1.coordinatesAreClockWise(coordinates);
            if (isClockwise) {
                feature = Object.assign(Object.assign({}, feature), { geometry: Object.assign(Object.assign({}, feature.geometry), { coordinates: [coordinates.reverse()] }) });
            }
        }
        const normalizeCoordinates = c => {
            if (typeof c[0] === "number") {
                return this.wrapGeoJSONCoordinate(c).map(_c => +_c.toFixed(6));
            }
            else {
                return c.map(normalizeCoordinates);
            }
        };
        feature = utils_1.updateImmutablyRecursivelyWith(feature, (key, value) => {
            if (key === "coordinates") {
                value = normalizeCoordinates(value);
            }
            return value;
        });
        const _a = feature.properties, { lajiMapIdx } = _a, properties = __rest(_a, ["lajiMapIdx"]); // eslint-disable-line
        return Object.assign(Object.assign({}, feature), { properties });
    }
    formatFeatureIn(feature, idx) {
        return Object.assign(Object.assign({}, feature), { properties: Object.assign(Object.assign({}, feature.properties), { lajiMapIdx: idx }) });
    }
    cloneFeatures(features) {
        const featuresClone = [];
        for (let i = 0; i < features.length; i++) {
            const feature = features[i];
            featuresClone[i] = this.formatFeatureIn(feature, i);
        }
        return featuresClone;
    }
    initializeDataItem(options, dataIdx) {
        var _a;
        dataIdx = dataIdx === undefined ? this.data.length : dataIdx;
        let item = (options || {});
        let { geoData } = item, _item = __rest(item, ["geoData"]);
        let format = item.format || geoData ? utils_1.detectFormat(geoData) : undefined;
        const crs = item.crs || (geoData || item.featureCollection) ? utils_1.detectCRS(geoData || item.featureCollection) : "WGS84";
        if ("geoData" in item) {
            const detectedFormat = utils_1.detectFormat(geoData);
            format = detectedFormat;
            if (detectedFormat === "GeoJSON") {
                const { type } = geoData;
                if (type === "FeatureCollection" || type === "Feature") {
                    format = "GeoJSONFeatureCollection";
                }
                else {
                    format = "GeoJSONGeometryCollection";
                }
            }
            const geoJSON = utils_1.convertAnyToWGS84GeoJSON(geoData);
            try {
                item = Object.assign(Object.assign({}, _item), { featureCollection: {
                        type: "FeatureCollection",
                        features: this.cloneFeatures(utils_1.flattenMultiLineStringsAndMultiPolygons(utils_1.anyToFeatureCollection(geoJSON).features))
                    }, format, crs });
            }
            catch (e) {
                throw new Error(`Invalid geoJSON type in data[${dataIdx}]`);
            }
            this.initializeDataItem(item, dataIdx);
            return;
            // Flatten features which have a geometry collection a geometry to features of the featureCollection
        }
        else if (item.featureCollection && item.featureCollection.features.some(feature => feature.geometry && feature.geometry.type === "GeometryCollection")) {
            item = Object.assign(Object.assign({}, item), { featureCollection: Object.assign(Object.assign({}, item.featureCollection), { features: item.featureCollection.features.reduce((features, f) => {
                        if (f.geometry.type === "GeometryCollection") {
                            f.geometry.geometries.forEach(g => {
                                features.push({ type: "Feature", geometry: g, properties: f.properties });
                            });
                        }
                        else {
                            features.push(f);
                        }
                        return features;
                    }, []) }) });
            this.initializeDataItem(item, dataIdx);
            return;
        }
        const features = ((_a = item.featureCollection) === null || _a === void 0 ? void 0 : _a.features)
            ? item.featureCollection.features
            : [];
        item = Object.assign(Object.assign({ getFeatureStyle: () => this._getDefaultDataStyle(item)(), getClusterStyle: () => this._getDefaultDataClusterStyle(item)(), getDraftStyle: (_dataIdx) => this._getDefaultDraftStyle(_dataIdx) }, item), { featureCollection: {
                type: "FeatureCollection",
                features: this.cloneFeatures(features)
            }, hasCustomGetFeatureStyle: item.hasOwnProperty("hasCustomGetFeatureStyle")
                ? item.hasCustomGetFeatureStyle
                : !!item.getFeatureStyle, idx: dataIdx });
        item.hasActive = ("activeIdx" in item) || item.hasActive;
        if (item.visible === undefined) {
            item.visible = true;
        }
        if (item.opacity === undefined) {
            item.opacity = 1;
        }
        this.idxsToIds[dataIdx] = {};
        this.idsToIdxs[dataIdx] = {};
        this._idxsToHovered[dataIdx] = [];
        this._idxsToContextMenuOpen[dataIdx] = [];
        if (this.data[dataIdx] && this.data[dataIdx].groupContainer) {
            this.data[dataIdx].groupContainer.clearLayers();
        }
        this._setOnChangeForItem(item, item.format, item.crs);
        this.data[dataIdx] = item;
        const layer = L.geoJSON(utils_1.convertAnyToWGS84GeoJSON(item.featureCollection), {
            pointToLayer: this._featureToLayer(item.getFeatureStyle, dataIdx),
            style: (feature) => {
                return this._getStyleForType([dataIdx, feature.properties.lajiMapIdx]);
            },
            onEachFeature: (feature, _layer) => {
                this._initializeLayer(_layer, [dataIdx, feature.properties.lajiMapIdx]);
            }
        });
        item.group = layer;
        item.groupContainer = layer;
        if (item.cluster) {
            item.groupContainer = L.markerClusterGroup(this.getClusterOptionsFor(item));
            item.group.addTo(item.groupContainer);
        }
        item.groupContainer.addTo(this.map);
        if (item.on)
            Object.keys(item.on).forEach(eventName => {
                item.groupContainer.on(eventName, (e) => {
                    var _a;
                    const { layer: _layer } = e;
                    const { feature } = _layer;
                    const idx = (_a = feature === null || feature === void 0 ? void 0 : feature.properties) === null || _a === void 0 ? void 0 : _a.lajiMapIdx;
                    if (eventName === "click" && this._interceptClick())
                        return;
                    const event = {
                        dataIdx
                    };
                    if (idx !== undefined) {
                        event.featureIdx = idx;
                        event.idx = idx; // for bw compatibility
                    }
                    if (layer) {
                        event.layer = _layer;
                    }
                    if (feature && _layer) {
                        event.feature = this.formatFeatureOut(feature, _layer);
                    }
                    item.on[eventName](e, event);
                });
            });
        item.group.on("click", (e) => {
            const { feature: { properties: { lajiMapIdx } } } = e.layer;
            if (!this._interceptClick())
                this._onActiveChange([item.idx, lajiMapIdx]);
        });
        item.group.on("dblclick", (e) => {
            this._disableDblClickZoom = true;
            const _layer = e.layer;
            this._setEditable(_layer);
            setTimeout(() => {
                this._disableDblClickZoom = false;
            }, 10);
        });
        item.group.on("mouseover", (e) => {
            var _a, _b;
            const _layer = e.layer;
            if (item.editable || item.hasActive || item.highlightOnHover) {
                const [_dataIdx, _featureIdx] = this._getIdxTupleByLayer(_layer);
                this._idxsToHovered[_dataIdx][_featureIdx] = true;
                this.updateLayerStyle(_layer);
            }
            if (utils_1.isObject(item.showMeasurements) && item.showMeasurements.showOnHover) {
                (_b = (_a = _layer).showMeasurements) === null || _b === void 0 ? void 0 : _b.call(_a);
            }
        });
        item.group.on("mouseout", (e) => {
            var _a, _b;
            const _layer = e.layer;
            if (item.editable || item.hasActive || item.highlightOnHover) {
                const [_dataIdx, _featureIdx] = this._getIdxTupleByLayer(_layer);
                this._idxsToHovered[_dataIdx][_featureIdx] = false;
                this.updateLayerStyle(_layer);
            }
            if (utils_1.isObject(item.showMeasurements) && item.showMeasurements.showOnHover) {
                (_b = (_a = _layer).hideMeasurements) === null || _b === void 0 ? void 0 : _b.call(_a);
            }
        });
        item.group.on("layeradd", (e) => {
            const _layer = e.layer;
            const { featureCollection: { features: _features } } = item;
            const featureIdx = _features.length - 1;
            const feature = this.formatFeatureOut(_layer.toGeoJSON(), _layer);
            feature.properties.lajiMapIdx = featureIdx;
            _layer.feature = feature;
            if (item.cluster) {
                item.groupContainer.addLayer(_layer);
            }
            this._initializeLayer(_layer, [dataIdx, featureIdx]);
            this.updateLayerStyle(_layer);
        });
        item.group.on("layerremove", (e) => {
            const _layer = e.layer;
            const [_dataIdx, _featureIdx] = this._getIdxTupleByLayer(_layer);
            this._idxsToContextMenuOpen[_dataIdx][_featureIdx] = false;
            this._idxsToHovered[_dataIdx][_featureIdx] = false;
        });
    }
    _getAllData() {
        const data = [...this.data];
        const draw = this.getDraw();
        if (draw) {
            data.push(draw);
        }
        return data;
    }
    zoomToData(options = {}) {
        if (!dependency_utils_1.depsProvided(this, "zoomToData", arguments))
            return;
        if (!options)
            return;
        if (options && !utils_1.isObject(options)) {
            options = {};
        }
        const bounds = this.getBoundsForZoomToDataOptions(options);
        if (!bounds.isValid())
            return;
        this.fitBounds(bounds, options);
    }
    getBoundsForZoomToDataOptions(options = {}) {
        if (options && !utils_1.isObject(options)) {
            options = {};
        }
        const { dataIdxs, draw } = options;
        const datasToZoom = dataIdxs || draw
            ? [...(dataIdxs || []), ...(draw ? [this.drawIdx] : [])]
            : undefined;
        return datasToZoom
            ? this.getBoundsForIdxs(datasToZoom)
            : this.getBoundsForData();
    }
    getBoundsForDraw() {
        return this.getBoundsForIdxs(this.drawIdx);
    }
    getBoundsForIdxs(...idxs) {
        return this.getBoundsForData(idxs.map(i => this.data[i]));
    }
    getBoundsForData(datas) {
        if (!datas) {
            datas = this._getAllData();
        }
        const featureGroup = L.featureGroup(datas.filter(item => item).reduce((layers, item) => {
            const newLayers = this._mapLayersToBoundableLayers(item.group.getLayers());
            layers = [...layers, ...newLayers];
            return layers;
        }, []));
        return featureGroup.getBounds();
    }
    getBoundsForLayers(layers) {
        return L.featureGroup(this._mapLayersToBoundableLayers(layers)).getBounds();
    }
    _mapLayersToBoundableLayers(layers) {
        return layers.map(layer => {
            if (layer instanceof L.Circle) { // getBounds fails for circles
                const { lat, lng } = layer.getLatLng();
                const polygonGeoJSON = utils_1.circleToPolygon([lng, lat], layer.getRadius(), 4);
                return L.polygon(polygonGeoJSON.coordinates[0].map(c => c.slice(0).reverse()));
            }
            return layer;
        });
    }
    _initializeLayer(layer, idxTuple) {
        this._setIdForLayer(layer, idxTuple);
        this._initializePopup(layer);
        this._initializeTooltip(layer);
        this._updateContextMenuForLayer(layer);
        if (isPolyline(layer))
            this._decoratePolyline(layer);
        const item = this.data[idxTuple[0]];
        if (item.showMeasurements
            && !(utils_1.isObject(item.showMeasurements) && item.showMeasurements.showOnHover)) {
            layer.options.showMeasurements = true;
        }
    }
    fitBounds(_bounds, options = {}) {
        let bounds = L.latLngBounds(_bounds);
        if (!bounds.isValid())
            return;
        const { paddingInMeters } = options;
        if (paddingInMeters) {
            bounds = L.latLngBounds(bounds.getSouthWest().toBounds(paddingInMeters).getSouthWest(), bounds.getNorthEast().toBounds(paddingInMeters).getNorthEast());
        }
        const { minZoom, maxZoom } = options, _options = __rest(options, ["minZoom", "maxZoom"]);
        this._swapToWorldOutsideFinland(bounds.getCenter());
        this.map.fitBounds(bounds, _options);
        if (options.hasOwnProperty("maxZoom")) {
            if (this.getNormalizedZoom() > maxZoom)
                this.setNormalizedZoom(maxZoom);
        }
        if (options.hasOwnProperty("minZoom")) {
            if (this.getNormalizedZoom() < minZoom)
                this.setNormalizedZoom(minZoom);
        }
    }
    setData(data) {
        if (!dependency_utils_1.depsProvided(this, "setData", arguments))
            return;
        this.data.forEach((item, idx) => {
            (idx !== this.drawIdx && item) && item.groupContainer.clearLayers();
        });
        const draw = this.getDraw();
        this.data = [];
        if (draw) {
            this.data[this.drawIdx] = draw;
        }
        if (!Array.isArray(data))
            data = [data];
        data.forEach((item, idx) => (idx !== this.drawIdx) && this.updateData(idx, item));
        this.reorderData();
        this.map.fire("lajiMap:dataChange", this.data);
        dependency_utils_1.provide(this, "data");
    }
    addData(items) {
        if (!items)
            return;
        if (!Array.isArray(items))
            items = [items];
        items.forEach(item => this.initializeDataItem(item));
        this.map.fire("lajiMap:dataChange", this.data);
    }
    updateData(dataIdx, item) {
        this.initializeDataItem(item, dataIdx);
        this.map.fire("lajiMap:dataChange", this.data);
    }
    updateDrawData(item) {
        this.updateData(this.drawIdx, this.getDrawOptions(item));
        this.map.fire("lajiMap:dataChange", this.data);
    }
    removeData(dataIdx) {
        if (this.data[dataIdx]) {
            this.data[dataIdx].groupContainer.clearLayers();
            delete this.data[dataIdx];
        }
        this.map.fire("lajiMap:dataChange", this.data);
    }
    getDrawOptions(options) {
        const drawAllowed = options !== false;
        let draw = Object.assign({}, this.getFeatureTypes().reduce((_options, key) => {
            let optionValue = {};
            if (options === false || utils_1.isObject(options) && options[key] === false)
                optionValue = false;
            else if (utils_1.isObject(options) && utils_1.isObject(options[key]))
                optionValue = options[key];
            _options[key] = optionValue;
            return _options;
        }, {}));
        draw = Object.assign(Object.assign({}, draw), Object.assign({}, (drawAllowed ? (utils_1.isObject(options) ? options : {}) : {})));
        if (options.data) {
            console.warn("laji-map warning: draw.data is deprecated and HAS BEEN REMOVED! Please move it's content to draw");
        }
        draw = Object.assign({ getFeatureStyle: () => this._getDefaultDrawStyle(), getClusterStyle: () => this._getDefaultDrawClusterStyle(draw), getDraftStyle: () => this._getDefaultDrawDraftStyle(), hasCustomGetFeatureStyle: !!draw.getFeatureStyle, editable: true }, draw);
        return draw;
    }
    setDraw(options) {
        if (!dependency_utils_1.depsProvided(this, "setDraw", arguments))
            return;
        // Using a negative idx lets us keep the original data indices.
        if (this.drawIdx === undefined)
            this.drawIdx = -1;
        this.updateDrawData(options);
        this.resetDrawUndoStack();
        this.reorderData();
        dependency_utils_1.provide(this, "draw");
    }
    getDraw() {
        return this.data[this.drawIdx];
    }
    drawIsAllowed() {
        const draw = this.getDraw();
        return this.getFeatureTypes().some(type => draw[type]);
    }
    drawIsEditable() {
        const draw = this.getDraw();
        return this.drawIsAllowed && draw.editable;
    }
    resetDrawUndoStack() {
        this._drawHistory = [{ featureCollection: {
                    type: "FeatureCollection",
                    features: this.cloneFeatures(this.getDraw().featureCollection.features)
                } }];
        this._drawHistoryPointer = 0;
    }
    _getEventsWithLayers(events) {
        let createCount = 0;
        return events.slice(0).reverse().map(e => {
            if (e.type === "create") {
                const _e = Object.assign(Object.assign({}, e), { layer: this.getDrawLayerByIdx(this.getDraw().featureCollection.features.length - 1 - createCount) });
                createCount++;
                return _e;
            }
            else if (e.type === "insert") {
                return Object.assign(Object.assign({}, e), { layer: this.getDrawLayerByIdx(e.idx) });
            }
            else if (e.type === "edit") {
                return Object.assign(Object.assign({}, e), { layers: Object.keys(e.features).reduce((layers, idx) => (Object.assign(Object.assign({}, layers), { [idx]: this.getDrawLayerByIdx(+idx) })), {}) });
            }
            if (e.type === "active") {
                this.setActive(this._getLayerByIdxTuple([this.drawIdx, e.idx]));
            }
            return e;
        });
    }
    drawUndo() {
        if (this._drawHistoryPointer <= 0)
            return;
        const { undoEvents: events } = this._drawHistory[this._drawHistoryPointer];
        this._drawHistoryPointer--;
        const { featureCollection } = this._drawHistory[this._drawHistoryPointer];
        this.updateData(this.drawIdx, Object.assign(Object.assign({}, this.getDraw()), { featureCollection }));
        if (events) {
            const withLayers = this._getEventsWithLayers(events);
            this._triggerEvent(withLayers, this.getDraw().onChange);
        }
    }
    drawRedo() {
        if (this._drawHistoryPointer >= this._drawHistory.length - 1)
            return;
        this._drawHistoryPointer++;
        const { featureCollection, redoEvents: events } = this._drawHistory[this._drawHistoryPointer];
        this.updateData(this.drawIdx, Object.assign(Object.assign({}, this.getDraw()), { featureCollection }));
        if (events) {
            const withLayers = this._getEventsWithLayers(events);
            this._triggerEvent(withLayers, this.getDraw().onChange);
        }
    }
    wrapGeoJSONCoordinate([lng, lat]) {
        const wrapped = this.map.wrapLatLng([lat, lng]);
        return [wrapped.lng, wrapped.lat];
    }
    _setOnChangeForItem(item, format = "GeoJSON", crs = "WGS84") {
        if (!dependency_utils_1.depsProvided(this, "_setOnChangeForItem", arguments))
            return;
        const convertCoordinateSystem = format === "GeoJSONFeatureCollection" || format === "GeoJSONGeometryCollection"
            ? "GeoJSON"
            : format;
        const onChange = item.onChange;
        let converted;
        item.onChange = events => {
            const _events = events.map(e => {
                switch (e.type) {
                    case "create":
                    case "insert":
                        converted = utils_1.convert(e.feature, convertCoordinateSystem, crs);
                        if (converted.type === "FeatureCollection" && format === "GeoJSONGeometryCollection") {
                            converted = {
                                type: "GeometryCollection",
                                geometries: converted.features.map(f => f.geometry)
                            };
                        }
                        e.geoData = converted;
                        break;
                    case "edit":
                        e.geoData = Object.keys(e.features).reduce((_converted, idx) => {
                            _converted[idx] = utils_1.convert(e.features[idx], convertCoordinateSystem, crs);
                            if (_converted[idx].type === "FeatureCollection" && format === "GeoJSONGeometryCollection") {
                                _converted[idx] = _converted[idx].geometry;
                            }
                            return _converted;
                        }, {});
                        break;
                }
                return e;
            });
            onChange && onChange(_events);
        };
    }
    clearItemData(item, commit = true) {
        const prevFeatureCollection = {
            type: "FeatureCollection",
            features: this.cloneFeatures(this.data[item.idx].featureCollection.features)
        };
        const event = {
            type: "delete",
            idxs: Object.keys(this.idxsToIds[item.idx]).map(idx => parseInt(idx)),
            features: this.cloneFeatures(this.data[item.idx].featureCollection.features)
        };
        commit && this._triggerEvent(event, item.onChange);
        this.updateData(item.idx, Object.assign(Object.assign({}, item), { geoData: undefined, featureCollection: { type: "FeatureCollection", features: [] } }));
        this._resetIds(item.idx);
        if (commit && item.idx === this.drawIdx)
            this._updateDrawUndoStack(event, prevFeatureCollection);
    }
    clearDrawData() {
        this.clearItemData(this.getDraw());
    }
    _startDrawRemove() {
        if (this._onDrawRemove)
            return;
        this._createTooltip("RemoveFeatureOnClick");
        this._drawRemoveLayers = [];
        this._onDrawRemove = (layer) => {
            this._drawRemoveLayers.push(layer);
            this._removeLayerFromItem(this.getDraw(), layer);
        };
        this.getDraw().group.on("click", ({ layer }) => this._onDrawRemove(layer));
        this._addKeyListener(globals_1.ESC, this._stopDrawRemove);
    }
    _stopDrawRemove() {
        this.getDraw().group.removeEventListener("click", ({ layer }) => this._onDrawRemove(layer));
        this._onDrawRemove = undefined;
        this._disposeTooltip();
        this._drawRemoveLayers = undefined;
        this._removeKeyListener(globals_1.ESC, this._stopDrawRemove);
        return true;
    }
    _finishDrawRemove() {
        const layers = this._drawRemoveLayers;
        this._stopDrawRemove();
        if (layers && layers.length)
            this._onDelete(this.drawIdx, layers.map(layer => L.Util.stamp(layer)));
    }
    _cancelDrawRemove() {
        const layers = this._drawRemoveLayers;
        this._stopDrawRemove();
        if (layers) {
            layers.forEach(layer => {
                this.getDraw().group.addLayer(layer);
                this.updateLayerStyle(layer);
            });
            return true;
        }
        return false;
    }
    _onDrawReverseHandler({ layer }) {
        this._onDrawReverse(layer);
    }
    _startDrawReverse() {
        if (this._onDrawReverse)
            return;
        this._createTooltip("ReverseLineOnClick");
        this._drawReverseLayers = [];
        this._onDrawReverse = (layer) => {
            if (!isPolyline(layer))
                return;
            this._drawReverseLayers.push(layer);
            this._reversePolyline(layer);
        };
        this.getDraw().group.on("click", this._onDrawReverseHandler);
        this._addKeyListener(globals_1.ESC, this._stopDrawReverse);
    }
    _stopDrawReverse() {
        if (!this._onDrawReverse)
            return;
        this.getDraw().group.removeEventListener("click", this._onDrawReverseHandler);
        this._onDrawReverse = undefined;
        this._drawReverseLayers = undefined;
        this._disposeTooltip();
        this._removeKeyListener(globals_1.ESC, this._stopDrawReverse);
    }
    _finishDrawReverse() {
        const layers = this._drawReverseLayers;
        this._stopDrawReverse();
        if (!layers || !layers.length)
            return;
        const editData = layers.reduce((idToLayer, layer) => {
            idToLayer[L.Util.stamp(layer)] = { layer };
            return idToLayer;
        }, {});
        this._onEdit(this.drawIdx, editData);
    }
    _cancelDrawReverse() {
        const layers = this._drawReverseLayers;
        this._stopDrawReverse();
        if (!layers)
            return;
        layers.forEach(layer => {
            layer.setLatLngs(this._origLatLngs[L.Util.stamp(layer)]);
            this._decoratePolyline(layer);
        }, {});
        this._origLatLngs = undefined;
    }
    _createIcon(options = {}, icon, feature) {
        var _a;
        const color = options.color || globals_1.NORMAL_COLOR;
        const opacity = (_a = options.opacity) !== null && _a !== void 0 ? _a : 1;
        if (icon) {
            return icon(Object.assign(Object.assign({}, options), { color, opacity }), feature);
        }
        return new L.VectorMarkers.Icon(Object.assign(Object.assign({ prefix: "glyphicon", icon: "record" }, options), { color,
            opacity }));
    }
    _getClusterIcon(data) {
        return (cluster) => {
            var _a;
            const childCount = cluster.getChildCount();
            let className = " marker-cluster-";
            if (childCount < 10) {
                className += "small";
            }
            else if (childCount < 100) {
                className += "medium";
            }
            else {
                className += "large";
            }
            let color = globals_1.NORMAL_COLOR;
            let opacity = (_a = data.opacity) !== null && _a !== void 0 ? _a : 0.5;
            if (data.getClusterStyle) {
                const featureStyle = data.getClusterStyle(childCount, cluster.getAllChildMarkers().map(marker => marker.feature.properties.lajiMapIdx), cluster);
                if (featureStyle.color)
                    color = featureStyle.color;
                if (featureStyle.opacity)
                    opacity = featureStyle.opacity;
            }
            if (data.getClusterClassName) {
                const customClassName = data.getClusterClassName(childCount, cluster.getAllChildMarkers().map(marker => marker.feature.properties.lajiMapIdx), cluster);
                if (customClassName) {
                    className = " " + customClassName;
                }
            }
            const styleObject = {
                "background-color": color,
                opacity
            };
            const styleString = Object.keys(styleObject)
                .reduce((style, key) => {
                style += `${key}:${styleObject[key]};`;
                return style;
            }, "");
            return L.divIcon({
                html: `<div style="${styleString}"><span>${childCount}</span></div>`,
                className: `marker-cluster${className}`,
                iconSize: new L.Point(40, 40)
            });
        };
    }
    setActive(layer) {
        const [dataIdx, featureIdx] = this._getIdxTupleByLayer(layer);
        const item = this.data[dataIdx];
        if (!item.hasActive)
            return;
        const prevActiveIdx = item.activeIdx;
        item.activeIdx = featureIdx;
        const prevActiveLayer = this._getLayerByIdxTuple([dataIdx, prevActiveIdx]);
        prevActiveLayer && this.updateLayerStyle(prevActiveLayer);
        this.updateLayerStyle(layer);
    }
    _resetIds(dataIdx) {
        // Maps item indices to internal ids and the other way around.
        // We use leaflet ids as internal ids.
        this.idxsToIds[dataIdx] = {};
        this.idsToIdxs[dataIdx] = {};
        let counter = 0;
        if (this.data[dataIdx].group)
            this.data[dataIdx].group.eachLayer((layer) => {
                this._setIdForLayer(layer, [dataIdx, counter]);
                counter++;
            });
    }
    _setIdForLayer(layer, idxTuple) {
        const [dataIdx, featureIdx] = idxTuple;
        if (!this.idxsToIds[dataIdx]) {
            this.idxsToIds[dataIdx] = {};
            this.idsToIdxs[dataIdx] = {};
        }
        const id = L.Util.stamp(layer);
        this.idxsToIds[dataIdx][featureIdx] = id;
        this.idsToIdxs[dataIdx][id] = featureIdx;
        this.idsToIdxTuples[id] = [dataIdx, featureIdx];
    }
    recluster() {
        this._reclusterData();
        this.draw && this._reclusterDrawData();
    }
    _reclusterDrawData() {
        this._reclusterDataItem(this.getDraw());
    }
    _reclusterData() {
        if (this.data)
            this.data.forEach(item => this._reclusterDataItem(item));
    }
    getClusterOptionsFor(item) {
        return Object.assign({ iconCreateFunction: this._getClusterIcon(item) }, (utils_1.isObject(item.cluster) ? item.cluster : {}));
    }
    _reclusterDataItem(item) {
        if (item.cluster) {
            this.map.removeLayer(item.groupContainer);
            item.groupContainer = L.markerClusterGroup(this.getClusterOptionsFor(item)).addTo(this.map);
            item.groupContainer.addLayer(item.group);
            item.group.eachLayer(l => this.updateLayerStyle(l));
        }
    }
    reorderData() {
        this._getAllData().forEach(i => i && i.group.bringToFront());
    }
    redrawDrawData() {
        this.redrawDataItem(this.drawIdx);
    }
    redrawDataItem(idx) {
        const dataItem = this.data[idx];
        if (!dataItem || !dataItem.group)
            return;
        this._updateDataLayerGroupStyle(idx);
        dataItem.group.eachLayer((layer) => {
            this._initializePopup(layer);
            this._initializeTooltip(layer);
        });
    }
    redrawData() {
        this.data.forEach((dataItem, idx) => {
            this.redrawDataItem(idx);
        });
    }
    redraw() {
        this.redrawData();
        this.redrawDrawData();
    }
    _initializePopup(layer) {
        const [dataIdx] = this._getIdxTupleByLayer(layer);
        const item = this.data[dataIdx];
        if (!item.getPopup)
            return;
        const that = this;
        let latlng = undefined;
        function openPopup(content) {
            const [, featureIdx] = that._getIdxTupleByLayer(layer);
            if (!latlng)
                return;
            if (that.editIdxTuple && that.editIdxTuple[0] === dataIdx && that.editIdxTuple[1] === featureIdx)
                return;
            const { markerPopupOffset = 40, featurePopupOffset = 5 } = that;
            const offset = (layer instanceof L.Marker) ? (-markerPopupOffset || 0) : (-featurePopupOffset || 0);
            that.popup = new L.Rrose({ offset: new L.Point(0, offset), closeButton: !that.popupOnHover, autoPan: false, y_bound: 80 - offset })
                .setContent(content)
                .setLatLng(latlng)
                .openOn(that.map);
        }
        function closePopup() {
            if (latlng)
                that.map.closePopup();
            if (that.onPopupClose)
                that.onPopupClose();
            latlng = undefined;
        }
        function getContentAndOpenPopup(_latlng) {
            const [, featureIdx] = that._getIdxTupleByLayer(layer);
            if (!that.popupCounter)
                that.popupCounter = 0;
            that.popupCounter++;
            latlng = _latlng;
            let { popupCounter } = that;
            // Allow either returning content or firing a callback with content.
            const content = item.getPopup({ dataIdx, featureIdx, feature: that.formatFeatureOut(layer.toGeoJSON(), layer), item }, callbackContent => that.popupCounter === popupCounter && openPopup(callbackContent));
            content !== undefined && typeof content !== "function" && openPopup(content);
        }
        if (this.popupOnHover) {
            layer.on("mousemove", (e) => {
                latlng = e.latlng;
                if (that.popup)
                    that.popup.setLatLng(latlng);
            });
            layer.on("mouseover", (e) => {
                getContentAndOpenPopup(e.latlng);
            });
            layer.on("mouseout", () => {
                closePopup();
            });
            layer.on("remove", () => {
                closePopup();
            });
        }
        else {
            layer.on("click", (e) => {
                if (item.getPopup) {
                    closePopup();
                    getContentAndOpenPopup(e.latlng);
                }
            });
        }
    }
    _initializeTooltip(layer) {
        const openTooltip = () => {
            const [dataIdx, featureIdx] = this._getIdxTupleByLayer(layer);
            const item = this.data[dataIdx];
            if (!item.getTooltip)
                return;
            function _openTooltip(content) {
                layer.bindTooltip(content, item.tooltipOptions).openTooltip();
            }
            // Allow either returning content or firing a callback with content.
            const content = item.getTooltip({ dataIdx, featureIdx, feature: this.formatFeatureOut(layer.toGeoJSON(), layer), item }, callbackContent => _openTooltip(callbackContent));
            if (content !== undefined && typeof content !== "function")
                _openTooltip(content);
        };
        const closeTooltip = () => {
            layer.unbindTooltip();
        };
        layer.on("mouseover", () => {
            openTooltip();
        });
        layer.on("mouseout", () => {
            closeTooltip();
        });
    }
    _updateContextMenuForLayer(layer) {
        if (!dependency_utils_1.depsProvided(this, "_updateContextMenuForLayer", arguments))
            return;
        const [dataIdx, featureIdx] = this._getIdxTupleByLayer(layer);
        const { translations } = this;
        layer.unbindContextMenu();
        let contextmenuItems = [];
        if (this.data[dataIdx] && this.data[dataIdx].editable) {
            contextmenuItems = [
                {
                    text: translations.EditFeature,
                    callback: () => this._setEditable(layer),
                    iconCls: "glyphicon glyphicon-pencil"
                },
                {
                    text: translations.DeleteFeature,
                    callback: () => this._onDelete(dataIdx, this.idxsToIds[dataIdx][featureIdx]),
                    iconCls: "glyphicon glyphicon-trash"
                },
            ];
            if (isPolyline(layer)) {
                contextmenuItems.push({
                    text: translations.ReverseFeature,
                    callback: () => {
                        this._reversePolyline(layer);
                        const id = this.idxsToIds[dataIdx][featureIdx];
                        this._onEdit(dataIdx, { [id]: { layer } });
                    },
                    iconCls: "glyphicon glyphicon-sort"
                });
            }
        }
        layer.bindContextMenu({
            contextmenuInheritItems: false,
            contextmenuItems
        });
    }
    setLocate(locate = false) {
        const defaultOptions = {
            on: true,
            onLocationFound: undefined,
            onLocationError: undefined,
            userLocation: undefined,
            panOnFound: true
        };
        let options = defaultOptions;
        if (utils_1.isObject(locate)) {
            options = Object.assign(Object.assign({}, defaultOptions), locate);
        }
        else if (typeof locate === "boolean") {
            options = Object.assign(Object.assign({}, defaultOptions), { on: !!locate });
        }
        else if (Array.isArray(locate)) {
            console.warn("laji-map warning: locate signature has changed, read the README. Options in array format will be removed in the future!");
            options = Object.assign(Object.assign({}, defaultOptions), { onLocationFound: locate[0], onLocationError: locate[1], userLocation: locate[2] });
        }
        const locateOn = locate
            ? options.on
            : !!locate;
        this.locateOptions = options;
        this._locateParam = locate;
        locateOn
            ? this._setLocateOn()
            : this.userLocationMarker
                ? this.userLocationMarker.remove() && this.userLocationRadiusMarker.remove()
                : undefined;
    }
    _setLocateOn(triggerEvent = false) {
        if (!dependency_utils_1.depsProvided(this, "_setLocateOn", arguments))
            return;
        this.locatingOn = true;
        const { on, userLocation } = this.locateOptions;
        if (on && this._located === undefined && userLocation) {
            this._onLocationFound(userLocation);
        }
        this.map.locate({ watch: true, enableHighAccuracy: true });
        triggerEvent && this.map.fire("locateToggle", { locate: typeof this._locateParam === "boolean"
                ? true
                : Object.assign(Object.assign({}, this.locateOptions), { on: true })
        });
    }
    _setLocateOff() {
        if (!dependency_utils_1.depsProvided(this, "_setLocateOff", arguments))
            return;
        this.locatingOn = false;
        this.map.stopLocate();
        if (this.userLocationLayer) {
            this.userLocationLayer.remove();
            this.userLocationLayer = undefined;
            this.userLocationMarker = undefined;
            this.userLocationRadiusMarker = undefined;
        }
        const { on, onLocationFound } = this.locateOptions;
        if (on && onLocationFound)
            onLocationFound(undefined);
        this._located = false;
        this.map.fire("locateToggle", { locate: typeof this._locateParam === "boolean"
                ? false
                : Object.assign(Object.assign({}, this.locateOptions), { on: false })
        });
    }
    _onLocationFound({ latlng, accuracy, bounds }) {
        if (!dependency_utils_1.depsProvided(this, "_onLocationFound", arguments))
            return;
        if (!this.locatingOn) {
            return;
        }
        if (!this._located && bounds && this.locateOptions.panOnFound)
            this.map.fitBounds(bounds);
        this._located = true;
        if (this.userLocationLayer) {
            this.userLocationRadiusMarker.setLatLng(latlng);
            this.userLocationRadiusMarker.setRadius(accuracy);
            this.userLocationMarker.setLatLng(latlng);
        }
        else {
            const { layerGroup, markerLayer, radiusLayer } = this.createUserLocationLayer(latlng, accuracy);
            this.userLocationLayer = layerGroup;
            this.userLocationRadiusMarker = radiusLayer;
            this.userLocationMarker = markerLayer;
            layerGroup.addTo(this.map);
        }
        this.userLocation = { latlng, accuracy };
        if (this.locateOptions.onLocationFound)
            this.locateOptions.onLocationFound(latlng, accuracy);
    }
    createUserLocationLayer(latlng, accuracy) {
        const layerGroup = L.layerGroup();
        const radiusLayer = L.circle(latlng, {
            radius: accuracy,
            color: globals_1.USER_LOCATION_COLOR,
            fillColor: globals_1.USER_LOCATION_COLOR,
            opacity: 0
        }).addTo(layerGroup);
        const markerLayer = L.circleMarker(latlng, {
            color: globals_1.USER_LOCATION_COLOR,
            fillColor: globals_1.USER_LOCATION_COLOR,
            fillOpacity: 0.7
        }).addTo(layerGroup);
        markerLayer.on("click", () => {
            !this._interceptClick() && this.map.fitBounds(radiusLayer.getBounds());
        });
        return { layerGroup, markerLayer, radiusLayer };
    }
    _onLocationNotFound(e) {
        const { onLocationError } = this.locateOptions;
        if (onLocationError)
            onLocationError(e);
    }
    getDrawLayerByIdx(idx) {
        return this._getLayerByIdxTuple([this.drawIdx, idx]);
    }
    getLayerByIdxTuple(idxTuple) {
        const [dataIdx, featureIdx] = idxTuple;
        const item = this.data[dataIdx];
        const id = this.idxsToIds[dataIdx][featureIdx];
        return item.group ? item.group.getLayer(id) : undefined;
    }
    _getLayerById(id) {
        const [dataIdx] = this.idsToIdxTuples[id];
        return this.data[dataIdx].group.getLayer(id);
    }
    _getIdxTupleByLayer(layer) {
        return this.idsToIdxTuples[layer._leaflet_id];
    }
    _getDrawLayerById(id) {
        return this._getLayerById(id);
    }
    _triggerEvent(e, handler) {
        if (!Array.isArray(e))
            e = [e];
        if (handler)
            handler(e);
    }
    _decoratePolyline(layer) {
        if (!isPolyline(layer))
            return;
        const idxTuple = this._getIdxTupleByLayer(layer);
        const { showStart, showDirection = true } = this._fillStyleWithGlobals(idxTuple);
        function warn() {
            console.warn("Failed to add a starting point to line");
        }
        const typelessLayer = layer;
        if (showDirection !== false) {
            const { clickable } = typelessLayer;
            typelessLayer.options.clickable = false;
            try {
                layer.setText(null).setText("→", { repeat: true, attributes: { "dy": 5, "font-size": 18 } });
            }
            catch (e) {
                console.warn("laji-map polyline text decorating failed");
            }
            typelessLayer.options.clickable = clickable;
        }
        if (!showStart)
            return;
        let firstPoint = undefined;
        if (!layer.feature.geometry.type) {
            warn();
            return;
        }
        switch (layer.feature.geometry.type) {
            case "MultiLineString":
                firstPoint = layer.getLatLngs()[0][0];
                break;
            case "LineString":
                firstPoint = layer.getLatLngs()[0];
                break;
            default:
                throw new Error("Tried to decorate a line layer but wasn't a line");
        }
        if (!firstPoint) {
            warn();
            return;
        }
        if (typelessLayer._startCircle) {
            typelessLayer._startCircle.remove();
        }
        typelessLayer._startCircle = L.circleMarker(firstPoint, this._getStartCircleStyle(layer)).addTo(this.map);
        layer.on("editdrag", () => {
            typelessLayer._startCircle.setLatLng(layer.getLatLngs()[0]);
        });
        layer.on("remove", () => {
            typelessLayer._startCircle.remove();
        });
        layer.on("add", () => {
            typelessLayer._startCircle.addTo(this.map);
        });
        const { bringToFront } = typelessLayer.__proto__;
        typelessLayer.__proto__.bringToFront = () => {
            bringToFront.call(typelessLayer);
            typelessLayer._startCircle.bringToFront();
        };
    }
    _reversePolyline(layer) {
        const { type } = layer.feature.geometry;
        if (type === "LineString") {
            if (!this._origLatLngs) {
                this._origLatLngs = {};
            }
            const id = L.Util.stamp(layer);
            if (!this._origLatLngs[id])
                this._origLatLngs[id] = layer.getLatLngs();
            layer.setLatLngs(layer.getLatLngs().slice(0).reverse());
            this._decoratePolyline(layer);
        }
    }
    _getStartCircleStyle(lineLayer) {
        let options = Object.assign(Object.assign({}, (this.polyline || {})), lineLayer.options);
        return Object.assign(Object.assign({}, options), { weight: 0, radius: options.weight + 1, fill: true, fillOpacity: 1 });
    }
    _updateDrawUndoStack(events, prevFeatureCollection, prevActiveIdx) {
        if (this._drawHistoryPointer < this._drawHistory.length - 1) {
            this._drawHistory = this._drawHistory.splice(0).splice(0, this._drawHistoryPointer + 1);
        }
        const featureCollection = {
            type: "FeatureCollection",
            features: this.cloneFeatures(this.getDraw().featureCollection.features)
        };
        let reverseEvents = [];
        (Array.isArray(events) ? events : [events]).forEach(e => {
            switch (e.type) {
                case "create":
                    reverseEvents.push({
                        type: "delete",
                        idxs: [featureCollection.features.length - 1]
                    });
                    break;
                case "edit":
                    reverseEvents.push({
                        type: "edit",
                        features: Object.keys(e.features).reduce((features, idx) => {
                            features[idx] = prevFeatureCollection.features[idx];
                            return features;
                        }, {}),
                        idxs: [featureCollection.features.length - 1]
                    });
                    break;
                case "delete":
                    e.idxs.sort().reverse().forEach(idx => reverseEvents.push({
                        type: "insert",
                        idx,
                        feature: prevFeatureCollection.features[idx]
                    }));
                    break;
                case "active":
                    reverseEvents.push({
                        type: "active",
                        idx: prevActiveIdx
                    });
                    break;
            }
        });
        this._drawHistory.push({
            featureCollection,
            undoEvents: reverseEvents,
            redoEvents: Array.isArray(events) ? events : [events]
        });
        this._drawHistoryPointer++;
    }
    _onAdd(dataIdx, layer, coordinateVerbatim) {
        var _a;
        if (isPolyline(layer) && layer.getLatLngs().length < 2)
            return;
        const prevActiveIdx = this.data[dataIdx].activeIdx;
        let item = this.data[dataIdx];
        const { single, featureCollection: { features } } = item;
        const feature = this.formatFeatureOut(layer.toGeoJSON(), layer);
        if (coordinateVerbatim && feature.geometry) {
            feature.geometry.coordinateVerbatim = coordinateVerbatim;
        }
        if (single && features.length > 0) {
            this.clearItemData(item, !"commit");
            item = this.data[dataIdx];
            item.featureCollection.features = [features[features.length - 1]];
            item.group.addLayer(layer);
            this._onEdit(dataIdx, { [L.Util.stamp(layer)]: { layer, coordinateVerbatim } });
            return;
        }
        features.push(feature);
        item.group.addLayer(layer);
        if (item.showMeasurements
            && !(utils_1.isObject(item.showMeasurements) && item.showMeasurements.showOnHover)) {
            (_a = layer.showMeasurements) === null || _a === void 0 ? void 0 : _a.call(layer);
        }
        const events = [
            {
                type: "create",
                feature,
                layer
            },
        ];
        if (item.hasActive) {
            events.push({ type: "active", idx: features.length - 1, layer });
        }
        this._triggerEvent(events, item.onChange);
        if (dataIdx === this.drawIdx) {
            this._updateDrawUndoStack(events, undefined, prevActiveIdx);
        }
        if (item.hasActive) {
            this.setActive(this._getLayerByIdxTuple([dataIdx, features.length - 1]));
        }
    }
    _onEdit(dataIdx, data) {
        // Fix bug:
        // 1. Circle in edit mode
        // 2. Move the circle
        // 3. Open context menu for it
        // 4. Click outside circle to stop editing
        // 5. If draw onChange resets draw data, contextmenu crashes
        if (this._contextMenuLayer) {
            this.map.contextmenu.hide();
        }
        const eventData = {};
        const layersData = {};
        const prevFeatureCollection = {
            type: "FeatureCollection",
            features: this.cloneFeatures(this.data[dataIdx].featureCollection.features)
        };
        for (let id in data) {
            const { layer, coordinateVerbatim } = data[id];
            const feature = this.formatFeatureOut(layer.toGeoJSON(), layer);
            if (coordinateVerbatim && feature.geometry) {
                feature.geometry.coordinateVerbatim = coordinateVerbatim;
            }
            const idx = this.idsToIdxs[dataIdx][id];
            eventData[idx] = feature;
            layersData[idx] = layer;
            this.data[dataIdx].featureCollection.features[idx] = this.formatFeatureIn(feature, idx);
        }
        for (let id in data) {
            const layer = this._getLayerById(+id);
            if (layer) {
                layer.closePopup().closeTooltip();
            }
        }
        const item = this.data[dataIdx];
        const event = {
            type: "edit",
            features: eventData,
            layers: layersData
        };
        this._triggerEvent(event, item.onChange);
        if (dataIdx === this.drawIdx) {
            this._updateDrawUndoStack(event, prevFeatureCollection);
        }
    }
    _onDelete(dataIdx, deleteIds) {
        this._clearEditable();
        const prevFeatureCollection = {
            type: "FeatureCollection",
            features: this.cloneFeatures(this.data[dataIdx].featureCollection.features)
        };
        if (!Array.isArray(deleteIds))
            deleteIds = [deleteIds];
        const deleteIdxs = deleteIds.map(id => this.idsToIdxs[dataIdx][id]);
        const item = this.data[dataIdx];
        const activeIdx = item.activeIdx;
        const { featureCollection: { features } } = item;
        const survivingIds = Object.keys(this.idsToIdxs[dataIdx])
            .map(id => parseInt(id)).filter(id => deleteIds.indexOf(id) === -1);
        let changeActive = false;
        let newActiveId = undefined;
        const activeId = this.idxsToIds[dataIdx][activeIdx];
        if (item.hasActive) {
            if (features && survivingIds.length === 0) {
                changeActive = true;
            }
            else if (activeIdx !== undefined && deleteIds.indexOf(activeId) !== -1) {
                changeActive = true;
                let closestSmallerId = undefined;
                let closestGreaterId = undefined;
                let closestDistance = undefined;
                let closestNegDistance = undefined;
                survivingIds.forEach(id => {
                    const dist = activeIdx - this.idsToIdxs[dataIdx][id];
                    if (dist > 0 && (closestDistance === undefined || dist < closestDistance)) {
                        closestDistance = dist;
                        closestSmallerId = id;
                    }
                    else if (dist < 0 && (closestNegDistance === undefined || dist > closestNegDistance)) {
                        closestNegDistance = dist;
                        closestGreaterId = id;
                    }
                });
                if (closestGreaterId !== undefined)
                    newActiveId = closestGreaterId;
                else if (closestSmallerId !== undefined)
                    newActiveId = closestSmallerId;
            }
            else {
                newActiveId = activeId;
                changeActive = true;
            }
        }
        const deletedFeatures = [];
        item.featureCollection.features = features.filter((feature, i) => {
            const filter = deleteIdxs.indexOf(i) === -1;
            if (!filter) {
                deletedFeatures.push(feature);
            }
            return filter;
        });
        deleteIds.forEach(id => {
            this._removeLayerFromItem(item, this._getLayerById(id));
        });
        this._resetIds(dataIdx);
        item.group.eachLayer((layer) => {
            this._updateContextMenuForLayer(layer);
        });
        this._reclusterDataItem(dataIdx);
        const events = [{
                type: "delete",
                idxs: deleteIdxs,
                features: deletedFeatures
            }];
        if (changeActive) {
            events.push({
                type: "active",
                idx: newActiveId !== undefined ? this.idsToIdxs[dataIdx][newActiveId] : undefined,
                layer: newActiveId !== undefined ? this._getLayerById(newActiveId) : undefined
            });
        }
        this._triggerEvent(events, item.onChange);
        if (dataIdx === this.drawIdx) {
            this._updateDrawUndoStack(events, prevFeatureCollection, newActiveId ? activeIdx : undefined);
        }
        if (changeActive && newActiveId !== undefined) {
            this.setActive(this._getLayerByIdxTuple([dataIdx, this.idsToIdxs[dataIdx][newActiveId]]));
        }
    }
    _removeLayerFromItem(item, layer) {
        if (!layer) {
            return;
        }
        if (item.group !== item.groupContainer && item.groupContainer.hasLayer(layer)) {
            item.groupContainer.removeLayer(layer);
        }
        if (item.group.hasLayer(layer)) {
            item.group.removeLayer(layer);
        }
    }
    _onActiveChange(idxTuple) {
        const [dataIdx, featureIdx] = idxTuple;
        const item = this.data[dataIdx];
        if (item.hasActive) {
            const layer = this._getLayerByIdxTuple(idxTuple);
            this._triggerEvent({ type: "active", idx: featureIdx, layer }, item.onChange);
            this.setActive(layer);
        }
    }
    focusToLayerByIdxs(idxTuple) {
        const [dataIdx, featureIdx] = idxTuple;
        const id = this.idxsToIds[dataIdx][featureIdx];
        if (featureIdx === undefined) {
            return;
        }
        let layer = this._getDrawLayerById(id);
        if (!layer)
            return;
        if (layer instanceof L.Marker) {
            this.map.panTo(layer.getLatLng(), { animate: false });
        }
        else {
            this.map.fitBounds(layer.getBounds());
        }
        this._onActiveChange(idxTuple);
    }
    focusToDrawLayer(idx) {
        this.focusToLayerByIdxs([this.drawIdx, idx]);
    }
    _setEditable(layer) {
        const [dataIdx, featureIdx] = this._getIdxTupleByLayer(layer);
        const item = this.data[dataIdx];
        if (!item.editable || this._onDrawRemove || this._onDrawReverse)
            return;
        this._clearEditable();
        this.editIdxTuple = [dataIdx, featureIdx];
        const editLayer = this._getLayerByIdxTuple(this.editIdxTuple);
        if (item.cluster) {
            item.groupContainer.removeLayer(editLayer);
            this.map.addLayer(editLayer);
        }
        const layerOptions = editLayer.options;
        layerOptions.editing || (layerOptions.editing = {}); // See https://github.com/Leaflet/Leaflet.draw/issues/804
        layer.editing.enable();
        if (!this.map._editTooltip) {
            // For some reason editing circle radius fails without this
            this.map._editTooltip = { updateContent: () => { } };
        }
        editLayer.closePopup();
        this.updateLayerStyle(editLayer);
    }
    _clearEditable() {
        if (this.editIdxTuple === undefined)
            return;
        const editLayer = this._getLayerByIdxTuple(this.editIdxTuple);
        editLayer.editing.disable();
        const item = this.data[this.editIdxTuple[0]];
        if (item.cluster) {
            this.map.removeLayer(editLayer);
            item.groupContainer.addLayer(editLayer);
        }
        this._reclusterDataItem(item);
        this.editIdxTuple = undefined;
    }
    _commitEdit() {
        const { editIdxTuple } = this;
        const [dataIdx, featureIdx] = editIdxTuple;
        const editId = this.idxsToIds[dataIdx][featureIdx];
        this._clearEditable();
        const editLayer = this._getLayerByIdxTuple(editIdxTuple);
        this.updateLayerStyle(editLayer);
        this._onEdit(dataIdx, { [editId]: { layer: editLayer } });
    }
    // Returns true if click was intercepted
    _interceptClick() {
        if (this._onDrawRemove || this._onDrawReverse || this.drawing)
            return true;
        if (this.editIdxTuple !== undefined) {
            this._commitEdit();
            return true;
        }
        return false;
    }
    setLayerStyle(layer, style) {
        if (!layer)
            return;
        layer.setStyle(style);
        layer._initStyle = style; // Fixes style when clustered layer unspiderfies.
        if (layer._startCircle)
            layer._startCircle.setStyle(this._getStartCircleStyle(layer));
    }
    _featureToLayer(getFeatureStyle, dataIdx) {
        function reversePointCoords(coords) {
            return [coords[1], coords[0]];
        }
        return (feature) => {
            let layer;
            if (feature.geometry.type === "Point") {
                const latLng = reversePointCoords(feature.geometry.coordinates);
                const params = { feature, featureIdx: feature.properties.lajiMapIdx };
                if (dataIdx !== undefined)
                    params[dataIdx] = dataIdx;
                layer = (feature.geometry.radius)
                    ? new L.Circle(latLng, feature.geometry.radius)
                    : L.marker(latLng, {
                        icon: this._createIcon(getFeatureStyle(params), (typeof dataIdx === "number" && this.data[dataIdx].marker && typeof this.data[dataIdx].marker !== "boolean")
                            && this.data[dataIdx].marker.icon, feature)
                    });
            }
            else {
                layer = L.GeoJSON.geometryToLayer(feature);
            }
            return layer;
        };
    }
    _getDefaultDrawDraftStyle() {
        return this._getStyleForType([this.drawIdx, undefined], { color: globals_1.EDITABLE_DATA_LAYER_COLOR, fillColor: globals_1.EDITABLE_DATA_LAYER_COLOR, opacity: 0.8 });
    }
    _fillStyleWithGlobals(idxTuple) {
        const [dataIdx, featureIdx] = idxTuple;
        const layer = this._getLayerByIdxTuple(idxTuple);
        const item = this.data[dataIdx];
        const feature = item.featureCollection.features[featureIdx];
        const dataStyles = item.getFeatureStyle({
            dataIdx,
            featureIdx,
            feature,
            item
        });
        const mergeOptions = (type) => {
            return Object.assign(Object.assign({}, (this[type] || {})), (item[type] && typeof item[type] !== "boolean" ? item[type] : {}));
        };
        let featureTypeStyle = undefined;
        if (layer) {
            if (layer instanceof L.Marker) {
                featureTypeStyle = mergeOptions("marker");
            }
            else if (isPolyline(layer)) {
                featureTypeStyle = mergeOptions("polyline");
            }
            else if (layer instanceof L.Rectangle) {
                featureTypeStyle = mergeOptions("rectangle");
            }
            else if (layer instanceof L.Polygon) {
                featureTypeStyle = mergeOptions("polygon");
            }
            else if (layer instanceof L.Circle) {
                featureTypeStyle = mergeOptions("circle");
            }
        }
        else {
            switch (feature.geometry.type) {
                case "LineString":
                case "MultiLineString":
                    featureTypeStyle = mergeOptions("polyline");
                    break;
                case "Polygon":
                    featureTypeStyle = mergeOptions("polygon");
                    break;
                case "Point":
                    featureTypeStyle = (feature.geometry.radius) ? mergeOptions("circle") : mergeOptions("marker");
                    break;
            }
        }
        return Object.assign(Object.assign({}, (featureTypeStyle || {})), (dataStyles || {}));
    }
    _getStyleForType(idx, overrideStyles = {}) {
        let dataIdx, featureIdx;
        if (typeof idx === "number") {
            dataIdx = idx;
        }
        else {
            [dataIdx, featureIdx] = idx;
        }
        const item = this.data[dataIdx];
        const feature = item.featureCollection.features[featureIdx];
        const active = item.activeIdx === featureIdx;
        const visible = item.visible !== undefined ? item.visible : true;
        const opacity = visible
            ? item.opacity !== undefined ? item.opacity : 1
            : 0;
        let editing = false;
        if (this.editIdxTuple) {
            const [_dataIdx, _featureIdx] = this.editIdxTuple;
            if (_dataIdx === dataIdx && _featureIdx === featureIdx) {
                editing = true;
            }
        }
        const hovered = (dataIdx !== undefined &&
            featureIdx !== undefined &&
            this._idxsToHovered[dataIdx][featureIdx]);
        let dataStyles = undefined;
        if (item.getFeatureStyle) {
            dataStyles = item.getFeatureStyle({
                dataIdx,
                featureIdx,
                feature,
                item,
                active,
                editing,
                hovered
            });
            if (dataStyles.color && !dataStyles.fillColor) {
                dataStyles.fillColor = dataStyles.color;
            }
        }
        else {
            dataStyles = this._fillStyleWithGlobals([dataIdx, featureIdx]);
        }
        let layer = undefined;
        if (this.idxsToIds[dataIdx])
            layer = this._getLayerByIdxTuple([dataIdx, featureIdx]);
        const isLine = (layer && isPolyline(layer)
            ||
                feature && (feature.geometry.type === "LineString" || feature.geometry.type === "MultiLineString"));
        let style = {
            color: globals_1.NORMAL_COLOR,
            fillColor: globals_1.NORMAL_COLOR,
        };
        if (isLine) {
            style.weight = 10;
        }
        style = Object.assign(Object.assign(Object.assign(Object.assign({}, style), dataStyles), computeOpacities(item.visible, opacity, item.maxFillOpacity)), overrideStyles);
        const colors = [];
        if (!item.hasCustomGetFeatureStyle && active) {
            colors.push(["#00ff00", 80]);
        }
        if (!item.hasCustomGetFeatureStyle && editing) {
            const r = active ? "--" : "00";
            const b = r;
            colors.push([`#${r}ff${b}`, 30]);
        }
        if (!item.hasCustomGetFeatureStyle && (hovered || this._idxsToContextMenuOpen[dataIdx][featureIdx])) {
            colors.push(["#ffffff", 30]);
        }
        if (colors.length || this._idxsToContextMenuOpen[dataIdx][featureIdx]) {
            style = Object.assign({}, style);
            ["color", "fillColor"].forEach(prop => {
                if (style[prop]) {
                    let finalColor = undefined;
                    if (hovered && (this._onDrawRemove || (this._onDrawReverse && isLine))) {
                        finalColor = "#ff0000";
                    }
                    else {
                        finalColor = colors.reduce((combined, [color, amount]) => utils_1.combineColors(combined, color, amount), style[prop]);
                    }
                    style[prop] = finalColor;
                }
            });
        }
        return style;
    }
    _getStyleForLayer(layer, overrideStyles) {
        const [dataIdx, featureIdx] = this._getIdxTupleByLayer(layer);
        return this._getStyleForType([dataIdx, featureIdx], overrideStyles);
    }
    updateLayerStyle(layer) {
        if (!layer)
            return;
        this.setLayerStyle(layer, this._getStyleForLayer(layer));
    }
    _getDefaultDrawStyle() {
        return { color: globals_1.NORMAL_COLOR, fillColor: globals_1.NORMAL_COLOR, opacity: 1, fillOpacity: 0.7 };
    }
    _getDefaultDrawClusterStyle(item) {
        return { color: this.getDraw().getFeatureStyle({}).color, opacity: item.opacity };
    }
    _getDefaultDataStyle(item) {
        return () => {
            const color = (item && item.editable) ?
                globals_1.EDITABLE_DATA_LAYER_COLOR :
                globals_1.DATA_LAYER_COLOR;
            return { color, fillColor: color, opacity: 1, fillOpacity: 0.7 };
        };
    }
    _getDefaultDraftStyle(dataIdx) {
        return this._getStyleForType(dataIdx, { color: globals_1.EDITABLE_DATA_LAYER_COLOR, fillColor: globals_1.EDITABLE_DATA_LAYER_COLOR, opacity: 0.8 });
    }
    _updateDataLayerGroupStyle(idx) {
        const item = this.data[idx];
        if (!item)
            return;
        item.group.eachLayer((layer) => {
            this.updateLayerStyle(layer);
        });
    }
    addTranslationHook(elemOrFn, translationKey, attr = "innerHTML") {
        const that = this;
        function translate() {
            if (typeof elemOrFn === "function") {
                const fn = elemOrFn;
                fn();
            }
            else {
                const elem = elemOrFn;
                const translation = (typeof translationKey === "function") ? translationKey() : that.translations[translationKey];
                if (typeof elem[attr] === "function") {
                    elem[attr](translation);
                }
                else {
                    elem[attr] = translation;
                }
            }
        }
        translate();
        this.onSetLangHooks.push(translate);
        return translate;
    }
    removeTranslationHook(hook) {
        const index = this.onSetLangHooks.indexOf(hook);
        if (index >= 0) {
            this.onSetLangHooks.splice(index, 1);
        }
    }
    _getDrawOptionsForType(featureType) {
        const baseStyle = this.getDraw().getDraftStyle();
        let additionalOptions = {};
        switch (featureType) {
            case "marker":
                additionalOptions = {
                    icon: this._createIcon(this.getDraw().getDraftStyle(), (this.getDraw().marker && typeof this.getDraw().marker !== "boolean")
                        && this.getDraw().marker.icon)
                };
                break;
            case "polygon":
                additionalOptions = {
                    allowIntersection: false
                };
                break;
        }
        const _userDefined = this.getDraw()[featureType];
        const userDefined = __rest(utils_1.isObject(_userDefined) ? _userDefined : {}, []);
        delete userDefined.icon; // let this._createIcon decorate icon.
        return Object.assign(Object.assign(Object.assign(Object.assign({ metric: true, showLength: true, showRadius: true }, baseStyle), additionalOptions), userDefined), { shapeOptions: Object.assign(Object.assign(Object.assign({ showArea: true, poly: {
                    allowIntersection: false
                } }, baseStyle), (additionalOptions.shapeOptions || {})), userDefined) });
    }
    abortDrawing(e) {
        if (e instanceof Event && (e === null || e === void 0 ? void 0 : e.preventDefault)) {
            e.preventDefault();
            e.stopPropagation();
        }
        if (this._draftDrawLayer)
            this._draftDrawLayer.disable();
        this._draftDrawLayer = undefined;
        this._removeKeyListener(globals_1.ESC, this.abortDrawing);
        this.map.off("controlClick", this.abortDrawing);
        this.map.removeEventListener("draw:created", this.abortDrawing);
    }
    addDrawAbortListeners() {
        this.map.on("draw:created", this.abortDrawing);
        this.map.on("controlClick", this.abortDrawing);
        this._addKeyListener(globals_1.ESC, this.abortDrawing);
    }
    triggerDrawing(featureType) {
        this._draftDrawLayer = new L.Draw[capitalizeFirstLetter(featureType)](this.map, this._getDrawOptionsForType(featureType));
        this._draftDrawLayer.enable();
        this.addDrawAbortListeners();
    }
    addFeatureToDraw(feature) {
        this.addFeatureToData(feature, this.drawIdx);
    }
    addFeatureToData(feature, dataIdx) {
        const layer = this._featureToLayer(this.data[dataIdx].getFeatureStyle)(feature);
        this._onAdd(dataIdx, layer);
    }
    getFeatureTypes() {
        return ["rectangle", "polyline", "polygon", "circle", "marker"];
    }
    _showError(e) {
        const alert = document.createElement("div");
        alert.style.display = "block";
        alert.className = "laji-map-popup alert alert-danger";
        const lajiMapError = (e._lajiMapError) ? e : undefined;
        const message = () => `${translations_1.default.errorHTML} ` + lajiMapError
            ? utils_1.stringifyLajiMapError(lajiMapError, this.translations)
            : e.message;
        const translationHook = this.addTranslationHook(alert, message);
        this.showClosableElement(alert, () => {
            this.removeTranslationHook(translationHook);
        });
    }
    showClosableElement(elem, onClose, blocker = false, getContainer = () => this.container) {
        const closeButton = document.createElement("button");
        closeButton.setAttribute("type", "button");
        closeButton.className = "close";
        closeButton.innerHTML = "✖";
        closeButton.addEventListener("click", close);
        elem.insertBefore(closeButton, elem.firstChild);
        const that = this;
        function close(e) {
            if (e)
                e.preventDefault();
            that._removeKeyListener(globals_1.ESC, close);
            getContainer().removeChild(elem);
            if (blocker) {
                that.blockerElem.style.display = "";
                that.blockerElem.removeEventListener("click", close);
            }
            if (onClose)
                onClose(e);
            that._closeDialog = undefined;
            that._openDialogs = that._openDialogs.filter(dialog => dialog !== elem);
            return true;
        }
        this._addKeyListener(globals_1.ESC, close);
        getContainer().appendChild(elem);
        if (blocker) {
            this.blockerElem.addEventListener("click", close);
            this.blockerElem.style.display = "block";
        }
        this._closeDialog = close;
        if (getContainer() === this._dialogRoot) {
            this._openDialogs.push(elem);
        }
    }
    _createTooltip(translationKey, error = false) {
        if (this._tooltip && this._tooltipTranslationHook) {
            this.removeTranslationHook(this._tooltipTranslationHook);
        }
        else {
            if (this._tooltip)
                this._disposeTooltip();
            this._tooltip = new L.Draw.Tooltip(this.map);
            this._onMouseMove = ({ latlng }) => this._tooltip.updatePosition(latlng);
            ["mousemove", "touchmove", "MSPointerMove"].forEach(eType => this.map.on(eType, this._onMouseMove));
            if (this._mouseLatLng)
                this._onMouseMove({ latlng: this._mouseLatLng });
        }
        if (translationKey in this.translations) {
            this._tooltipTranslationHook = this.addTranslationHook(() => this._tooltip.updateContent({ text: this.translations[translationKey] }));
        }
        else {
            this._tooltip.updateContent({ text: translationKey });
        }
        if (error)
            this._tooltip.showAsError();
        else
            this._tooltip.removeError();
        return this._tooltip;
    }
    _disposeTooltip() {
        if (this._onMouseMove)
            ["mousemove", "touchmove", "MSPointerMove"].forEach(eType => this.map.off(eType, this._onMouseMove));
        this._onMouseMove = undefined;
        if (this._tooltip)
            this._tooltip.dispose();
        this._tooltipTranslationHook && this.removeTranslationHook(this._tooltipTranslationHook);
        this._tooltip = undefined;
    }
    _showDialog(container, onClose) {
        const _container = document.createElement("div");
        _container.className = "laji-map-dialog panel panel-default panel-body";
        if (this._dialogRoot === document.body) {
            _container.className += " fixed";
        }
        _container.appendChild(container);
        function close(e) {
            if (onClose)
                onClose(e);
        }
        this.showClosableElement(_container, close, !!"showBlocker", () => this._dialogRoot);
    }
    setLineTransectGeometry(geometry, events) {
        console.warn("line transect mixin not included!");
    }
    setGoogleApiKey(googleApiKey) {
        this.googleApiKey = googleApiKey;
    }
    setGoogleSearchUrl(url) {
        this.googleSearchUrl = url;
    }
    setGeocodingProvider() {
        this.providers = [
            ["Google", new GoogleProvider({ params: { key: this.googleApiKey, language: this.lang, region: "fi" } })],
            ["MML", new mml_provider_1.default({ params: { language: this.lang, region: "fi" } })]
        ];
        dependency_utils_1.provide(this, "geocodingProvider");
    }
    geocode(query, additional, zoom, provider = "google") {
        if (!this.providers) {
            console.warn("googleApiKey not provided for geocode");
            return;
        }
        const [, _provider] = provider === "mml"
            ? this.providers[1]
            : this.providers[0];
        _provider.search(Object.assign({ query }, (additional || {}))).then((results = []) => {
            if (!this.map || !results.length)
                return;
            const [first] = results;
            const { x, y } = first;
            this.map.panTo([y, x], { animate: false });
            if (zoom) {
                this.setNormalizedZoom(zoom);
            }
        });
    }
    setViewLocked(value) {
        if (!dependency_utils_1.depsProvided(this, "setViewLocked", arguments))
            return;
        const method = value ? "disable" : "enable";
        this.map.dragging[method]();
        this.map.touchZoom[method]();
        this.map.smoothWheelZoom[method]();
        this.map.boxZoom[method]();
        this.map.keyboard[method]();
        if (this.map.tap) {
            this.map.tap[method]();
        }
        this._disableDblClickZoom = value;
        this.viewLocked = value;
        if (dependency_utils_1.isProvided(this, "viewLocked")) {
            this.setClickBeforeZoomAndPan(this._clickBeforeZoomAndPan);
        }
        dependency_utils_1.provide(this, "viewLocked");
    }
    toggleFullscreen() {
        this._fullscreen ? this.setFullscreenOff() : this.setFullscreenOn();
    }
    setFullscreenOn() {
        this._fullscreen = true;
        this._fullscreenElem = document.createElement("div");
        this._fullscreenElem.className = "laji-map-fullscreen";
        document.body.appendChild(this._fullscreenElem);
        this._beforeFullscreen = {
            rootElem: this.rootElem,
            bodyAsDialogRoot: this._dialogRoot === document.body,
            clickBeforeZoomAndPan: this._clickBeforeZoomAndPan,
            bodyOverflowY: document.body.style.overflowY
        };
        this.setRootElem(this._fullscreenElem);
        this.map.getContainer().focus();
        this.setBodyAsDialogRoot(false);
        this.setClickBeforeZoomAndPan(false);
        document.body.style.overflowY = "hidden";
        this._addKeyListener(globals_1.ESC, this.setFullscreenOff);
        this._fullscreenCloseElem = document.createElement("button");
        this._fullscreenCloseElem.addEventListener("click", () => this.setFullscreenOff());
        this._fullscreenCloseElem.className = "btn btn-danger fullscreen-exit";
        this._fullscreenTranslateHook = this.addTranslationHook(this._fullscreenCloseElem, "MapExitFullscreen");
        this.rootElem.appendChild(this._fullscreenCloseElem);
    }
    setFullscreenOff() {
        this._fullscreen = false;
        const { rootElem, bodyAsDialogRoot, clickBeforeZoomAndPan, bodyOverflowY } = this._beforeFullscreen;
        this.setRootElem(rootElem);
        this.setBodyAsDialogRoot(bodyAsDialogRoot);
        this.setClickBeforeZoomAndPan(clickBeforeZoomAndPan);
        document.body.style.overflowY = bodyOverflowY;
        document.body.removeChild(this._fullscreenElem);
        this._removeKeyListener(globals_1.ESC, this.setFullscreenOff);
        this._fullscreenCloseElem.remove();
        this.removeTranslationHook(this._fullscreenTranslateHook);
    }
}
__decorate([
    dependency_utils_1.dependsOn("rootElem")
], LajiMap.prototype, "setBodyAsDialogRoot", null);
__decorate([
    dependency_utils_1.dependsOn("rootElem", "map", "translations", "viewLocked")
], LajiMap.prototype, "setClickBeforeZoomAndPan", null);
__decorate([
    dependency_utils_1.dependsOn("rootElem", "center", "zoom")
], LajiMap.prototype, "_initializeMap", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "_initializeView", null);
__decorate([
    dependency_utils_1.dependsOn("tileLayer")
], LajiMap.prototype, "_swapToWorldOutsideFinland", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "_initializeMapEvents", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setEventListeners", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setTileLayerByName", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setAvailableTileLayers", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setAvailableTileLayerBlacklist", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setAvailableTileLayerWhitelist", null);
__decorate([
    dependency_utils_1.dependsOn("map", "view")
], LajiMap.prototype, "setTileLayers", null);
__decorate([
    dependency_utils_1.dependsOn("tileLayer")
], LajiMap.prototype, "setTileLayerOpacity", null);
__decorate([
    dependency_utils_1.dependsOn("tileLayer")
], LajiMap.prototype, "setOverlaysByName", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setAvailableOverlaysBlacklist", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setAvailableOverlaysWhitelist", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setAvailableOverlays", null);
__decorate([
    dependency_utils_1.dependsOn("map", "view")
], LajiMap.prototype, "setNormalizedZoom", null);
__decorate([
    dependency_utils_1.dependsOn("zoom", "view")
], LajiMap.prototype, "setCenter", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setLang", null);
__decorate([
    dependency_utils_1.dependsOn("data", "draw", "view")
], LajiMap.prototype, "zoomToData", null);
__decorate([
    dependency_utils_1.dependsOn("map", "translations")
], LajiMap.prototype, "setData", null);
__decorate([
    dependency_utils_1.dependsOn("map", "data")
], LajiMap.prototype, "setDraw", null);
__decorate([
    dependency_utils_1.dependsOn("data")
], LajiMap.prototype, "_setOnChangeForItem", null);
__decorate([
    dependency_utils_1.dependsOn("translations")
], LajiMap.prototype, "_updateContextMenuForLayer", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "_setLocateOn", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "_setLocateOff", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "_onLocationFound", null);
__decorate([
    dependency_utils_1.reflect(),
    dependency_utils_1.dependsOn("translations")
], LajiMap.prototype, "setGeocodingProvider", null);
__decorate([
    dependency_utils_1.dependsOn("map")
], LajiMap.prototype, "setViewLocked", null);
exports.default = LajiMap;
