"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
describe("Draw copy control", () => {
    const [lat, lng] = [60.5, 25.5];
    let map, control;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({
            draw: { geoData: { type: "Point", coordinates: [lng, lat] } },
            controls: {
                draw: {
                    copy: true
                }
            }
        });
        control = map.getCoordinateCopyControl();
    }));
    it("opens on click", () => __awaiter(void 0, void 0, void 0, function* () {
        const $button = control.$getButton();
        yield $button.click();
        expect(yield control.$getContainer().isPresent()).toBe(true);
    }));
    it("closes on close button click", () => __awaiter(void 0, void 0, void 0, function* () {
        yield control.$getCloseButton().click();
        expect(yield control.$getContainer().isPresent()).toBe(false);
    }));
    describe("converts", () => {
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            yield control.$getButton().click();
        }));
        const formats = [
            { format: "GeoJSON", getLatLng: str => JSON.parse(str).features[0].geometry.coordinates.reverse() },
            { format: "ISO6709", getLatLng: str => str.replace(/\nCRS.*$/, "").match(/[0-9]+\.?[0-9]+/g) },
            { format: "WKT", getLatLng: str => str.replace(/\PROJCS.*$/, "").match(/[0-9]+\.?[0-9]+/g).reverse() }
        ];
        const crss = [
            { crs: "WGS84", toWGS84: latLng => latLng },
            { crs: "YKJ", toWGS84: test_utils_1.ykjToWgs84 },
            { crs: "ETRS", toWGS84: test_utils_1.etrsToWgs84 }
        ];
        for (const { format, getLatLng } of formats) {
            describe(format, () => {
                for (const { crs, toWGS84 } of crss) {
                    it(crs, () => __awaiter(void 0, void 0, void 0, function* () {
                        yield control[format]();
                        yield control[crs]();
                        const latLng = getLatLng(yield control.getConverted()).map(c => +c);
                        const asWGS84 = toWGS84(latLng);
                        yield expect(asWGS84[0]).toBeCloseTo(lat);
                        yield expect(asWGS84[1]).toBeCloseTo(lng);
                    }));
                }
            });
        }
    });
});
