"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const utils_1 = require("laji-map/lib/utils");
describe("Coordinate input control", () => {
    let map, control;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({
            draw: true,
            controls: true
        });
        control = map.getCoordinateInputControl();
    }));
    it("opens on click", () => __awaiter(void 0, void 0, void 0, function* () {
        const $button = control.$getButton();
        yield $button.click();
        expect(yield control.$getContainer().isPresent()).toBe(true);
    }));
    it("is disabled when no latlng", () => __awaiter(void 0, void 0, void 0, function* () {
        const $submit = control.$getSubmit();
        expect(yield $submit.isEnabled()).toBe(false);
    }));
    it("is disabled with invalid latlng", () => __awaiter(void 0, void 0, void 0, function* () {
        const $submit = control.$getSubmit();
        yield control.enterLatLng(1231231231, 123092834);
        expect(yield $submit.isEnabled()).toBe(false);
    }));
    it("closes on close button click", () => __awaiter(void 0, void 0, void 0, function* () {
        yield control.$getCloseButton().click();
        expect(yield control.$getContainer().isPresent()).toBe(false);
    }));
    describe("accepts", () => {
        beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
            const $button = control.$getButton();
            yield $button.click();
        }));
        afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
            if (yield control.$getContainer().isPresent()) {
                yield control.$getCloseButton().click();
            }
        }));
        const getGeometry = () => map.e("getDraw().featureCollection.features[map.getDraw().featureCollection.features.length - 1].geometry");
        it("WGS84 point", () => __awaiter(void 0, void 0, void 0, function* () {
            const [lat, lng] = [60, 25];
            yield control.enterLatLng(lat, lng);
            expect(yield control.getCRS()).toBe("WGS84");
            yield control.$getSubmit().click();
            const mapCoordinates = (yield getGeometry()).coordinates;
            expect(mapCoordinates[0]).toBe(lng);
            expect(mapCoordinates[1]).toBe(lat);
        }));
        it("WGS84 point with decimal", () => __awaiter(void 0, void 0, void 0, function* () {
            const [lat, lng] = [60.5, 25.5];
            yield control.enterLatLng(lat, lng);
            expect(yield control.getCRS()).toBe("WGS84");
            yield control.$getSubmit().click();
            const mapCoordinates = (yield getGeometry()).coordinates;
            expect(mapCoordinates[0]).toBe(lng);
            expect(mapCoordinates[1]).toBe(lat);
        }));
        it("WGS84 point with negative lng", () => __awaiter(void 0, void 0, void 0, function* () {
            const [lat, lng] = [50, -3];
            yield control.enterLatLng(lat, lng);
            expect(yield control.getCRS()).toBe("WGS84");
            yield control.$getSubmit().click();
            const mapCoordinates = (yield getGeometry()).coordinates;
            expect(mapCoordinates[0]).toBe(lng);
            expect(mapCoordinates[1]).toBe(lat);
        }));
        it("YKJ point", () => __awaiter(void 0, void 0, void 0, function* () {
            const [lat, lng] = [6666666, 3333333];
            yield control.enterLatLng(lat, lng);
            expect(yield control.getCRS()).toBe("YKJ");
            yield control.$getSubmit().click();
            const geometry = yield getGeometry();
            expect(geometry.type).toBe("Point");
            expect(utils_1.reverseCoordinate(geometry.coordinates)).toEqual(test_utils_1.ykjToWgs84([lat, lng]));
            expect(geometry.coordinateVerbatim).toBe(`${lat}:${lng}`);
        }));
        it("YKJ grid", () => __awaiter(void 0, void 0, void 0, function* () {
            const [lat, lng] = [666666, 333333];
            const wgs84LatLngs = [
                [6666660, 3333330],
                [6666660, 3333340],
                [6666670, 3333340],
                [6666670, 3333330],
            ].map((latLng) => test_utils_1.ykjToWgs84(latLng));
            yield control.enterLatLng(lat, lng);
            expect(yield control.getCRS()).toBe("YKJ");
            yield control.$getSubmit().click();
            const geometry = yield getGeometry();
            expect(geometry.type).toBe("Polygon");
            expect(geometry.coordinates[0].length).toBe(5);
            expect(utils_1.reverseCoordinate(geometry.coordinates[0][0])).toEqual(wgs84LatLngs[0]);
            expect(utils_1.reverseCoordinate(geometry.coordinates[0][1])).toEqual(wgs84LatLngs[1]);
            expect(utils_1.reverseCoordinate(geometry.coordinates[0][2])).toEqual(wgs84LatLngs[2]);
            expect(utils_1.reverseCoordinate(geometry.coordinates[0][3])).toEqual(wgs84LatLngs[3]);
            expect(utils_1.reverseCoordinate(geometry.coordinates[0][4])).toEqual(wgs84LatLngs[0]);
            expect(geometry.coordinateVerbatim).toBe(`${lat}:${lng}`);
        }));
        it("ETRS point", () => __awaiter(void 0, void 0, void 0, function* () {
            const [lat, lng] = [6666666, 333333];
            yield control.enterLatLng(lat, lng);
            expect(yield control.getCRS()).toBe("ETRS-TM35FIN");
            yield control.$getSubmit().click();
            const geometry = yield getGeometry();
            expect(geometry.type).toBe("Point");
            expect(utils_1.reverseCoordinate(geometry.coordinates)).toEqual(test_utils_1.etrsToWgs84([lat, lng]));
            expect(geometry.coordinateVerbatim).toBe(`${lat}:${lng}`);
        }));
    });
});
