"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const utils = require("laji-map/lib/utils");
const protractor_1 = require("protractor");
describe("Drawing", () => {
    const getLastGeometry = () => map.e("getDraw().featureCollection.features[map.getDraw().featureCollection.features.length - 1].geometry");
    let map;
    let control;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({
            draw: true,
            controls: {
                draw: true
            }
        });
        control = map.getDrawControl();
    }));
    const clear = () => map.e("clearDrawData()");
    describe("point", () => {
        afterAll(clear);
        it("can be drawn", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.drawMarker();
            const geometry = yield getLastGeometry();
            expect(geometry.type).toBe("Point");
        }));
        it("coordinates length", () => __awaiter(void 0, void 0, void 0, function* () {
            const geometry = yield getLastGeometry();
            expect(geometry.type).toBe("Point");
            expect(`${geometry.coordinates[0]}`.split(".")[1].length).toBeLessThan(7);
            expect(`${geometry.coordinates[1]}`.split(".")[1].length).toBeLessThan(7);
        }));
        it("coordinates are wrapped", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clear();
            yield map.e("setCenter([60, 300])");
            yield map.drawMarker();
            const geometry = yield getLastGeometry();
            for (const c of geometry.coordinates) {
                expect(c).toBeLessThan(180);
                expect(c).toBeGreaterThan(-180);
            }
        }));
        it("can be set editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clear();
            yield map.drawMarker();
            yield map.doubleClickAt(0, -10);
            expect(yield protractor_1.$(".leaflet-marker-draggable").isPresent()).toBe(true);
        }));
        // Drag doesn't work...
        // it("can be moved when editing", async () => {
        // 	await map.drag([0, 0], [-100, -100]);
        // });
        it("can finish editing", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.clickAt(20, 20);
            expect(yield protractor_1.$(".leaflet-marker-draggable").isPresent()).toBe(false);
        }));
    });
    describe("polyline", () => {
        afterAll(clear);
        const addLine = () => __awaiter(void 0, void 0, void 0, function* () {
            yield control.$getPolylineButton().click();
            yield map.clickAt(0, 0);
            yield protractor_1.browser.sleep(test_utils_1.SAFE_CLICK_WAIT);
            yield map.clickAt(10, 0);
            yield protractor_1.browser.sleep(test_utils_1.SAFE_CLICK_WAIT);
            yield map.clickAt(20, 10);
            yield protractor_1.browser.sleep(test_utils_1.SAFE_CLICK_WAIT);
            yield map.clickAt(20, 10);
        });
        it("can be drawn", () => __awaiter(void 0, void 0, void 0, function* () {
            yield addLine();
            const geometry = yield getLastGeometry();
            expect(geometry.type).toBe("LineString");
        }));
        it("can be set editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.doubleClickAt(0, 0);
            expect(yield protractor_1.$$(".leaflet-marker-draggable").count()).toBeGreaterThan(0);
        }));
        it("can finish editing", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.clickAt(-30, -30);
            expect(yield protractor_1.$$(".leaflet-marker-draggable").count()).toBe(0);
        }));
        it("coordinates length", () => __awaiter(void 0, void 0, void 0, function* () {
            const geometry = yield getLastGeometry();
            expect(geometry.coordinates.length).toBe(3);
            for (const c of geometry.coordinates) {
                expect(`${c[0]}`.split(".")[1].length).toBeLessThan(7);
                expect(`${c[1]}`.split(".")[1].length).toBeLessThan(7);
            }
        }));
        it("coordinates are wrapped", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clear();
            yield map.e("setCenter([60, 300])");
            yield addLine();
            const geometry = yield getLastGeometry();
            for (const coords of geometry.coordinates) {
                for (const c of coords) {
                    expect(c).toBeLessThan(180);
                    expect(c).toBeGreaterThan(-180);
                }
            }
        }));
    });
    describe("polygon", () => {
        afterAll(clear);
        const traveller = new test_utils_1.PointTraveller();
        // Clockwise
        const coordinates = [
            traveller.travel(0, 0),
            traveller.travel(0, -30),
            traveller.travel(30, 0),
            traveller.travel(0, 30),
            traveller.initial()
        ];
        const addPolygon = () => __awaiter(void 0, void 0, void 0, function* () {
            yield control.$getPolygonButton().click();
            for (const c of coordinates) {
                yield protractor_1.browser.sleep(test_utils_1.SAFE_CLICK_WAIT);
                yield map.clickAt(...c);
            }
        });
        it("can be drawn", () => __awaiter(void 0, void 0, void 0, function* () {
            yield addPolygon();
            const geometry = yield getLastGeometry();
            expect(geometry.type).toBe("Polygon");
            expect(geometry.coordinates.length).toBe(1);
            expect(geometry.coordinates[0].length).toBe(coordinates.length);
        }));
        it("can be set editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.doubleClickAt(0, 0);
            expect(yield protractor_1.$$(".leaflet-marker-draggable").count()).toBeGreaterThan(0);
        }));
        it("can finish editing", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.clickAt(-30, -30);
            expect(yield protractor_1.$$(".leaflet-marker-draggable").count()).toBe(0);
        }));
        it("coordinates length", () => __awaiter(void 0, void 0, void 0, function* () {
            const geometry = yield getLastGeometry();
            for (const c of geometry.coordinates[0]) {
                expect(`${c[0]}`.split(".")[1].length).toBeLessThan(7);
                expect(`${c[1]}`.split(".")[1].length).toBeLessThan(7);
            }
        }));
        it("coordinates are counter clockwise", () => __awaiter(void 0, void 0, void 0, function* () {
            const geometry = yield getLastGeometry();
            expect(utils.coordinatesAreClockWise(geometry.coordinates[0])).toBe(false);
        }));
        it("coordinates are counter clockwise when drawn counter clockwise direction", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clear();
            yield control.$getPolygonButton().click();
            for (const c of coordinates.slice(0).reverse()) {
                yield protractor_1.browser.sleep(test_utils_1.SAFE_CLICK_WAIT);
                yield map.clickAt(...c);
            }
            const geometry = yield getLastGeometry();
            expect(utils.coordinatesAreClockWise(geometry.coordinates[0])).toBe(false);
        }));
        it("coordinates are wrapped", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clear();
            yield map.e("setCenter([60, 300])");
            yield addPolygon();
            const geometry = yield getLastGeometry();
            for (const coords of geometry.coordinates[0]) {
                for (const c of coords) {
                    expect(c).toBeLessThan(180);
                    expect(c).toBeGreaterThan(-180);
                }
            }
        }));
    });
    describe("rectangle", () => {
        afterAll(clear);
        it("can be drawn", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.drawRectangle();
            const geometry = yield getLastGeometry();
            expect(geometry.type).toBe("Polygon");
            expect(geometry.coordinates.length).toBe(1);
            expect(geometry.coordinates[0].length).toBe(5);
        }));
        it("coordinates length", () => __awaiter(void 0, void 0, void 0, function* () {
            const geometry = yield getLastGeometry();
            geometry.coordinates[0].forEach((c) => __awaiter(void 0, void 0, void 0, function* () {
                expect(`${c[0]}`.split(".")[1].length).toBeLessThan(7);
                expect(`${c[1]}`.split(".")[1].length).toBeLessThan(7);
            }));
        }));
        it("coordinates are wrapped", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clear();
            yield map.e("setCenter([60, 300])");
            yield map.drawRectangle();
            const geometry = yield getLastGeometry();
            for (const coords of geometry.coordinates[0]) {
                for (const c of coords) {
                    expect(c).toBeLessThan(180);
                    expect(c).toBeGreaterThan(-180);
                }
            }
        }));
        it("coordinates are counter clockwise", () => __awaiter(void 0, void 0, void 0, function* () {
            const geometry = yield getLastGeometry();
            expect(utils.coordinatesAreClockWise(geometry.coordinates[0])).toBe(false);
        }));
        const drags = [
            [-10, 10],
            [10, -10],
            [-10, -10]
        ];
        it("coordinates are counter clockwise when drawn from any direction", () => __awaiter(void 0, void 0, void 0, function* () {
            const geometry = yield getLastGeometry();
            expect(utils.coordinatesAreClockWise(geometry.coordinates[0])).toBe(false);
            for (const drag of drags) {
                yield clear();
                yield control.$getRectangleButton().click();
                const traveller = new test_utils_1.PointTraveller();
                yield map.drag(traveller.initial(), traveller.travel(...drag));
                const lastGeometry = yield getLastGeometry();
                expect(utils.coordinatesAreClockWise(lastGeometry.coordinates[0])).toBe(false);
            }
        }));
        describe("editing", () => {
            const lastDestination = drags[drags.length - 1];
            let geometryBeforeEdit;
            beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
                geometryBeforeEdit = yield getLastGeometry();
            }));
            it("can be started", () => __awaiter(void 0, void 0, void 0, function* () {
                yield map.doubleClickAt(...lastDestination);
                expect(yield protractor_1.$$(".leaflet-marker-draggable").count()).toBeGreaterThan(0, "Rectangle wasn't editable");
            }));
            it("can be finished", () => __awaiter(void 0, void 0, void 0, function* () {
                const traveller = new test_utils_1.PointTraveller(...lastDestination);
                yield map.drag(traveller.initial(), traveller.travel(-10, -10));
                yield map.clickAt(...traveller.travel(-10, -10));
            }));
            it("geometry is changed", () => __awaiter(void 0, void 0, void 0, function* () {
                const lastGeometry = yield getLastGeometry();
                expect(lastGeometry).not.toEqual(geometryBeforeEdit);
            }));
            it("coordinates are counter clockwise", () => __awaiter(void 0, void 0, void 0, function* () {
                const lastGeometry = yield getLastGeometry();
                expect(utils.coordinatesAreClockWise(lastGeometry.coordinates[0])).toBe(false);
            }));
            it("coordinates length", () => __awaiter(void 0, void 0, void 0, function* () {
                const lastGeometry = yield getLastGeometry();
                lastGeometry.coordinates[0].forEach((c) => __awaiter(void 0, void 0, void 0, function* () {
                    expect(`${c[0]}`.split(".")[1].length).toBeLessThan(7);
                    expect(`${c[1]}`.split(".")[1].length).toBeLessThan(7);
                }));
            }));
        });
    });
    describe("circle", () => {
        afterAll(clear);
        const addCircle = () => __awaiter(void 0, void 0, void 0, function* () {
            yield control.$getCircleButton().click();
            yield map.drag([0, 0], [10, 0]);
        });
        it("can be drawn", () => __awaiter(void 0, void 0, void 0, function* () {
            yield addCircle();
            const geometry = yield getLastGeometry();
            expect(geometry.type).toBe("Point");
            expect(geometry.radius).toBeGreaterThan(0);
            expect(geometry.coordinates.length).toBe(2);
        }));
        it("can be set editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.doubleClickAt(0, 0);
            expect(yield protractor_1.$$(".leaflet-marker-draggable").count()).toBeGreaterThan(0);
        }));
        it("can finish editing", () => __awaiter(void 0, void 0, void 0, function* () {
            yield map.clickAt(-10, -10);
            expect(yield protractor_1.$$(".leaflet-marker-draggable").count()).toBe(0);
        }));
        it("coordinates length", () => __awaiter(void 0, void 0, void 0, function* () {
            const { coordinates } = yield getLastGeometry();
            expect(`${coordinates[0]}`.split(".")[1].length).toBeLessThan(7);
            expect(`${coordinates[1]}`.split(".")[1].length).toBeLessThan(7);
        }));
        it("coordinates are wrapped", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clear();
            yield map.e("setCenter([60, 300])");
            yield addCircle();
            const geometry = yield getLastGeometry();
            for (const c of geometry.coordinates) {
                expect(c).toBeLessThan(180);
                expect(c).toBeGreaterThan(-180);
            }
        }));
    });
});
