"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SAFE_CLICK_WAIT = exports.LatLngTraveller = exports.PointTraveller = exports.etrsToWgs84 = exports.ykjToWgs84 = exports.createMap = exports.DrawControlPageObject = exports.DeleteControl = exports.MapPageObject = void 0;
const { HOST, PORT, VERBOSE, DELAY } = process.env;
const protractor_1 = require("protractor");
const utils = require("laji-map/lib/utils");
const joinParams = (params) => Object.keys(params).reduce((s, a, i) => `${s}${i === 0 ? "?" : "&"}${a}=${JSON.stringify(params[a])}`, "");
const navigateToMap = (params = {}) => __awaiter(void 0, void 0, void 0, function* () {
    const url = `http://${HOST}:${PORT}${joinParams(Object.assign({ testMode: true }, params))}`;
    VERBOSE && console.log(url);
    yield protractor_1.browser.get(url);
});
function getControlButton(name) {
    return protractor_1.$(`.button-${name.replace(/\./g, "_")}`);
}
class MapPageObject {
    constructor(options = {}) {
        this.getDrawData = () => this.e("getDraw().featureCollection");
        this.options = options;
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            yield navigateToMap(this.options);
            if (DELAY) {
                yield protractor_1.browser.sleep(parseInt(DELAY));
            }
        });
    }
    e(path, ...params) {
        return protractor_1.browser.executeScript(`return window.map.${path}`, ...params);
    }
    $getElement() {
        return protractor_1.$(".laji-map");
    }
    mouseMove(x, y) {
        return protractor_1.browser.actions()
            .mouseMove(this.$getElement().getWebElement(), { x: x + 1, y })
            .mouseMove(this.$getElement().getWebElement(), { x, y })
            .perform();
    }
    clickAt(x, y) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.mouseMove(x, y);
            return protractor_1.browser.actions()
                .click().perform();
        });
    }
    doubleClickAt(x, y) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.mouseMove(x, y);
            yield protractor_1.browser.sleep(500);
            return protractor_1.browser.actions()
                .doubleClick().perform();
        });
    }
    drag([x, y], [x2, y2]) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.mouseMove(x, y);
            yield protractor_1.browser.actions()
                .mouseDown()
                .perform();
            yield this.mouseMove(x2, y2);
            return protractor_1.browser.actions()
                .mouseUp()
                .perform();
        });
    }
    drawMarker(x = 0, y = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getDrawControl().$getMarkerButton().click();
            yield this.clickAt(x, y);
        });
    }
    drawRectangle() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getDrawControl().$getRectangleButton().click();
            yield this.drag([0, 0], [10, 10]);
        });
    }
    getCoordinateInputControl() { return new CoordinateInputControlPageObject(); }
    getCoordinateUploadControl() { return new CoordinateUploadControlPageObject(); }
    getCoordinateCopyControl() { return new CoordinateCopyControlPageObject(); }
    getDrawControl() { return new DrawControlPageObject(); }
    getTileLayersControl() { return new TilelayersControlPageObject(this); }
    getDeleteControl() { return new DeleteControl(); }
}
exports.MapPageObject = MapPageObject;
class DeleteControl {
    start() {
        return getControlButton("drawUtils.delete").click();
    }
    finish() {
        return protractor_1.$$(".leaflet-draw-actions a").last().click();
    }
}
exports.DeleteControl = DeleteControl;
class CoordinateInputControlPageObject {
    $getButton() {
        return getControlButton("drawUtils.coordinateInput");
    }
    $getContainer() {
        return protractor_1.$(".laji-map-coordinates").element(protractor_1.by.xpath("..")); // eslint-disable-line protractor/no-by-xpath
    }
    $getCloseButton() {
        return this.$getContainer().$(".close");
    }
    enterLatLng(lat, lng) {
        return __awaiter(this, void 0, void 0, function* () {
            yield protractor_1.$("#laji-map-coordinate-input-lat").sendKeys(lat);
            return protractor_1.$("#laji-map-coordinate-input-lng").sendKeys(lng);
        });
    }
    getCRS() {
        return this.$getContainer().$(".crs-info span:last-child").getText();
    }
    $getSubmit() {
        return this.$getContainer().$("button[type=\"submit\"]");
    }
}
class CoordinateUploadControlPageObject {
    $getButton() {
        return getControlButton("drawUtils.upload");
    }
    $getContainer() {
        return protractor_1.$(".laji-map-coordinate-upload").element(protractor_1.by.xpath("..")); // eslint-disable-line protractor/no-by-xpath
    }
    $getCloseButton() {
        return this.$getContainer().$(".close");
    }
    type(text) {
        return this.$getContainer().$("textarea").sendKeys(text);
    }
    getCRS() {
        return this.$getContainer().$(".crs-info span:last-child").getText();
    }
    getFormat() {
        return this.$getContainer().$(".format-info span:last-child").getText();
    }
    $getSubmit() {
        return this.$getContainer().$("button[type=\"submit\"]");
    }
}
class CoordinateCopyControlPageObject {
    $getButton() {
        return getControlButton("drawUtils.copy");
    }
    $getContainer() {
        return protractor_1.$(".laji-map-draw-copy-table").element(protractor_1.by.xpath("..")); // eslint-disable-line protractor/no-by-xpath
    }
    $getCloseButton() {
        return this.$getContainer().$(".close");
    }
    getConverted() {
        return this.$getContainer().$("textarea").getAttribute("value");
    }
    GeoJSON() {
        return this.$getContainer().element(protractor_1.by.cssContainingText("a", "GeoJSON")).click();
    }
    ISO6709() {
        return this.$getContainer().element(protractor_1.by.cssContainingText("a", "ISO 6709")).click();
    }
    WKT() {
        return this.$getContainer().element(protractor_1.by.cssContainingText("a", "WKT")).click();
    }
    WGS84() {
        return this.$getContainer().element(protractor_1.by.cssContainingText("a", "WGS84")).click();
    }
    YKJ() {
        return this.$getContainer().element(protractor_1.by.cssContainingText("a", "YKJ")).click();
    }
    ETRS() {
        return this.$getContainer().element(protractor_1.by.cssContainingText("a", "ETRS-TM35FIN")).click();
    }
}
class DrawControlPageObject {
    $getButton(name) {
        return protractor_1.$(`.leaflet-draw-draw-${name}`);
    }
    $getMarkerButton() {
        return this.$getButton("marker");
    }
    $getPolygonButton() {
        return this.$getButton("polygon");
    }
    $getRectangleButton() {
        return this.$getButton("rectangle");
    }
    $getCircleButton() {
        return this.$getButton("circle");
    }
    $getPolylineButton() {
        return this.$getButton("polyline");
    }
}
exports.DrawControlPageObject = DrawControlPageObject;
class TilelayersControlPageObject {
    constructor(mapPO) {
        this.mapPO = mapPO;
    }
    $getContainer() {
        return protractor_1.$("div.laji-map-control-layers");
    }
    $getButton() {
        return protractor_1.$(".leaflet-control-layers-toggle");
    }
    showList() {
        return protractor_1.browser.actions().mouseMove(this.$getButton()).perform();
    }
    $getFinnishList() {
        return this.$getContainer().$(".finnish-list");
    }
    $getWorldList() {
        return this.$getContainer().$(".world-list");
    }
    selectFinnishList() {
        return this.$getFinnishList().$("legend").click();
    }
    selectWorldList() {
        return this.$getWorldList().$("legend").click();
    }
    $getOverlayList() {
        return this.$getContainer().$(".overlay-list");
    }
    $getLayerElement(name) {
        return this.$getContainer().$(`#${name}`);
    }
}
const createMap = (options) => __awaiter(void 0, void 0, void 0, function* () {
    const map = new MapPageObject(options);
    yield map.initialize();
    return map;
});
exports.createMap = createMap;
const ykjToWgs84 = (latLng) => utils.convertLatLng(latLng, "EPSG:2393", "WGS84");
exports.ykjToWgs84 = ykjToWgs84;
const etrsToWgs84 = (latLng) => utils.convertLatLng(latLng, "+proj=utm +zone=35 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs", "WGS84");
exports.etrsToWgs84 = etrsToWgs84;
class PointTraveller {
    constructor(x = 0, y = 0) {
        this.x = x;
        this.y = y;
        this.initX = x;
        this.initY = y;
    }
    travel(xAmount, yAmount) {
        this.x = this._x(this.x, yAmount);
        this.y = this._y(this.y, xAmount);
        return this.return(this.x, this.y);
    }
    initial() {
        return this.return(this.initX, this.initY);
    }
    return(x, y) {
        return [x, y];
    }
    _x(curVal, amount) { return curVal + amount; }
    _y(curVal, amount) { return curVal + amount; }
}
exports.PointTraveller = PointTraveller;
class LatLngTraveller extends PointTraveller {
    constructor(lat, lng, options = {}) {
        super(lng, lat);
        if (options.onlyForward) {
            this.onlyForward = true;
        }
    }
    _y(curVal, amount) {
        if (this.onlyForward && amount < 0)
            throw "Can travel only positive amounts if 'onlyForward' true";
        return super._y(curVal, -amount);
    }
    _x(curVal, amount) {
        if (this.onlyForward && amount < 0)
            throw "Can travel only positive amounts if 'onlyForward' true";
        return super._x(curVal, amount);
    }
    northEast(north, east) {
        return this.travel(east, north);
    }
    return(x, y) {
        return [y, x];
    }
}
exports.LatLngTraveller = LatLngTraveller;
exports.SAFE_CLICK_WAIT = 300;
