"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
let map;
describe("Initializing", () => {
    it("tileLayers overrides tileLayerName", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({
            tileLayerName: "ortokuva",
            tileLayers: {
                layers: {
                    maastokartta: true
                }
            }
        });
        yield expect(yield map.e("tileLayerName")).toBe("maastokartta");
    }));
    it("with tileLayerName work", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({
            tileLayerName: "maastokartta",
        });
        yield expect(yield map.e("tileLayerName")).toBe("maastokartta");
    }));
    it("with finnish tilelayer and view outside Finland swaps to world map", () => __awaiter(void 0, void 0, void 0, function* () {
        const congo = {
            lat: 79.3499057749654,
            lng: 21.160612106323246
        };
        map = yield test_utils_1.createMap({
            tileLayerName: "taustakartta",
            center: congo
        });
        yield expect(yield map.e("tileLayerName")).toBe("openStreetMap");
    }));
});
describe("Tile layers control", () => {
    let control;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        control = map.getTileLayersControl();
    }));
    it("is rendered", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({ controls: true });
        expect(yield control.$getContainer().isPresent()).toBe(true);
    }));
    it("opens on click", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({ controls: true });
        expect(yield control.$getFinnishList().isDisplayed()).toBe(false);
        yield control.showList();
        expect(yield control.$getFinnishList().isDisplayed()).toBe(true);
    }));
    const tests = {
        "doesn't render finnish layers when not available": {
            options: {
                tileLayerName: "openStreetMap",
                availableTileLayerNamesWhitelist: ["openStreetMap"]
            },
            test: () => __awaiter(void 0, void 0, void 0, function* () {
                yield control.showList();
                expect(yield control.$getWorldList().isPresent()).toBe(true);
                expect(yield control.$getWorldList().$("legend").isPresent()).toBe(true);
                expect(yield control.$getWorldList().$("legend").getText()).toBe(yield map.e("translations.Maps"));
                expect(yield control.$getFinnishList().isPresent()).toBe(false);
            })
        },
        "doesn't render world layers when not available": {
            options: {
                tileLayerName: "taustakartta",
                availableTileLayerNamesWhitelist: ["taustakartta"]
            },
            test: () => __awaiter(void 0, void 0, void 0, function* () {
                yield control.showList();
                expect(yield control.$getFinnishList().isPresent()).toBe(true);
                expect(yield control.$getFinnishList().$("legend").getText()).toBe(yield map.e("translations.Maps"));
                expect(yield control.$getWorldList().isPresent()).toBe(false);
            })
        },
        "doesn't render overlays when not available": {
            options: {
                availableOverlayNameWhitelist: []
            },
            test: () => __awaiter(void 0, void 0, void 0, function* () {
                yield control.showList();
                expect(yield control.$getOverlayList().isPresent()).toBe(false);
            })
        }
    };
    for (const name of Object.keys(tests)) {
        let { options, test } = tests[name];
        options = Object.assign({ controls: true }, options);
        it(`${name} after initialization`, () => __awaiter(void 0, void 0, void 0, function* () {
            map = yield test_utils_1.createMap(options);
            yield test();
        }));
        it(`${name} when set after initialization`, () => __awaiter(void 0, void 0, void 0, function* () {
            map = yield test_utils_1.createMap({ controls: true });
            yield map.e(`setOptions(${JSON.stringify(options)})`);
            yield test();
        }));
    }
    it("adds layer when added to options after control initialization", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({ controls: true, availableTileLayerNamesBlacklist: ["maastokartta"] });
        yield control.showList();
        expect(yield control.$getLayerElement("maastokartta").isDisplayed()).toBe(false);
        yield map.e("setOption('availableTileLayerNamesBlacklist', [])");
        yield expect(yield control.$getLayerElement("maastokartta").isDisplayed()).toBe(true);
    }));
    it("removes layer when removed from options after control initialization", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({ controls: true, availableTileLayerNamesBlacklist: [] });
        yield control.showList();
        expect(yield control.$getLayerElement("maastokartta").isDisplayed()).toBe(true);
        yield map.e("setOption('availableTileLayerNamesBlacklist', ['maastokartta'])");
        expect(yield control.$getLayerElement("maastokartta").isDisplayed()).toBe(false);
    }));
    it("Sets openStreetMap as active when changing to world projection and there are no visible world layers", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({ controls: true, tileLayerName: "taustakartta" });
        yield control.showList();
        yield control.selectWorldList();
        const layerOptions = yield map.e("getTileLayers()");
        expect(layerOptions.layers.openStreetMap.visible).toBe(true);
    }));
    it("Sets taustakartta as active when changing to finnish projection and there are no visible finnish layers", () => __awaiter(void 0, void 0, void 0, function* () {
        map = yield test_utils_1.createMap({ controls: true, tileLayerName: "openStreetMap" });
        yield control.showList();
        yield control.selectFinnishList();
        const layerOptions = yield map.e("getTileLayers()");
        expect(layerOptions.layers.taustakartta.visible).toBe(true);
    }));
});
