/// <reference path="leaflet.nontiledlayer-types.d.ts" />
import * as L from "leaflet";
import * as G from "geojson";
import LajiMap from "./map";
import { DrawOptions, DataItemType, LajiMapEvent, TileLayersOptions } from "./map.defs";
import { ControlsOptions, CustomControl, FormatDetectorOptions, InternalControlsOptions } from "./controls.defs";
declare type Constructor<LM> = new (...args: any[]) => LM;
export default function LajiMapWithControls<LM extends Constructor<LajiMap>>(Base: LM): {
    new (...args: any[]): {
        getOptionKeys(): any;
        _setLang(): void;
        setLang(lang: any): void;
        _initializeMapEvents(): void;
        _toggleLocate(): void;
        _setLocateOn(triggerEvent?: boolean): void;
        _setLocateOff(): void;
        _onLocationFound(e: L.LocationEvent): void;
        setTileLayers(options: TileLayersOptions): void;
        _updateUserLocate(value: boolean): void;
        _updateMapControls(): void;
        _updateDrawControls(): void;
        updateDrawData(item: DrawOptions): void;
        _updateCustomControls(): void;
        _updateLineTransectControls(): void;
        setControlsWarn(controlSettings: any): void;
        setControls(controlSettings: ControlsOptions | boolean): void;
        setCustomControls(controls?: CustomControl[]): void;
        _controlIsAllowed(name: any, custom?: boolean): any;
        _addControl(name: any, control: L.Control): void;
        _createControlButton(that: any, container: any, fn: any, name?: any): HTMLElement;
        _createControlItem(that: any, container: any, glyphName: any, title: any, fn: any, name: any): HTMLElement;
        _getDrawControl(): L.Control.Draw;
        getZoomControl(): L.Control.Zoom;
        setTileLayerOpacity(opacity: any, triggerEvent?: any): void;
        setLineTransectGeometry(feature: any, undo: any): void;
        resetDrawUndoStack(): void;
        _updateDrawUndoStack(events: LajiMapEvent[] | LajiMapEvent, prevFeatureCollection: any, prevActiveIdx?: any): void;
        drawUndo(): void;
        drawRedo(): void;
        _updateUndoButton(buttonName: any, history: any, historyPointer: any): void;
        _updateRedoButton(buttonName: any, history: any, historyPointer: any): void;
        updateLTUndoButton(): void;
        updateLTRedoButton(): void;
        updateDrawUndoButton(): void;
        updateDrawRedoButton(): void;
        openCoordinatesInputDialog(): void;
        openDrawCopyDialog(): void;
        createFormatDetectorElem(options?: FormatDetectorOptions): {
            elem: HTMLElement;
            validate: (value?: string) => {
                valid: boolean;
                geoJSON: G.GeoJSON;
            };
            unmount: () => void;
        };
        openDrawUploadDialog(): void;
        _joinTranslations(...words: any[]): string;
        _updateContextMenu(): void;
        getGoogleGeocodingControl(): any;
        setFullscreenOn(): void;
        setFullscreenOff(): void;
        _updateFullscreenControl(): void;
        triggerDrawing(featureType: DataItemType): void;
        shouldNotPreventScrolling(): boolean;
        googleApiKey: string;
        googleSearchUrl: string;
        container: HTMLElement;
        mapElem: HTMLElement;
        blockerElem: HTMLElement;
        rootElem: HTMLElement;
        translations: any;
        data: import("./map.defs").Data[];
        map: L.Map;
        _onDrawReverse: (layer: import("./map.defs").DataItemLayer) => void;
        _onDrawRemove: (layer: import("./map.defs").DataItemLayer) => void;
        idxsToIds: {
            [dataIdx: number]: {
                [featureIdx: number]: number;
            };
        };
        idsToIdxs: {
            [dataIdx: number]: {
                [id: number]: number;
            };
        };
        idsToIdxTuples: {
            [id: number]: import("./map.defs").IdxTuple;
        };
        editIdxTuple: import("./map.defs").IdxTuple;
        drawing: boolean;
        _drawHistoryPointer: number;
        _drawHistory: import("./map.defs").DrawHistoryEntry[];
        polyline: L.Polyline<G.MultiLineString | G.LineString, any>;
        userLocation: {
            latlng: L.LatLng;
            accuracy: number;
        };
        userLocationMarker: L.CircleMarker<any>;
        userLocationRadiusMarker: L.Circle<any>;
        userLocationLayer: L.LayerGroup<any>;
        popup: any;
        featurePopupOffset: number;
        markerPopupOffset: number;
        lang: import("./map.defs").Lang;
        tileLayerOpacity: number;
        onSetLangHooks: (() => void)[];
        options: import("./map.defs").Options;
        _dialogRoot: HTMLElement;
        _openDialogs: any[];
        _closeDialog: (e?: Event) => void;
        draw: import("./map.defs").Data;
        drawIdx: number;
        _draftDrawLayer: import("./map.defs").DataItemLayer;
        _idxsToHovered: {
            [dataIdx: number]: {
                [featureIdx: number]: boolean;
            };
        };
        _idxsToContextMenuOpen: {
            [dataIdx: number]: {
                [id: number]: boolean;
            };
        };
        locate: boolean | [(event: L.LocationEvent) => void, (error: L.ErrorEvent) => void];
        _located: boolean;
        locatingOn: boolean;
        popupOnHover: boolean;
        popupCounter: number;
        onPopupClose: () => void;
        _drawReverseLayers: L.Polyline<G.MultiLineString | G.LineString, any>[];
        _drawRemoveLayers: import("./map.defs").DataItemLayer[];
        _zoomToData: boolean | import("./map.defs").LajiMapFitBoundsOptions;
        _disableDblClickZoom: boolean;
        dictionary: {
            [lang: string]: any;
        };
        _domCleaners: (() => void)[];
        _documentEvents: {
            [eventName: string]: EventListener[];
        };
        zoom: number;
        center: L.LatLngExpression;
        tileLayer: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        overlaysByNames: {
            [name: string]: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        };
        availableOverlaysByNames: {
            [name: string]: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        };
        overlays: (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[];
        finnishTileLayers: {
            maastokartta?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            taustakartta?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            ortokuva?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            laser?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            afeGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            atlasGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            ykjGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            ykjGridLabels?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        };
        worldTileLayers: {
            openStreetMap?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            googleSatellite?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            cgrsGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        };
        tileLayers: {
            maastokartta?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            taustakartta?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            ortokuva?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            laser?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            openStreetMap?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            googleSatellite?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            afeGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            cgrsGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            atlasGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            geobiologicalProvinces?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            geobiologicalProvinceBorders?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            municipalities?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            counties?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            ely?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            forestVegetationZones?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            mireVegetationZones?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            threatenedSpeciesEvaluationZones?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            biodiversityForestZones?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            ykjGrid?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            ykjGridLabels?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            kiinteistojaotus?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            kiinteistotunnukset?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            currentProtectedAreas?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            plannedProtectedAreas?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            flyingSquirrelPredictionModel?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            birdAtlasSocietyGridZones?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
            barentsRegion?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        };
        tileLayerName: import("./map.defs").TileLayerName;
        _tileLayers: import("./map.defs").InternalTileLayersOptions;
        availableTileLayers: {
            [name: string]: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        };
        _listenedEvents: L.LeafletEventHandlerFnMap;
        _keyListeners: {
            [eventName: string]: {
                [key: string]: ((e: Event) => boolean | void)[];
            };
        };
        _swapToWorldFlag: boolean;
        _mouseLatLng: L.LatLng;
        _contextMenuLayer: L.Layer;
        _preventScrollDomCleaner: () => void;
        _onDrawStopPreventScrolling: () => void;
        _onDrawStartPreventScrolling: () => void;
        _preventScroll: boolean;
        _scrollPreventElem: HTMLElement;
        _scrollPreventTextElemContainer: HTMLElement;
        _scrollPreventTextElem: HTMLElement;
        _scrollPreventScrollListeners: [string, EventListener][];
        _startPreventScrollingTimeout: () => void;
        _onControlClickPreventScrolling: () => boolean;
        _onMouseDownPreventScrolling: EventListener;
        _onTouchPreventScrolling: EventListener;
        _preventScrolling: () => void;
        _showPreventAnimationTimeout: any;
        _showPreventShowTimeout: any;
        _showPreventHideTimeout: any;
        _showingPreventScroll: boolean;
        _LTEditPointIdxTuple: import("./map.defs").IdxTuple;
        _clickBeforeZoomAndPan: boolean;
        _origLatLngs: {
            [id: string]: L.LatLng[];
        };
        _tooltip: L.Draw.Tooltip;
        _tooltipTranslationHook: () => void;
        _onMouseMove: (e: any) => void;
        providers: ["Google" | "MML", import("leaflet-geosearch/lib/providers/provider").Provider<any, any>][];
        leafletOptions: L.MapOptions;
        _viewCriticalSection: number;
        _tileLayersSet: boolean;
        activeProjName: "finnish" | "world";
        _tileLayerOrder: string[];
        viewLocked: boolean;
        _locateParam: boolean | import("./map.defs").UserLocationOptions;
        locateOptions: import("./map.defs").UserLocationOptions;
        availableTileLayersWhitelist: import("./map.defs").TileLayerName[];
        availableTileLayersBlacklist: import("./map.defs").TileLayerName[];
        availableOverlaysBlacklist: import("./map.defs").OverlayName[];
        availableOverlaysWhitelist: import("./map.defs").OverlayName[];
        _initialized: boolean;
        mmlProj: L.Proj.CRS;
        _trySwapToFinnishOnInitialization: boolean;
        _fullscreen: boolean;
        _fullscreenElem: HTMLElement;
        _fullscreenCloseElem: HTMLElement;
        _fullscreenTranslateHook: () => void;
        _beforeFullscreen: {
            rootElem: HTMLElement;
            bodyAsDialogRoot: boolean;
            clickBeforeZoomAndPan: boolean;
            bodyOverflowY: string;
        };
        lajiGeoServerAddress: string;
        controls: L.Control[];
        _customControls: CustomControl[];
        layerControl: L.Control.Layers;
        controlItems: import("./controls.defs").ControlOptions[];
        _controlItemsByName: {
            [controlName: string]: import("./controls.defs").ControlOptions;
        };
        activeControl: L.Control;
        controlSettings: InternalControlsOptions;
        drawControl: L.Control.Draw;
        _locateOn: boolean;
        _controlButtons: {
            [controlName: string]: HTMLElement;
        };
        _opacitySetBySlide: boolean;
        _slider: any;
        _contextMenuItems: {
            [buttonName: string]: HTMLElement;
        };
        _internalTileLayersUpdate: boolean;
        _allPoints: L.CircleMarker<any>[];
        _hoveredIdxTuple: import("./map.defs").IdxTuple;
        LTFeature: import("./line-transect.defs").LineTransectFeature;
        _splitIdxTuple: import("./map.defs").IdxTuple;
        _splitPoint: L.LatLng;
        _lineLayers: L.Polyline<G.LineString, any>[][];
        _pointLayers: L.CircleMarker<any>[][];
        _pointIdsToIdxTuples: {
            [id: number]: import("./map.defs").IdxTuple;
        };
        _corridorLayers: L.Polygon<G.Polygon>[][];
        _allSegments: L.Polyline<G.LineString, any>[];
        _allCorridors: L.Polygon<G.Polygon>[];
        _lineSplitFn: (idxTuple: import("./map.defs").IdxTuple, splitPoint: L.LatLng) => void;
        _selectLTMode: "segment" | "line";
        _LTActiveIdx: number;
        _onLTChange: (events: import("./line-transect.defs").LineTransectEvent[]) => void;
        _LTDragging: boolean;
        _getLTFeatureStyle: (options: import("./line-transect.defs").GetLineTransectFeatureStyleOptions) => L.PathOptions;
        _getLTTooltip: (lineIdx: number, text: string, callback?: (callbackText: string) => void) => string;
        _LTHistory: import("./line-transect.defs").LineTransectHistoryEntry[];
        _LTHistoryPointer: number;
        _LTPrintMode: boolean;
        _LTEditable: boolean;
        _pointDragSnapMode: boolean;
        _dragCorridor: L.Polygon<G.Polygon>;
        _origLineTransect: L.FeatureGroup<any>;
        _pointLayerGroup: L.FeatureGroup<any>;
        _lineLayerGroup: L.FeatureGroup<any>;
        _corridorLayerGroup: L.FeatureGroup<any>;
        _overlappingNonadjacentPointIdxTuples: {
            [idxTupleString: string]: import("./map.defs").IdxTuple;
        };
        _overlappingAdjacentPointIdxTuples: {
            [idxTupleString: string]: import("./map.defs").IdxTuple;
        };
        _lineIdxsTupleStringsToLineGroupIdxs: {
            [idxTupleString: string]: number;
        };
        _groupIdxsToLineIdxs: {
            [groupIdx: number]: number[];
        };
        _LTStartText: L.Polyline<G.MultiLineString | G.LineString, any>;
        _LTGroups: L.FeatureGroup<any>[];
        _tooltipIdx: number;
        _overlappingPointDialogSegmentIdxTuple: import("./map.defs").IdxTuple;
        leafletIdsToCorridorLineIdxs: {
            [id: string]: number;
        };
        leafletIdsToCorridorSegmentIdxs: {
            [id: string]: number;
        };
        leafletIdsToFlatCorridorSegmentIdxs: {
            [id: string]: number;
        };
        leafletIdsToFlatPointIdxs: {
            [id: string]: number;
        };
        lineIdsToCorridorIds: {
            [id: string]: number;
        };
        corridorFlatIdxsToLeafletIds: {
            [id: string]: number;
        };
        _hoveredType: "point" | "corridor";
        _LTClickTimeout: number;
        _closebyPointIdxTuple: import("./map.defs").IdxTuple;
        _pointLTShiftMode: boolean;
        _onSelectLT: (idxTuple: import("./map.defs").IdxTuple) => void;
        _firstLTSegmentToRemoveIdx: import("./map.defs").IdxTuple;
        _LTPointExpander: L.CircleMarker<any>;
        _LTdragPoint: L.CircleMarker<any>;
        _LTContextMenuLayer: import("./line-transect.defs").SegmentLayer;
        _featureBeforePointDrag: import("./line-transect.defs").LineTransectFeature;
        _LTPointLatLngBeforeDrag: L.LatLng;
        _hoveringDragPoint: boolean;
        _dragPointStart: L.LatLng;
        _dragMouseStart: L.LatLng;
        _cutLine: L.Polygon<any>;
        _lineCutIdx: import("./map.defs").IdxTuple;
        _ltTooltip: L.Draw.Tooltip;
        messages: import("./line-transect.defs").TooltipMessages;
        setOptions(options?: import("./map.defs").Options): void;
        setOption(option: keyof import("./map.defs").Options, value: any): void;
        getOption(option: keyof import("./map.defs").Options): any;
        getOptions(): import("./map.defs").Options;
        setRootElem(rootElem: HTMLElement): void;
        setBodyAsDialogRoot(value?: boolean): void;
        setClickBeforeZoomAndPan(value?: boolean): void;
        getMMLProj(): L.CRS;
        _initializeMap(): void;
        _initializeView(): void;
        _isOutsideFinland(latLng: L.LatLngExpression): boolean;
        _swapToWorldOutsideFinland(latLng: L.LatLngExpression): void;
        keyHandlerForType(type: string, e: any): boolean;
        _addDocumentEventListener(type: string, fn: EventListener, useCapture?: boolean): void;
        _removeDocumentEventListener(type: string, fn: EventListener): void;
        _addKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
        _removeKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
        _triggerKeyEvent(key: string, e: Event, type: string): boolean;
        setEventListeners(eventListeners: L.LeafletEventHandlerFnMap): void;
        _getDefaultCRSLayers(): (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[];
        _getMMLCRSLayers(): (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[];
        setTileLayerByName(name: import("./map.defs").TileLayerName): void;
        getListForAvailableLayers(names: import("./map.defs").LayerNames[], condition: any, _tileLayers?: {
            [name: string]: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        }): {};
        setAvailableTileLayers(names: import("./map.defs").TileLayerName[], condition: boolean, _tileLayers?: {
            [name: string]: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        }): void;
        getDefaultTileLayerBlacklist(): import("./map.defs").TileLayerName[];
        getDefaultTileLayerWhitelist(): import("./map.defs").TileLayerName[];
        setAvailableTileLayerBlacklist(names: import("./map.defs").TileLayerName[]): void;
        setAvailableTileLayerWhitelist(names: import("./map.defs").TileLayerName[]): void;
        getAvailableOverlaysByNames(): {
            [name: string]: (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[];
        };
        getAvailableWorldTileLayers(): {
            [name: string]: (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[];
        };
        getAvailableFinnishTileLayers(): {
            [name: string]: (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[];
        };
        getAvailableLayersFor(_layers: any, availables: any): {
            [name: string]: (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[];
        };
        setTileLayer(layer: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>): void;
        getActiveLayers(options: import("./map.defs").InternalTileLayersOptions): {
            maastokartta?: import("./map.defs").TileLayerOptions;
            taustakartta?: import("./map.defs").TileLayerOptions;
            ortokuva?: import("./map.defs").TileLayerOptions;
            laser?: import("./map.defs").TileLayerOptions;
            openStreetMap?: import("./map.defs").TileLayerOptions;
            googleSatellite?: import("./map.defs").TileLayerOptions;
            afeGrid?: import("./map.defs").TileLayerOptions;
            cgrsGrid?: import("./map.defs").TileLayerOptions;
            atlasGrid?: import("./map.defs").TileLayerOptions;
            geobiologicalProvinces?: import("./map.defs").TileLayerOptions;
            geobiologicalProvinceBorders?: import("./map.defs").TileLayerOptions;
            municipalities?: import("./map.defs").TileLayerOptions;
            counties?: import("./map.defs").TileLayerOptions;
            ely?: import("./map.defs").TileLayerOptions;
            forestVegetationZones?: import("./map.defs").TileLayerOptions;
            mireVegetationZones?: import("./map.defs").TileLayerOptions;
            threatenedSpeciesEvaluationZones?: import("./map.defs").TileLayerOptions;
            biodiversityForestZones?: import("./map.defs").TileLayerOptions;
            ykjGrid?: import("./map.defs").TileLayerOptions;
            ykjGridLabels?: import("./map.defs").TileLayerOptions;
            kiinteistojaotus?: import("./map.defs").TileLayerOptions;
            kiinteistotunnukset?: import("./map.defs").TileLayerOptions;
            currentProtectedAreas?: import("./map.defs").TileLayerOptions;
            plannedProtectedAreas?: import("./map.defs").TileLayerOptions;
            flyingSquirrelPredictionModel?: import("./map.defs").TileLayerOptions;
            birdAtlasSocietyGridZones?: import("./map.defs").TileLayerOptions;
            barentsRegion?: import("./map.defs").TileLayerOptions;
        };
        _getInternalTileLayerOptions(options: TileLayersOptions): import("./map.defs").InternalTileLayersOptions;
        getTileLayers(): TileLayersOptions;
        setOverlays(overlays?: (L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>)[]): void;
        _setOverlaysByName(overlayNames: import("./map.defs").OverlayName[]): void;
        getOverlaysByName(): import("./map.defs").OverlayName[];
        setOverlaysByName(overlayNames?: import("./map.defs").OverlayName[]): void;
        getDefaultOverlaysBlacklist(): import("./map.defs").OverlayName[];
        getDefaultOverlaysWhitelist(): import("./map.defs").OverlayName[];
        setAvailableOverlaysBlacklist(overlayNames: import("./map.defs").OverlayName[]): void;
        setAvailableOverlaysWhitelist(overlayNames: import("./map.defs").OverlayName[]): void;
        setAvailableOverlays(overlayNames: import("./map.defs").OverlayName[], condition: any, _tileLayers?: {
            [name: string]: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>;
        }): void;
        getNormalizedZoom(zoom?: number, tileLayer?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>): number;
        getDenormalizedZoom(zoom?: number, tileLayer?: L.TileLayer | import("leaflet.nontiledlayer").WMS | L.LayerGroup<L.TileLayer | import("leaflet.nontiledlayer").WMS>): number;
        initZoom(zoom: any): void;
        setNormalizedZoom(zoom: any, options?: {
            animate: boolean;
        }): void;
        setCenter(center: L.LatLngExpression): void;
        destroy(): void;
        cleanDOM(): void;
        _addDomCleaner(fn: () => void): void;
        _removeDomCleaner(fn: () => void): void;
        _constructDictionary(): void;
        formatFeatureOut(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, layer?: import("./map.defs").DataItemLayer): G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
        formatFeatureIn(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, idx: number): G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
        cloneFeatures(features: G.Feature<G.Geometry, {
            [name: string]: any;
        }>[]): G.Feature<G.Geometry, {
            [name: string]: any;
        }>[];
        initializeDataItem(options: import("./map.defs").DataOptions, dataIdx?: number): void;
        _getAllData(): {
            group: L.FeatureGroup<any>;
        }[];
        zoomToData(options?: boolean | import("./map.defs").ZoomToDataOptions): void;
        getBoundsForZoomToDataOptions(options?: boolean | import("./map.defs").ZoomToDataOptions): L.LatLngBounds;
        getBoundsForDraw(): L.LatLngBounds;
        getBoundsForIdxs(...idxs: any[]): L.LatLngBounds;
        getBoundsForData(datas?: {
            group: L.FeatureGroup<any>;
        }[]): L.LatLngBounds;
        getBoundsForLayers(layers: L.Layer[]): L.LatLngBounds;
        _mapLayersToBoundableLayers(layers: L.Layer[]): L.Layer[];
        _initializeLayer(layer: import("./map.defs").DataItemLayer, idxTuple: import("./map.defs").IdxTuple): void;
        fitBounds(_bounds: L.LatLngBoundsExpression, options?: import("./map.defs").LajiMapFitBoundsOptions): void;
        setData(data: import("./map.defs").Data | import("./map.defs").Data[]): void;
        getData: () => import("./map.defs").Data[];
        addData(items: import("./map.defs").DataOptions[]): void;
        updateData(dataIdx: any, item: import("./map.defs").DataOptions): void;
        removeData(dataIdx: number): void;
        getDrawOptions(options: boolean | import("./map.defs").DataOptions): import("./map.defs").DataOptions;
        setDraw(options: import("./map.defs").DataOptions): void;
        getDraw(): import("./map.defs").Data;
        drawIsAllowed(): boolean;
        drawIsEditable(): boolean;
        _getEventsWithLayers(events: LajiMapEvent[]): (import("./map.defs").LajiMapDeleteEvent | import("./map.defs").LajiMapActivateEvent | {
            layer: import("./map.defs").DataItemLayer;
            type: "create";
            feature: G.Feature<G.Geometry, {
                [name: string]: any;
            }>;
        } | {
            layer: import("./map.defs").DataItemLayer;
            type: "insert";
            idx: number;
            feature: G.Feature<G.Geometry, {
                [name: string]: any;
            }>;
        } | {
            layers: {};
            type: "edit";
            features: {
                [idx: number]: G.Feature<G.Geometry, {
                    [name: string]: any;
                }>;
            };
        })[];
        wrapGeoJSONCoordinate([lng, lat]: G.Position): G.Position;
        _setOnChangeForItem(item: any, format?: import("./map.defs").OnChangeGeometryFormat, crs?: string): void;
        clearItemData(item: import("./map.defs").Data, commit?: boolean): void;
        clearDrawData(): void;
        _startDrawRemove(): void;
        _stopDrawRemove(): boolean;
        _finishDrawRemove(): void;
        _cancelDrawRemove(): boolean;
        _onDrawReverseHandler({ layer }: L.GeoJSONEvent): void;
        _startDrawReverse(): void;
        _stopDrawReverse(): void;
        _finishDrawReverse(): void;
        _cancelDrawReverse(): void;
        _createIcon(options?: L.PathOptions, icon?: (pathOptions: L.PathOptions, feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>) => L.Icon<L.IconOptions>, feature?: G.Feature<G.Geometry, {
            [name: string]: any;
        }>): L.Icon<L.IconOptions>;
        _getClusterIcon(data: import("./map.defs").Data): (cluster: L.MarkerCluster) => L.DivIcon;
        setActive(layer: import("./map.defs").DataItemLayer): void;
        _resetIds(dataIdx: number): void;
        _setIdForLayer(layer: import("./map.defs").DataItemLayer, idxTuple: import("./map.defs").IdxTuple): void;
        recluster(): void;
        _reclusterDrawData(): void;
        _reclusterData(): void;
        getClusterOptionsFor(item: import("./map.defs").Data): {
            maxClusterRadius?: number | ((zoom: number) => number);
            iconCreateFunction: ((cluster: L.MarkerCluster) => L.DivIcon) | ((cluster: L.MarkerCluster) => L.Icon<L.IconOptions> | L.DivIcon);
            clusterPane?: string;
            spiderfyOnEveryZoom?: boolean;
            spiderfyOnMaxZoom?: boolean;
            showCoverageOnHover?: boolean;
            zoomToBoundsOnClick?: boolean;
            singleMarkerMode?: boolean;
            disableClusteringAtZoom?: number;
            removeOutsideVisibleBounds?: boolean;
            animate?: boolean;
            animateAddingMarkers?: boolean;
            spiderfyShapePositions?: (count: number, centerPoint: L.Point) => L.Point[];
            spiderfyDistanceMultiplier?: number;
            spiderLegPolylineOptions?: L.PolylineOptions;
            chunkedLoading?: boolean;
            chunkDelay?: number;
            chunkInterval?: number;
            chunkProgress?: (processedMarkers: number, totalMarkers: number, elapsedTime: number) => void;
            polygonOptions?: L.PolylineOptions;
            pane?: string;
            attribution?: string;
        };
        _reclusterDataItem(item: import("./map.defs").Data): void;
        reorderData(): void;
        redrawDrawData(): void;
        redrawDataItem(idx: number): void;
        redrawData(): void;
        redraw(): void;
        _initializePopup(layer: import("./map.defs").DataItemLayer): void;
        _initializeTooltip(layer: import("./map.defs").DataItemLayer): void;
        _updateContextMenuForLayer(layer: import("./map.defs").DataItemLayer): void;
        setLocate(locate?: boolean | import("./map.defs").UserLocationOptions): void;
        createUserLocationLayer(latlng: L.LatLng, accuracy: number): {
            layerGroup: L.LayerGroup<any>;
            markerLayer: L.CircleMarker<any>;
            radiusLayer: L.Circle<any>;
        };
        _onLocationNotFound(e: L.ErrorEvent): void;
        getDrawLayerByIdx(idx: number): import("./map.defs").DataItemLayer;
        _getLayerByIdxTuple: (idxTuple: import("./map.defs").IdxTuple) => import("./map.defs").DataItemLayer;
        getLayerByIdxTuple(idxTuple: import("./map.defs").IdxTuple): import("./map.defs").DataItemLayer;
        _getLayerById(id: number): import("./map.defs").DataItemLayer;
        _getIdxTupleByLayer(layer: any): import("./map.defs").IdxTuple;
        _getDrawLayerById(id: any): import("./map.defs").DataItemLayer;
        _triggerEvent(e: any, handler: any): void;
        _decoratePolyline(layer: L.Polyline<G.MultiLineString | G.LineString, any>): void;
        _reversePolyline(layer: any): void;
        _getStartCircleStyle(lineLayer: any): L.PathOptions;
        _onAdd(dataIdx: number, layer: import("./map.defs").DataItemLayer, coordinateVerbatim?: string): void;
        _onEdit(dataIdx: any, data: any): void;
        _onDelete(dataIdx: any, deleteIds: any): void;
        _removeLayerFromItem(item: import("./map.defs").Data, layer: import("./map.defs").DataItemLayer): void;
        _onActiveChange(idxTuple: import("./map.defs").IdxTuple): void;
        focusToLayerByIdxs(idxTuple: import("./map.defs").IdxTuple): void;
        focusToDrawLayer(idx: number): void;
        _setEditable(layer: import("./map.defs").DataItemLayer): void;
        _clearEditable(): void;
        _commitEdit(): void;
        _interceptClick(): boolean;
        setLayerStyle(layer: L.FeatureGroup<any> | import("./map.defs").DataItemLayer, style: L.PathOptions): void;
        _featureToLayer(getFeatureStyle: (options: import("./map.defs").GetFeatureStyleOptions) => L.PathOptions, dataIdx?: number): (feature: any) => any;
        _getDefaultDrawDraftStyle(): L.PathOptions;
        _fillStyleWithGlobals<T extends L.PathOptions>(idxTuple: import("./map.defs").IdxTuple): T;
        _getStyleForType(idx: number | import("./map.defs").IdxTuple, overrideStyles?: L.PathOptions): L.PathOptions;
        _getStyleForLayer(layer: import("./map.defs").DataItemLayer, overrideStyles?: any): L.PathOptions;
        updateLayerStyle(layer: import("./map.defs").DataItemLayer): void;
        _getDefaultDrawStyle(): L.PathOptions;
        _getDefaultDrawClusterStyle(item: import("./map.defs").Data): L.PathOptions;
        _getDefaultDataStyle(item: import("./map.defs").Data): () => L.PathOptions;
        _getDefaultDataClusterStyle: (item: any) => () => {
            color: string;
            opacity: any;
        };
        _getDefaultDraftStyle(dataIdx: number): L.PathOptions;
        _updateDataLayerGroupStyle(idx: number): void;
        addTranslationHook(elemOrFn: HTMLElement | (() => void), translationKey?: string | (() => string), attr?: string): () => void;
        removeTranslationHook(hook: () => void): void;
        _getDrawOptionsForType<T_1 extends L.PathOptions>(featureType: DataItemType): T_1;
        abortDrawing(e?: Event | L.LeafletEvent): void;
        addDrawAbortListeners(): void;
        addFeatureToDraw(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>): void;
        addFeatureToData(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, dataIdx: number): void;
        getFeatureTypes(): DataItemType[];
        _showError(e: Error): void;
        showClosableElement(elem: HTMLElement, onClose: (e: Event) => void, blocker?: boolean, getContainer?: () => HTMLElement): void;
        _createTooltip(translationKey: string, error?: boolean): L.Draw.Tooltip;
        _disposeTooltip(): void;
        _showDialog(container: HTMLElement, onClose?: (e: Event) => void): void;
        setGoogleApiKey(googleApiKey: string): void;
        setGoogleSearchUrl(url: string): void;
        setGeocodingProvider(): void;
        geocode(query: string, additional?: any, zoom?: number, provider?: "mml" | "google"): void;
        setViewLocked(value: any): void;
        toggleFullscreen(): void;
    };
} & LM;
export {};
